*-- Author :    Hartwig Albrecht   05/10/95

      SUBROUTINE ALPYTH

************************************************************************
*
* Description: generate one PYTHIA event. Option for proper treatment
*					of meson-mixing exists. 
*-- Author :    Hartwig Albrecht   05/10/95
*
* Modified by Ivan Belyaev 12/05/98 to avoid  FRITIOF hanging for
* bbar or ccbar production
*   
* Called from: ALEVNT
*
* External references: LUND, PYTHIA
*
* Arguments: none
*
************************************************************************

      IMPLICIT NONE

#include "arte/ZDECAY.fi"
#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "arte/ZIOCOM.fi"
#include "jets/LUDAT3.fi"
#include "jets/LUJETS.fi"
#include "pyth/PYSUBS.fi"
#include "pyth/PYPARS.fi"

      INTEGER NSRC, I, IFRIT, LUCOMP
      CHARACTER*1 CTARG
      REAL PQQ(4) 

C     *
C     *     Fritiof does not like to work with initial 
C     *      beam energy less then ENERGYCUT
C     *     Let's introduce this artificial  energy cut-off.  (I.B.)
C     *
      REAL  ENERGYCUT , ENERGYREST 
      PARAMETER(ENERGYCUT = 5.0 )

#include "arte/Arte.ff"

C Jetset 7.4 and Fritiof: Needs Pythia initialization for every event.
C The routines ALSAVE and ALREST work only with jetset 7.3.
C Wanted: Someone who modifies Fritiof/ALSAVE/ALREST in a way that
C the time consuming PYINIT needs to be called only at the very beginning.

      CTARG = 'P'
      IF (TARGETMAT .EQ. 'N')  CTARG = 'N'
      IFRIT = 1
      IF (TARGETMAT .EQ. 'P' .OR. TARGETMAT .EQ. 'N')  IFRIT = 0

      CALL PYINIT ('FIXT', 'p+', 'n0', HERAEBEAM)

C loop over generated events until requested particle found

      DO 20 NSRC = 1, 1000

C PYTHIA !

C     If there is a neutral meson which mixes and should be treated 
C     differently from its anti-state, then switch of the decay
C     completely at this first stage.
C
C     This procedure can also be used for special treatment of any 
C     kind of particle, whenever duplicate special decays are to be
C     avoided.	

      IF(meson.ne.0) MDCY(LUCOMP(gKF(abs(meson))),1) = 0

        CALL PYEVNT

C search for the requested particle

        IF (LUNDRPART .EQ. 0)  GO TO 30
        DO 10 I = 1, N
          IF (K(I,2) .EQ. LUNDRPART)  GO TO 30
 10     CONTINUE

        IF (NSRC .EQ. 20)  CALL MEMESC (71345, 10, ILARGEST,
     +    'ALPYTH: more than 20 searches for the requested particle')

 20   CONTINUE

      CALL MEMESC (325476, 10, 100,
     +  'ALPYTH: more than 1000 searches for the requested particle')
      GO TO 90

 30   IF(meson.ne.0) call ALSPEC

C working space for pointers from LUJETS to MTRA; pointer < 0: discarded

      CALL MMBWRK (JTEMP, N, 2, 0)

C search for the primary quarks and its kin

      PQQ(1) = 0.
      PQQ(2) = 0.
      PQQ(3) = 0.
      PQQ(4) = 0.

      DO 40 I = 1, N

c remove empty and comment lines:

        IF (K(I,1) .EQ. 0)  THEN
          IQ(L_A(1,I,JTEMP)) = -1
        ELSE IF (K(I,1) .GE. 21 .AND. K(I,1) .LE. 30)  THEN
          IQ(L_A(1,I,JTEMP)) = -1

C Jpsi production

        ELSE IF (MSEL .EQ. 0)  THEN
          GO TO 40

C dont remove the primary quarks

        ELSE IF (ABS(K(I,2)) .NE. MSEL)  THEN

C remove all orphans and the daughters of removed particles

          IF (K(I,3) .EQ. 0 .OR. I_V(1,K(I,3),JTEMP) .LT. 0)  THEN
C but only if Fritiof will be called
            IF (IFRIT .NE. 0)  IQ(L_A(1,I,JTEMP)) = -1

C cluster/string from heavy quark

          ELSE IF (ABS (K(K(I,3),2)) .EQ. MSEL)  THEN
            IQ(L_A(1,I,JTEMP)) = 1

C granddaughter of heavy quark

          ELSE IF (I_V(1,K(I,3),JTEMP) .GT. 0)  THEN

C add 4-momentum of the heavy hadrons

            IF (MOD (ABS (K(I,2)), 1000) / 100 .EQ. MSEL .OR.
     +          MOD (ABS (K(I,2)), 10000) / 1000 .EQ. MSEL)  THEN
              PQQ(1) = PQQ(1) + P(I,1)
              PQQ(2) = PQQ(2) + P(I,2)
              PQQ(3) = PQQ(3) + P(I,3)
              PQQ(4) = PQQ(4) + P(I,4)

C other fragments of heavy quarks

            ELSE
              IQ(L_A(1,I,JTEMP)) = -1

            END IF
          END IF
        END IF

 40   CONTINUE

C fill Arte tables

      CALL ALMCIN
      CALL ALMTRA

C the rest of the event is now generated by Fritiof.
C Fritiof is not yet able to use the full 4-vector PQQ.
C Note that Fritiof is initialized every time.
C The initialization FRPYINI is called up to 4 times / event.
C Someone has to streamline this program.


      IF (IFRIT .EQ. 0)  GO TO 90

      CALL ALSAVE
      MSEL = 1
C     *
C     * Before invoking FRITIOF, check the fritiof energy.
C     * in b-bar or ccbar production, fritiof likes to hang up
C     * if the total energy less then ENERGYCUT
C     * Just skip FRITIOF for these events.
C     * Introduce a small energy nonconservation of about ENERGYCUT for 
C     * these quite rare events.
C     *                                  (I.B.)
      ENERGYREST = HERAEBEAM - PQQ(4)
  
      IF( ENERGYREST .LE. ENERGYCUT ) THEN 
       C78='Alpyth.F warning! FRITIOF Energy='
       CALL CH78AF( ENERGYREST , -3 )
       CALL CH78AC(' < ' ) 
       CALL CH78AF( ENERGYCUT , -3 ) 
       CALL ME78PC( 9237 , 300 ) 
c      * Empty LUND event record 
       N = 0 
      ELSE 
       CALL FREVENT ('FIXT', 'P', TARGETMAT, ENERGYREST )
      ENDIF
 
      CALL ALREST
      CALL ALFSTO

 90   END

************************************************************************
* End of ALPYTH
************************************************************************


