//                              -*- Mode: C++ -*-
//: ECStrategyFactory.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#include "care/ECStrategyFactory.hh"
#include "care/ESPeakPos.hh"
#include "care/ESCorCOGPos.hh"
#include "care/ESCTCValley.hh"
#include "care/ESPID04.hh"

/*
CLASS
  ECStrategyFactory


*/

///////////////////  static data definition  //////////////////////
// initial value for curren data
ECStrategyFactory::StratTypes ECStrategyFactory::currdata = 
                     {COG, Valley, PeakDis, Pid04};

// default values
ECStrategyFactory::StratTypes ECStrategyFactory::defValues =
                     {COG, Valley, PeakDis, Pid04};

/////////////////// member function definitions /////////////////////
// constructor and destructor
ECStrategyFactory::ECStrategyFactory(){}
ECStrategyFactory::~ECStrategyFactory(){}

 //  Set the defaults
void ECStrategyFactory::setDefaults(struct StratTypes & newdef){
  defValues = newdef;
}


 //--
  //   Get a brand new strategy for position
ESPosStrategy* ECStrategyFactory::PositionStrategy() const{
  return PositionStrategy(currdata.ptype);
}
//--
//   Get a new PositionStrategy of type pos
ESPosStrategy* ECStrategyFactory::PositionStrategy(PosTyp pos) const {
    ESPosStrategy* posstrat = 0;
    switch(pos){
    case Peak:
      posstrat =  new ESPeakPos();
      break;
    case COG:
      posstrat =  new ESCOGPos();
      break;
    case E_CorCOG:
      posstrat =  new ESCorCOGPos();
      break;
    default:
      posstrat =  new ESPeakPos();
      break;
    }
    return posstrat;
}

//--
//   Get a brand new strategy for Clus-To-Clus dist
ESCTCDisStrat* ECStrategyFactory::CTCStrategy() const {
  return CTCStrategy(currdata.ctctype);
}

//--
//   Get a new strategy for Clus-To-Clus dist of type ctcs
ESCTCDisStrat* ECStrategyFactory::CTCStrategy(ECStrategyFactory::ClToClDis 
					                  ctcs) 
  const {
  ESCTCDisStrat* disstrat = 0;
  switch(ctcs){
  case ECStrategyFactory::Valley:
     disstrat = new ESCTCValley();
     break;
  default:
    disstrat = new ESCTCValley();
    break;
  }
  return disstrat;
}

//--
//   Get a new strategy for PID analysis
ESPIDStrategy* ECStrategyFactory::PidStrategy() const{
  return  PidStrategy(currdata.pidtype);
}

//--
//   Get a new strategy for PID analysis
ESPIDStrategy* ECStrategyFactory::PidStrategy(PIDtypes pidt) const{
  ESPIDStrategy* pid = 0;
 switch(pidt){
  case ECStrategyFactory::Pid04:
     pid =  new ESPID04();
     break;
     // case ECStrategyFactory::Pid05:
     //     return new PID05();
     //     break;
     //case ECStrategyFactory::Pid1:
     //     return new PID1();
     //     break;
  default:
    pid =  new ESPID04();
    break;
  }
  return  pid;
}


