//                              -*- Mode: C++ -*-
//: CmdBase.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#ifndef CMDBASE_HH_
#define CMDBASE_HH_

#define USEKUIP  // to turn on the kuip routines

/**
Abstract base class CmdBase.
  Abstract base class to define an interface for the commands
  issued throu the Kuip interface or throu other ways (to be defined)
  The flag USEKUIP is used to select the KUIP interface. 


*/

class CmdBase {
public:
  ///-- Constructors
  CmdBase();
  //-- Destructor
  /// Destructor: currently void
  virtual ~CmdBase();
  
  //-- Behavioural operations
  ///   Reset stored data to default values
  virtual void defValues() = 0;
  //--
  ///   Execute stored command
  virtual void execute() = 0;

# ifdef USEKUIP
  //--
  ///   Read a command from external
  virtual void read() = 0;
  //--
  ///   Read and execute
  virtual void readAndExe();
# endif
  //--
  ///   Dump internal data
  virtual void dump() = 0;

protected:
# ifdef USEKUIP
  //--
  ///  Read a float
  float readFloat() const;
  //--
  ///  Read an integer
  int readInt() const;
# endif
  //-------------------- Data Part ---------------------------
  //--
  // No datum. The Class has the size of a function pointer.
};

#endif  //   CMDBASE_HH_
