//                              -*- Mode: C++ -*-
//: CmdCareDglev.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#ifndef CMDCAREDGLEV_HH_
#define CMDCAREDGLEV_HH_

#include "care/CmdBase.hh" 


enum Dglev_t {NODIGI, LOWDIGI, FULLDIGI};

/*
  Class to deal with the care/dglev kuip command.
  Set the digitization level:
  Level   value  Input  Output  Description
  NODIGI   (0)   MCAL   HITC    Standar digitalization for arte-01-05 and 
                                earlier arte versions. The Q entry in the HITC
                                bank contain an uncalibrated energy responce 
                                (in GeV)
  LOWDIGI  (1)    MCAL   HITC   The Q entry in the HITC bank contains a
                                integer number (in floating representation)
                                that is the "likely" responce of a readout ADC
                                channel to the (calibrated) signal observed.
  FULLDIGI (2)    MCAL   DCAL   The digitization output is exactly the Second
                                Level buffer of the calorimeter. The HITC bank
                                will be created and filled in the 
                                reconstruction routines.
*/

class CmdCareDglev : public CmdBase {
public:
  ///-- Constructors
  CmdCareDglev();
  //-- Destructor
  /// Destructor: currently void
  virtual ~CmdCareDglev();
  
  //-- Behavioural operations
  ///   Reset stored data to default values
  virtual void defValues();
  //--
  ///   Execute stored command
  virtual void execute();

# ifdef USEKUIP
  //--
  ///   Read a command from external
  virtual void read();
# endif
  //--
  ///   Dump internal data
  virtual void dump();
  //--
  ///   Static access member function
  static Dglev_t dglevel();

private:
  //   ------------------- Data Part -----------------------------
  // stored digitization level
  static  Dglev_t level;
};

#endif  //   CMDCAREDGLEV_HH_
