//                              -*- Mode: C++ -*-
//: EAMCTruthAnalyzer.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 26/5/97
// Last Update ........
//

#ifndef EAMCTRUTHANALYZER_HH_
#define EAMCTRUTHANALYZER_HH_

#include "care/EAClusterAnalyzer.hh"
#include "care/ECMCDeposit.hh"
#include "arte/Arte.hh"
#include "auto/TOPX.hh"
#include "auto/MTRA.hh"

#ifdef _OLD_NAMES_
#  include <vector.h>
#else
#  include <vector>
#endif

/**
class  EAMCTruthAnalyzer.
  Class to find the monte carlo particle that contributes the most to
  the energy released in a cluster.

USAGE
  Call the cluster function analyzeMe() with an instance of this class as
  a parameter; then ask for the maximum energy deposit:
  EXAMPLE
    <PRE>
    EMCluster  clust;
    // ... fill clust in some way
    
    EAMCTruthAnalyzer MCtruth;

    clust.analyzeMe(MCtruth);
    
    ECMCDeposit  dep = MCtruth.maxDeposit();
    </PRE>
  END
*/

class EAMCTruthAnalyzer : public EAClusterAnalyzer {
public:
  /// Constructor
  EAMCTruthAnalyzer();
  /// Destructor
  virtual ~EAMCTruthAnalyzer();

  /**@name Beheavioral methods */
  //@{ 
  //-- 
  /// EMClusterMember Analyzer
  virtual void doAnalysis(const EMClusterMember * clsMem);
  //--
  /// EMHitCell Analyzer
  virtual void doAnalysis(const EMHitCell * hitT);
  //--
  /// EMCluster Analyzer
  virtual void doAnalysis(const EMCluster * clus);
  //--
  /// EMClusterTree Analyzer
  virtual void doAnalysis(const EMClusterTree * clus);
  //--
  /// EMTrackOnEcal Analyzer
  virtual void doAnalysis(const EMTrackOnEcal * track);
  //--
  /// Reset informations
  void reset();
  //-- 
  ///  Sort table
  void sortDeposits();
  //@}
  
  //--
  ///  Return infos on the particle with the maximum energy deposited
  ECMCDeposit  maxDeposit();
  //--
  ///  Return the second deposit after the maximum
  ECMCDeposit  second();
  //--
  ///  Return deposit vector
  const vector<ECMCDeposit> & getVector() const { return depList;};

private:
 
  vector<ECMCDeposit>  depList;
};

#endif // EAMCTRUTHANALYZER_HH_
