//                              -*- Mode: C++ -*-
//:ECBuilder.hh
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 11/4/97
// Last Update ........
//
//
#ifndef ECBUILDER_HH_
#define ECBUILDER_HH_

#include "care/EMCluster.hh"
#include "care/EMTrackOnEcal.hh"
#include "care/ECHitCellArray.hh"
#include "care/ECCellID.hh"
#include "care/ELNearClusters.hh"

/**
class ECBuilder.
  Provide a collection of basic function for the building process
  of the hierarchical cluster tree. 

  */

class ECBuilder {
public:
  //--
  /// Constructor: it does nothing.
  inline ECBuilder();
  /**@name Beheavioral methods */
  //@{ 
  //--
  /** Prepare internal data for a loop on all cell. 
    As parameters it takes the list of the HITC array to use
    for clustering and the TopCluster list in which it writes the tree.
    It does the sorting of HitCellArray. */
  void startClustering(ECHitCellArray & signalCells, 
		       EMClusterTree& topClusterList);
  //--
  /** Clusterize the current cell.
    Insert it in an old cluster or in a new one.  */
  void doClusterize();  // Consider next operation on cluster
  //--
  ///  Consider next cell.
  inline void next();
  //--
  /// Test if there are other cells to clusterize
  inline int more() const;
  //--
  /// return the Cell in which the point on ECAL is
  //  EMHitCell* whichClust(float x, float y) const;

  //--
  /// Store parameters of a track
  //void storeTrack(TrackInfo & trackInfo);

  //--
  /// Perform the Track-Cluster Match
  void trackClusterMatch();
 
  //--
  /// start a new cluster with a given cell. The passed cell is actually MOVED
  EMCluster * newCluster(EMClusterMember * clusmem) const;

  //--
  /// Try to recover unmatched tracks
  void recoverUnmatchedTracks();

  //--
  /// fill a fraction of cell and store it in the tree
  void cellFraction(EMHitCell * cell, EMCluster* cluster, Energy_t fraction);

  //@}

  //--
  /**  return a pointer to a cluster in which the current cell could be
    inserted in substitution to the cluster found by extrapolate. */
  EMCluster*  findOtherCluster(EMTrackOnEcal* track) const;
 
  /**
    return a pointer to a cluster in which the current cell could be
    inserted. */
  EMCluster*  findACluster(EMTrackOnEcal* track, CellIdx_t cell) const;

private:
 
  /// Ordering function
  inline static int comp_cluster_pointers(EMClusterMember *p1,
					  EMClusterMember *p2);
  //--
  ///  HitCellArray Pointer: it is defined by the constructor
  ECHitCellArray * pHitCells;
  ///  TopCluster Pointer: it is defined by the constructor
  EMClusterTree *      pTopClusterList;
  ///  Iterator on HitCellArray:
  vector<int>::const_iterator currHitCell;
  /// Maximum number of possible NNC
  enum {MaxNNC = 13 };  // high number: to take care of all possibilities
  ///  array with the pointers to the nearest neibouring clusters
  ELNearClusters nearClus;

  /// current cell index
  CellIdx_t currIdx;
 
};

#include "care/ECBuilder.icc"
#endif // ECBUILDER_HH_
