//                              -*- Mode: C++ -*-
//:ECDcalIO.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 9/4/97
// Last Update ........
//
#ifndef ECDCALIO_HH_
#define ECDCALIO_HH_
#include "care/ECALtypes.hh"
#include "auto/DCAL.hh"
typedef unsigned short unshort;

/**
class  ECDcalIO.

It provides function to operate on the stream of bytes produced by
the DAQ system. Actually it has the possibility to write a simulation
of that stream, so that it can be used in the MC simulations.

*/

class ECDcalIO {
public:
  // Constructors
  //--
  /// Default constructor
  ECDcalIO();
  //
  //--
  ///  Start operations
  void start(EcalEvent_t* event);
  //--
  ///  Start operations
  void start();
  //
  //-- 
  ///  Write to DCAL
  ECDcalIO & operator << (unsigned short value);
  //-- 
  ///  Read from DCAL
  inline ECDcalIO & operator >> (unsigned short & value);
  //--
  ///  Test if pointers are still valid
  inline int valid() const;

private:
  //--
  /// Pointer to dcal
  ArtePointer<DCAL> dcal;
  //--
  /// pointer to the half word
  int  hfwd;
  //--
  /// Number of dcal entries
  unsigned nrows;
  // -- 
  /// 
  union  DCALEntry {
    char  bits[4];
    unsigned short hfw[2];
  } entry;
  //--
  /// Number of current ushort
  unsigned current;
};

#include "care/ECDcalIO.icc"
#endif // ECDCALIO_HH_
