//                              -*- Mode: C++ -*-
//:ECMCcalib.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 27/5/97
// Last Update ........
//
#ifndef ECMCCALIB_HH_
#define ECMCCALIB_HH_
#include "care/care.hh"
#include "arte/Arte.hh"
#include "auto/HITC.hh"
#ifdef MCARLO
#include "auto/MVEX.hh"
#include "auto/MCME.hh"
#include "auto/MTMT.hh"
#endif
#include "care/ECALtypes.hh"
#include "care/ECCellID.hh"
/**
class  ECMCcalib.
  Store calibration constant for MC hits.
  The object returns the calibration constant for a given HITC, cellID or
  number of cell divisions.
  It is assumed that the only meaningfull cell divisions are 5, 2, 1,
  respectively for the inner, middle and outer part of the calorimeter.
*/

class ECMCcalib {
public:
  // Constructors
  //--
  /// Default constructor.
  inline ECMCcalib();
  //--
  ///  calibrate MC with a HITC pointer.
  inline Energy_t operator[](ArtePointer<HITC> hitp) const;
  //--
  ///  return calibration constant for cell with the given identifier 
  inline Energy_t operator[](ECCellID cellID) const;
  //--
  ///  return calibration constant for cell with ndiv 
  inline Energy_t operator[](int ndiv) const;

  //--
  ///   Set the calibration parameters.
  static void setCalibration(Energy_t innerCal, Energy_t middleCal,
                             Energy_t outerCal);
  //--
  ///   Printout Information for the current calibration data stored
  static void dumpCalib();

private:
  //--
  ///  Calibration constant to consider
  static Energy_t  calibEnergy[6];
};

#include "care/ECMCcalib.icc"
#endif // ECMCCALIB_HH_
