//                              -*- Mode: C++ -*-
//: EDClusTraHypotModifier.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 2/5/97
// Last Update ........
//

#ifndef EDCLUSTRAHYPOTMOD_HH_
#define EDCLUSTRAHYPOTMOD_HH_

#include"care/EDClusterModifier.hh"
#include "ECClHyp.hh"
class ECBuilder;

/**
class  EDClusTraHypotModifier.
  Class for finding the number of hypothetical particles that gave origin to
  a cluster. Clusters are labelled as charged or neutals

  The algorithm choseen make use of CutOff level to define the distance of
  a track from a cluster.

*/

class EDClusTraHypotModifier : public EDClusterModifier {
public:
  //-- Constructors
  /// Only default constructor is provided
  EDClusTraHypotModifier(ECBuilder* pbuilder);
  /// The virtual default destructor
  virtual ~EDClusTraHypotModifier();

  /**@name Beheavioral methods */
  //@{ 
  ///  Base function to be called with the TopCluster List
  void analyzeTopList(EMClusterTree & topList);
  ///  Base function to be called with the TopCluster List
  void doModify(EMClusterTree * tree);
  //-- 
  ///  EMClusterMember Analyzer
  virtual void doModify(EMClusterMember * clsMem);
  //--
  ///  EMHitCell Analyzer
  virtual void doModify(EMHitCell * hitT) ;
  //--
  ///  EMCluster Analyzer
  virtual void doModify(EMCluster * clus);
  
  //--
  ///  EMTrackOnEcal Analyzer
  virtual void doModify(EMTrackOnEcal * trackInT);
  //@}

private:
  ///-- struct for memorizing number of hypotheses
  ECClHyp hyp;
  /// the builder pointer
  ECBuilder* builder;
};

#endif // EDCLUSTRAHYPOTMOD_HH_
