//                              -*- Mode: C++ -*-
//:ELNearClusterFrac.hh
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 27/11/97
// Last Update ........
//
//

#ifndef ELNEARCLUSTERFRAC_HH_
#define ELNEARCLUSTERFRAC_HH_

#include "care/ELNearClusters.hh"
class ELNearHitCells;
class ELNearHitCellFrac;
class EMClusterTree;

/**
class ELNearClusterFrac.
  Get the list of clusters nearby a given cell
  and provide access to the list
*/

class ELNearClusterFrac : public ELNearClusters {
public:
  //--
  /// default constructor
  ELNearClusterFrac();
  //--
  /// constructor with an already initialized ELNearHitCellFrac object
  ELNearClusterFrac(ELNearHitCellFrac & nearT);
  //--
  ///  destructor
  virtual ~ELNearClusterFrac();
  //--

  /**@name Beheavioral methods */
  //@{ 
  //--
  ///  Evaluate the neighbouring clusters of a given set of cells
  void of(ELNearHitCellFrac & nearT);
  //--
  ///  Evaluate the neighbouring cluster of a given cell
  void of(const EMHitCell* cell, const EMClusterTree & tree);
  //--
  ///  Evaluate neighbouring clusters of a given cell
  void of(CellIdx_t cell, const EMClusterTree & tree);
  //--
  ///  Sort in energy
  void sortClusters();
  //--
  ///  Minimize the internal near cluster array
  void minimize();
  //@}
  
  /**@name Iterator methods */
  //@{ 
  //--
  //  get the fraction
  inline Energy_t value() const { return fractions[curr];}
  //@}

protected:
  //--
  ///  Insert the current cluster if it is not already present
  void insert(EMCluster* clus, Energy_t fraction);
  //--
  /**  Test if there are more than one occurrence of a given cluster.
    If it is so put to 0 all occurreces after the first. */
  void oneOccurrence();
  //--
  ///  Reduce the array removing null pointers
  void reduce();
  //--
  ///  Array containing the cells
  //EMCluster*  neibClusters[20];

private:
  //--
  ///  Array containing the fraction of the cells
  Energy_t fractions[20];
};

#endif // ELNEARCLUSTERFRAC_HH_
