//                              -*- Mode: C++ -*-
//: ESCOGPos.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 14/4/97
// Last Update ........
//

#ifndef ESCOGPOS_HH_
#define ESCOGPOS_HH_

#include "care/ECALtypes.hh"
#include "care/ESPosStrategy.hh"

/**
class ESCOGPos.
  Concrete class for cluster analysis: it evaluates total energy and
  the center of gravity of a cluster
*/

class ESCOGPos : public ESPosStrategy {
public:
  /// Constructor
  ESCOGPos();
  /// Destructor: currently void but needs to be virtual
  virtual ~ESCOGPos();
 
  /**@name Beheavioral methods */
  //@{ 
  ///   Evaluate the position for a cluster.
  virtual void evaluatePos(const EMCluster* clus);

  /// EMClusterMember Analyzer
  virtual void doAnalysis(const EMClusterMember * clsMem);
  //--
  /// EMHitCell Analyzer
  virtual void doAnalysis(const EMHitCell * hitT);
  //--
  /// Do a Cluster analysis
  virtual void doAnalysis(const EMCluster* clus);
  //--
  /// Do a ClusterTree analysis
  virtual void doAnalysis(const EMClusterTree* clus);
  //--
  /// EMTrackOnEcal analysis
  virtual void doAnalysis(const EMTrackOnEcal * Track);
  //@}
 

  struct COGdata{
    int  ndata; // number of data
    Energy_t energySum; // partial energy sum
    Energy_t energyErr; // partial sum of energy errors
    Coord_t  xSum;   // sum of all x coord
    Coord_t  ySum;   // sum of all y coord
    Coord_t  xxSum;  // sum of all x*x coord
    Coord_t  xySum;  // sum of all x*y coord
    Coord_t  yySum;  // sum of all y*y coord
    COGdata() : ndata(0), energySum(0.0), energyErr(0.0),
      xSum(0.0), ySum(0.0),
      xxSum(0.0), xySum(0.0), yySum(0.0) {};
  };
  
private:
  COGdata currClust;
  // the analyzer for the peak
  static ESPosStrategy* peakSt;
};

#endif // ESCOGPOS_HH_
