//                              -*- Mode: C++ -*-
//: dgcal1.C
//
// Author: Mauro Villa   
// ECAL group
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 21/8/97
// Last Update ........
//
// Digitalization routine for level 1
//    Level  Input  Output  Description
//      1    MCAL   HITC    The Q entry in the HITC bank contains a
//                          integer number (in floating representation) that 
//                          is the "likely" responce of a readout ADC channel 
//                          to the (calibrated) signal observed.

#include "arte/ArteMessage.hh"
#include "care/ECALtypes.hh"
#include "care/ECMCcalib.hh"
#include "care/ECCellID.hh"
#include "care/dgcal.hh"
#include "auto/HITC.hh"
#include "auto/MVEX.hh"
#include "auto/MCME.hh"
#include "auto/MTMT.hh"



void dgcal1_(int *isl){

  ECMCcalib  mcCalib; // Calibrator object
  
  // Start calling the standard digitalization for arte-01-05  
  // MIMP -> HITC (Q=energy)
  dgcal0_(isl);

  // Now loop on all the HITC bank to calibrate the energy responce
  // and to convert the energy in a ADC channel value
  Energy_t  ener;
  int       adcCh;
  CellIdx_t  cellidx;
  if(ArtePointer<HITC>::getLink()()==0) return;
  for(ArteTableIterator<HITC> itHitc(1); itHitc.valid(); itHitc.next()){
    cellidx = itHitc->gcce; // get the index in the cell bank
    int ncl = ECCellID(cellidx->ceid).modulePointer()->ncl;
    ener = itHitc->q * mcCalib[ncl];      // calibrated energy
    adcCh = dgcalech(cellidx.index(), ener);         // digitalized energy
    itHitc->q = (float) adcCh;               // digi stored back in HITC
    itHitc->adc = itHitc->q;                 // store the float ADC value
  }
}





 
