//                              -*- Mode: C++ -*-
//:dgcal_enecalc.C
// 
//:description - ENERGY DEPOSITION CALCULATION IN EACH CELL
//
// Author: Benedetto Giacobbe   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: giacobbe@bo.infn.it
//
// Created 26/7/97
// Last Update ........
//

#include "arte/Arte.hh"
#include "auto/GEPC.hh"
#include "auto/MCAL.hh"
#include "auto/MCAX.hh"
#include "auto/GCAL.hh"
#include "auto/GCAX.hh"
#include "auto/GMAX.hh"
#include "auto/GSLX.hh"
#include "auto/HITC.hh"
#include "auto/MVEX.hh"
#include "auto/MCME.hh"
#include "auto/MTMT.hh"
#include "care/dgcal.hh"

extern "C"
{
  float dgcal_enecalc(int ipl, int ix, int iy){
    
    ArtePointer<GEPC> gepc(ipl);
    int nmim = 0;
    float ener = 0.;
    if(gepc->mim1 != 0) nmim = gepc->miml - gepc->mim1 + 1; //num MCAL in IPL
    if(nmim > 0){
      ArtePointer<GCAL> gcal(ipl); 	
      int ndim = gcal->ncl;
      for(int l = 0; l < nmim; l++){
	int il = gepc->mim1 + l;
	ArtePointer<MCAL> mcal(il);
	if((mcal->gcal-ipl*100) == ndim*ix+iy){
	  ener += mcal->de;
	}
      }
    } else {
      ener = 0.;
    }
    return ener;
  }  
}
