//                              -*- Mode: C++ -*-
//:dgcalech.C
// 
//:description - ENERGY -> ADC CHANNEL CONVERSION
//
// Author: Benedetto Giacobbe   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: giacobbe@bo.infn.it
//
// Created 26/7/97
// Last Update ........
//

#include <stdlib.h>
#include <math.h>
#include "arte/Arte.hh"
#include "care/dgcal.hh"
#include "auto/ECA.hh"
#include "care/ECHF_status.hh"

extern "C" void  granor_(float * y1,float *y2);



int dgcalech(int idcell, float mc_en){
  
  float y1 = 0.0, y2 = 0.0;
    
  // if channel is on & low noisy & calibration was ok
  
  ArtePointer<ECHS> echs(idcell);      
  ArtePointer<ECHF> echf(idcell);      
  if((echf->stat & Echfs::usable) == Echfs::usable){
    
    /* ENERGY -> ADC CONVERSION */
    
    // q_mips = DAC channel (proportional to charge) relative to 
    // mips (0.3 GeV)
    
    //      float q_mips = echs->pcal[0];
    //      float q_mips_red = echs->mip/4095.;
    //      float q_mips_redpow = q_mips_red;
    //      for(int i=1;i<6;i++){
    //	q_mips += echs->pcal[i]*q_mips_redpow;
    //	q_mips_redpow *= q_mips_red; 
    //      }
      
    // DAC channels (proportional to charge) relative to energy 
    // e_ch = mc_en : q_ch = e_ch / 0.3 (GeV) * q_mips
    
    //      float q_ch = (mc_en) / 0.3 * q_mips;
    float q_ch = (mc_en) / echs->vmip;
    
    // ADC channel relative to DAC channel (proportional to charge) q_ch
    //  To change as soon as possible
    float ch = echs->pcal[0]+echs->pcal[1]*q_ch;
    
    /*
      float ch = echs->ecal[0]+echs->ecal[1]*(1.-exp(-(q_ch/echs->ecal[2])))
      +echs->ecal[3]*(1.-exp(-(q_ch/echs->ecal[4])**2))+echs->ecal[5]*q_ch;
      */

    ch *= 4095.;
    
    // alteration of channel with error of calibration 
      
    granor_(&y1,&y2);	
    y1 *= echf->wid;
    ch += y1;
    
    // sum of alterated pedestal 
    
    granor_(&y1,&y2);	      
    y1 *= echf->wid;
    y1 += echf->pedf;
    ch += y1;
    int ich = (int)(ch);

    /* **************************************************** */      
    /* CONVERSION ADC - > ENERGY for check : temporary call */ 
    /*      float en_rec = dgcalche(idcell, ich);           */
    /* **************************************************** */
    
    // cout<<"Energy: "<<mc_en<<"  Q: "<<q_ch<<"  channel: "<<ich<<endl;
    return ich;
  } else {
    return 0;
  }                                          /* if(echs->stat      */   
}                                            /* void digital       */




