//                              -*- Mode: C++ -*-
//: dgcalinit.C
//
// Author: Benedetto Giacobbe & Mauro Villa
// ECAL group
// HERA-B experiment, Desy
// e-mail: giacobbe@bo.infn.it, villa@bo.infn.it
//
// Created 21/7/97
// Last Update ........
//
// Initialization of the Digitalization routine 


#include "arte/Arte.hh"
#include "auto/ECA.hh"
#include "auto/GCCE.hh"
#include "care/ECCellID.hh"
#include "care/ECHitCellArray.hh"
//#include "mizzi/htable.h"
#include "care/dgcal.hh"
#include "arte/ArteMessage.hh"
#include "care/dgcalinit.hh"
#include "care/ELLUTsManager.hh"
#include "care/ELCellGrid.hh"
#include "care/CmdCareECBoards.hh"
#include "care/CmdCareCalerr.hh"
#include "auto/HITC.hh"
#include "care/ECHF_status.hh"
#include <algorithm>

#define NUMCHANN 32

typedef unsigned short ushort;


extern "C"
{
  void dgcalinit_(){
    int ndim=0;
    ECCellID  cellid(0);
    float minEnergy[6];
    ECHitCellArray::getMinims(minEnergy[5],minEnergy[2],minEnergy[1]);
    //    arteMsg<<" Minimum energies :"<<minEnergy[5]<<"  "<<minEnergy[2]
    //	<<"  "<<minEnergy[1];
    // arteMsg.allp();
    // Get the overall calibration precision
    CmdCareCalerr ccalerr;
    float calerr = ccalerr.precfactor();


    /* Write and Read calibration constants */
    if( ArtePointer<ECBO>::getLink()()==0){
      // if the calibration tables are not present then
      // read them!

      // Start booking the ECHS and ECHF banks with the default settings 
      // (some settings are missing)

      ECBO board = {0,0,0,0,
		    {-1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
		     -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1} };
      ECHS channelbases = { 50., // info pedestal (peds)
			   {0., 1., 0., 0., 0., 0.}, // 6 calibration constants
			   {0., 1., 0., 0., 0., 0.}, // 6 calibration constants
			   0.1, .10, // energy thresholds
			   10., 2., 4095., // MIP data
			   -1, -1, -1}; // FED channel data
      ECHS channelInfos = channelbases;
      ECHF channelbasef = { Echfs::usable | Echfs::readout | 
			    Echfs::pretrig | Echfs::usedcal | 
			    Echfs::avalcal,     // status of the channel
			    50.,2.,             // info pedestal (pedf, wid)
			    1.0};               // Q value 
      ECHF channelInfof = channelbasef;

      for(ArteTableIterator<GCCE> gcce(1); gcce.valid(); gcce.next()){
	ArtePointer<ECHF>::append(channelInfof);  // ECHF booked
	ArtePointer<ECHS>::append(channelInfos);  // ECHS booked
      }
      
      // Read from external file the board types and placements

      vector<BoardData> boards;
      vector<EcalDet> calos;
      CmdCareECBoards boardfile;
      ECALBoard(boardfile.boardfile(), boards, calos);
      char* boardtype = new char[boards.size()+1];
      for(unsigned int ibc=0; ibc<boards.size(); ibc++){
	boardtype[ibc] = boards[ibc].type;
      }

      // Book the ECBO bank and connect cells
      
      ELLUTsManager LM;
      boardtype[boards.size()] = 0;
      // loop on calorimeter types
      for(vector<EcalDet>::iterator detec = calos.begin();
	  detec!=calos.end(); detec++){
	// now loop on all datas:
	EcalDet * calr = &*detec;
	int sx = (calr->xmax-calr->xmin+1)/calr->mstepx;
	int sy = (calr->ymax-calr->ymin+1)/calr->mstepy;
	for(int ix=0; ix<sx; ix++){
	  for(int iy=0; iy<sy; iy++){
	    int j=(sy-1-iy)*sx+ix;
	    char bt = calr->data[j];
	    char * type = find(&boardtype[0], &boardtype[boards.size()],
			       bt);
	    if( *type==bt){
	      // in the current position there is 
	      // a board of type nboard
	      int nboard = type-&boardtype[0];

	      ArtePointer<ECBO>::append(board);
	      ArtePointer<ECBO> ecbo(ArtePointer<ECBO>::nrow());
	      ecbo->fdid = 0x0EC00+ ecbo.index();    // Front End Identifier
	      ecbo->numb = ecbo.index()-1;           // number of board
	      ecbo->stat = 1+boards[nboard].trigflag;  // board used 
	      //
	      // Now loop on modules covered by this board 
	      //
	      int numchan = 0;
	      for( int mdcx=0; mdcx<calr->mstepx; mdcx++){
		for( int mdcy=0; mdcy<calr->mstepy; mdcy++){
		  //
		  // Find the module in position (imx, imy)
		  // 
		  int imx = calr->xmin+mdcx+ix*calr->mstepx;
		  int imy = calr->ymin+mdcy+iy*calr->mstepy;
		  CellIdx_t cell = 0;
		  if( imx>=0 && imx<56 && imy>=0 && imy<42 )
		    cell = LM.cellsGrid().lookUp(imx,imy);
		  if(cell!=0){ 
		    // cell exists. Is it in the right calorimeter?
		    ECCellID cellid(cell->ceid);
		    int idim = cellid.modulePointer()->ncl;
		    if( (idim==5 && calr->ecaltype==3) ||
			(idim==2 && calr->ecaltype==2) ||
			(idim==1 && calr->ecaltype==1) ){
		      // OK! it is in the right calorimeter
		      for(int mx=0; mx<idim; mx++){
			for(int my=0; my<idim; my++,
			      cell=cell.index()+1, numchan++){
			  int chnum = (mdcy*idim+my)*calr->mstepx*idim+
			    mdcx*idim+mx;
			  //cout<<" Channel number: "<<chnum;
			  ecbo->chan[chnum] = cell.index();
			  ArtePointer<ECHS> echs(cell.index());
			  ArtePointer<ECHF> echf(cell.index());
			  echs->fdid = ecbo->fdid; // copy fed id on chan info
			  echs->ecbo = ecbo.index(); // ECBO data on chan info
			  echs->chan = chnum;      // chan pos on BOARD
			  echs->mip = boards[nboard].mipChan;
			  echs->peds = boards[nboard].pedChan;
			  echf->pedf = echs->peds;
			  echf->wid = boards[nboard].sigmaped;
			  float q_mips = echs->pcal[0];
			  float mipch_red = echs->mip/4095.;
			  float mipch_redpow = mipch_red;
			  for(int i=1;i<6;i++){
			    q_mips += echs->pcal[i]*mipch_redpow;
			    mipch_redpow *= mipch_red;
			  }
			  echf->q = 0.3/q_mips;
			  echs->vmip = echf->q;
			  // Error on channel (squared)
			  // ecal[0] = noise error given by the sigma pedestal
			  echs->ecal[0] = 0.3*boards[nboard].sigmaped/
			    boards[nboard].mipChan;
			  echs->ecal[0] *= echs->ecal[0];
			  // ecal[1] = Sampling term squared, 
			  //           given by the value read by ECALboard
			  echs->ecal[1] = 
			    boards[nboard].ersampl*boards[nboard].ersampl;
			  // ecal[2] = constant term squared, 
			  //           given by the value read by ECALboard+
			  //           the calibration error
			  echs->ecal[2] =   
			    boards[nboard].erconst*boards[nboard].erconst+
			    calerr*calerr;
			}
		      }
		    }
		  }
		}
	      }
	      if( numchan==0)
		ecbo->stat = -1;  // board not used
	    }
	  }
	}
      }
      
      delete [] boardtype;


      //   find the energy thresholds
      //
      float ener, wid;
      ArtePointer<ECHS> echs;
      ArtePointer<ECHF> echf;
      for(ArteTableIterator<GCCE> gcce2; gcce2.valid(); gcce2.next()){
	cellid(gcce2->ceid);
	ndim = cellid.modulePointer()->ncl;
	ener = minEnergy[ndim];
	echs = gcce2.index();
	echf = echs;
	wid = echf->wid;
	echs->thr1 = echs->mip+wid*10;
	echs->thr2 =  echs->mip+wid*20;
	echs->vmip = echf->q*ccalerr.factor();        
      }
    }
  }
}
