************************************************************************
*
* DGOTR
* written by Siegmund Nowak 08/93
*
* This routines translates GEANT output hits into ZEBRA bank structures
* JHITB and JDIGB for Honeycomb chambers
*
*       called from DGEVNT
*
* Arguments  kind         description
*-----------------------------------------------------------------------
*  IPL        in          Detector number in BANK JGEDE
*
************************************************************************

      SUBROUTINE DGOTR(IPL)
C-----------------------------

      IMPLICIT NONE
#include "arte/Arte.fi"
#include "auto/GEO.fi"
#include "auto/MC.fi"
#include "auto/DIGB.fi"
#include "auto/HITB.fi"

      INTEGER NHITDE,NMIM,NDIGS,NHITS,NN,NN1
      INTEGER I,IH,N,NF,NL,ISN,IPL,JVH,NTUBE,IFMIMP,ILOOK(4,2),KORDER
      REAL WIRED,COSX,SINX,X
      REAL XT1,XTV,XTH,XFW,XWIR,XTREF,ZWIR,DELTAZ,RNDM,TGTRX,CSTRX,DD,F
      LOGICAL LBIGR

      INTEGER MXHIT,IWIR,IND0,IND1,IXMIMP,IDIGX
      REAL XMIMP,DIGX,HTOR,DGOR
      PARAMETER (MXHIT=1000)
      COMMON /WRKSP1/ IWIR(1000,4),XMIMP(6,MXHIT),
     +     DIGX(7,MXHIT),HTOR(MXHIT),DGOR(MXHIT),IND0(MXHIT),IND1(MXHIT)
      DIMENSION IXMIMP(6,MXHIT),IDIGX(7,MXHIT)
      EQUIVALENCE (XMIMP,IXMIMP),(DIGX,IDIGX)

      DATA ILOOK /1,3,2,4,   3,1,4,2/

#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/MC.ff"
#include "auto/DIGB.ff"
#include "auto/HITB.ff"

c     calculate number of mimps
      NMIM = 0
      IFMIMP  = IGEPT_MIM1(IPL)
      IF(IFMIMP.NE.0) NMIM  = IGEPT_MIML(IPL) - IFMIMP + 1
      IF(NMIM.LE.0) GO TO 998
      IF(NMIM.GT.MXHIT) THEN
         C78 = 'DGOTR: NMIM > MAXHIT for IPL '
         CALL CH78AI(NMIM)
         CALL CH78AC(' for ')
         CALL CH78AI(IPL)
         CALL ME78WC(18282804)
         NMIM = MXHIT
      ENDIF

c     define some often used quantitites
      WIRED = FGEDE_PIT(IPL)     ! wire distance
      NTUBE = IGEDE_NSL(IPL)     ! number of sublayers within detector
      COSX =  FGEDE_COS(IPL)     ! cosine of the wire angle
      SINX =  FGEDE_SIN(IPL)     ! sine of the wire angle

c     decide how to sort (always from out- to inside)
      LBIGR = .FALSE.
      IF(ABS(FGEDE_XMAX(IPL)).LE.ABS(FGEDE_XMIN(IPL))) LBIGR = .TRUE.

c     if wire 1 in layer1 has smaller x then wire 1 in layer2: KOrder = 1
c     if wire 1 in layer1 has  larger x then wire 1 in layer2: KOrder = 2
      IF (FGEDE_XFW(1,IPL).lt.FGEDE_XFW(2,IPL)) THEN
         KOrder = 1
      ELSE
         KOrder = 2
      END IF
c
C*    get all HITS in detector IPL from GEANT data structure MIMP

C*    the first loop over the hits -  transform coordinates in front
C     and backside of the sensitive volumen and prepare sorting

      DO 10 I=1,NMIM
      IH = I+IFMIMP-1
      XT1 = COSX*FMIMP_X(IH) - SINX*FMIMP_Y(IH)
      XMIMP(1,I) = XT1
      XMIMP(2,I) = COSX*FMIMP_TX(IH)-SINX*FMIMP_TY(IH)
      XMIMP(3,I) = FMIMP_Z(IH)
      IXMIMP(4,I) = OMIMP_MTRA(IH)
      IXMIMP(5,I) = IH
      HTOR(I) = XT1
   10 CONTINUE

c     sort the hits matrix due to increasing x
      IF(LBIGR) THEN
         CALL SORTZV(HTOR,IND0,NMIM,1,0,0)
      ELSE
         CALL SORTZV(HTOR,IND0,NMIM,1,1,0)
      ENDIF

C*  the second loop over the hits fills the data structures JHIT and JMHIT
      CALL VZERO(IWIR,4000)
      NHITDE = 0

      DO 20 IH=1,NMIM
      I = IND0(IH)
      TGTRX = XMIMP(2,I)
      XTV = XMIMP(1,I)
      XTH = XTV + TGTRX * (FGEDE_ZMAX(IPL)-FGEDE_ZMIN(IPL))
      CSTRX = 0.
      IF(ABS(TGTRX).LT.1.E+10) CSTRX = 1./SQRT(1.+ TGTRX**2)

      NF = INT((XTV-(FGEDE_XFW(1,IPL)- 0.33333*WIRED))/WIRED) + 1
      NL = INT((XTH-(FGEDE_XFW(NTUBE,IPL) - 0.33333*WIRED))/WIRED) + 1
      IF(NF.LE.0.OR.NF.GT.IGEDE_NWI(NTUBE,IPL)) THEN
         C78 = 'DGOTR: NF outside'
         CALL ME78MC(46783,0,ILARGEST)
      ENDIF
      IF(NL.LE.0.OR.NL.GT.IGEDE_NWI(NTUBE,IPL)) THEN
         C78 = 'DGOTR NL outside'
         CALL ME78MC(748208,0,ILARGEST)
      ENDIF
      IF(NF.GT.NL) THEN
         ISN=-1
         NF = MIN0(NF+1,IGEDE_NWI(NTUBE,IPL))
         NL = MAX0(1,NL-1)
      ELSE
         ISN = 1
         NF = MAX0(1,NF-1)
         NL = MIN0(NL+1,IGEDE_NWI(NTUBE,IPL))
      ENDIF
      IF(NF.GT.500.OR.NL.GT.500) THEN
         C78 =  'DGOTR: NF or NL .GT.500 '
         CALL ME78ER
      ENDIF

C*    loop over tubes hitten by these particle
      DO 30 JVH = 1,NTUBE
      ZWIR = FGEDE_ZFW(JVH,IPL)
      DELTAZ = ZWIR-XMIMP(3,I)
      XTREF = DELTAZ * TGTRX + XTV
      XFW = FGEDE_XFW(JVH,IPL)


      DO 25 N=NF,NL,ISN
      XWIR = XFW + (FLOAT(N)-1.)*WIRED
      DD = (XTREF-XWIR) * CSTRX        ! drift distance to wires
C*    smear the drift distance according to ACC
      CALL GRANOR(F,X)
      DD = DD + FGEDE_ACC(IPL)*F
      IF(ABS(DD) .GT. FGEDE_DOD(IPL)) GO TO 25
      CALL GRNDM(RNDM,1)
      IF(RNDM.GT.FGEDE_EFF(IPL)) GO TO 25
      IF(NHITDE.GE.MXHIT) THEN
         C78 =  'DGOTR: NHITDE gt. MXHIT'
         CALL ME78MC(910673,0,ILARGEST)
         go to 20
      ENDIF
      NHITDE = NHITDE + 1
      IDIGX(1,NHITDE) = N
      IDIGX(2,NHITDE) = JVH
      DIGX (3,NHITDE) = DD
      DIGX (4,NHITDE) = XWIR
      DIGX (5,NHITDE) = ZWIR
      IDIGX(6,NHITDE) = IXMIMP(5,I)
      IDIGX(7,NHITDE) = IXMIMP(4,I)
      DGOR(NHITDE) = XWIR
 25   CONTINUE
 30   CONTINUE
 20   CONTINUE

      IF(NHITDE.LE.0) GO TO 998

C--   SORT THE  MATRIX DGOR DUE TO INCREASING X

      IF(LBIGR) THEN
         CALL SORTZV(DGOR,IND0,NHITDE,1,0,0)
      ELSE
         CALL SORTZV(DGOR,IND0,NHITDE,1,1,0)
      ENDIF

C*       set digit and hit bank

      NDIGS = 0
      NHITS = 0
      DO 40 IH = 1, NHITDE
      I = IND0(IH)
      N = IDIGX(1,I)
      JVH = IDIGX(2,I)
      CALL MMBOOK(JDIGB,1)
      NN1 = NDIGB()
      IF(NDIGS.EQ.0) IQ(IQGEPT_DIG1(IPL)) = NN1
      NDIGS = NDIGS+1
      IQ(IQDIGB_ID_L()) = 10000*((N-1)*4 + ILOOK(JVH,KORDER))+ IPL
      Q (QDIGB_Q_L()) = DD
      CALL MMBREL(JMIDI,IDIGX(6,I),NN1)

      IF(IWIR(N,JVH).EQ.0) THEN
         CALL MMBOOK(JHITB,1)
         NN = NHITB()
         IF(NHITS.EQ.0) THEN
            IQ(IQGEPT_HIT1(IPL)) = NN
         ENDIF
         NHITS = NHITS+1
         IQ(IQHITB_GEDE_L()) = IPL
         Q (QHITB_X_L()) = DIGX(4,I)
         Q (QHITB_Z_L()) = DIGX(5,I)
         Q (QHITB_Q_L()) = DIGX(3,I)
         Q (QHITB_R_L()) = FGEDE_ACC(IPL)
         CALL MMBREL(JMIHI,IDIGX(6,I),NN)
         CALL MMBREL(JHIDI,NN,NN1)
         IWIR(N,JVH) = NN
      ELSE
         IF(ABS(DIGX(3,I)).LT.ABS(FHITB_Q(IWIR(N,JVH))))
     +                      Q(QHITB_Q(IWIR(N,JVH))) = DIGX(3,I)
         CALL MMBREL(JMIHI,IDIGX(6,I),IWIR(N,JVH))
         CALL MMBREL(JHIDI,IWIR(N,JVH),NN1)
      ENDIF

   40 CONTINUE

      IQ(IQGEPT_DIGL(IPL)) = NN1
      IQ(IQGEPT_HITL(IPL)) = NN

 999  RETURN

 998  IQ(IQGEPT_DIG1(IPL)) = 0
      IQ(IQGEPT_HIT1(IPL)) = 0
      IQ(IQGEPT_DIGL(IPL)) = -1
      IQ(IQGEPT_HITL(IPL)) = -1
      RETURN
      END



