      SUBROUTINE DGPTQ( ROUT , NOUT )
************************************************************************
*                                                                      *
* DDDDDDDDD      GGGGGGGGGG   PPPPPPPPPPP   TTTTTTTTTTTT   QQQQQQQQQQ  *
* DDDDDDDDDD    GGGGGGGGGGGG  PPPPPPPPPPPP  TTTTTTTTTTTT  QQQQQQQQQQQQ *
* DD       DD   GG        GG  PP        PP       TT       QQ        QQ *
* DD        DD  GG            PP        PP       TT       QQ        QQ *
* DD        DD  GG            PP        PP       TT       QQ        QQ *
* DD        DD  GG            PPPPPPPPPPPP       TT       QQ        QQ *
* DD        DD  GG     GGGGG  PPPPPPPPPPP        TT       QQ        QQ *
* DD        DD  GG     GGGGG  PP                 TT       QQ    QQ  QQ *
* DD        DD  GG        GG  PP                 TT       QQ     QQ QQ *
* DD       DD   GG        GG  PP                 TT       QQ      QQQQ *
* DDDDDDDDDD    GGGGGGGGGGGG  PP                 TT       QQQQQQQQQQQ  *
* DDDDDDDDD      GGGGGGGGGG   PP                 TT        QQQQQQQQ QQ *
*                                                                      *
************************************************************************
*   Original version by A. Schwartz/A. Tumanov   Aug, 1997             *
*   Rewritten by I.Belyaev 18.09.97                                    *
*   Description: this routine returns vector of random numbers sampled *
*                according probability distribution function given by  *
*                DGPTF  real function                                  *
*                                                                      *
*   Called from: DGPTO                                                 *
*                                                                      *
*   Modified CERN package FUNLUX is used (all programs starting with   *
*   FUN* are renamed to F0N*, use of GRNDM instead RANLUX.             *
*                                                                      *
*   Arguments:   ROUT     Output value of the random generator         *
*                NOUT     Dimension of ROUT                            *
************************************************************************

      IMPLICIT NONE

#include "arte/ArteUtil.fi"
      
C     *  Input(output) arguments 
      REAL          ROUT(*)
      INTEGER       NOUT 
      
      INTEGER       NNOUT , IOUT 
      
C     *  Range for parameter variation 
      REAL          RMAX 
      PARAMETER    (RMAX=90.72749)
      REAL          RMIN 
      PARAMETER    (RMIN=0.0)
      
      
C     *  Random number vector
      INTEGER       NRAN
      PARAMETER    ( NRAN=128 )
      REAL          RAN( NRAN ) 
      
      INTEGER       IRAN 
      SAVE          IRAN 
      
C     *  Working space for modified FUNLUX package
      INTEGER       SLEN
      PARAMETER    (SLEN=1000)
      REAL          SPACE(SLEN) 
      SAVE          SPACE 
      
      LOGICAL       FIRST
      SAVE          FIRST 
      
C     *  Used in error message geration
      CHARACTER*78  CTEXT
      
      INTEGER       LENOCC 
      EXTERNAL      LENOCC
      
C     * Probability density function   
      REAL          DGPTF
      EXTERNAL      DGPTF  
      
C     * All DATA statenenbt should be the last declaration statements
      DATA          FIRST / .TRUE. / 
      DATA          IRAN  / 0 / 
      
      
      
      IF( FIRST ) THEN
C     *    Generate vector of random numbers 
C     *    and store it in vector RAN
         CALL F0NLXP( DGPTF , SPACE( 1 ) , RMIN , RMAX ) 
         CALL F0NLUX( SPACE( 1 ) , RAN , NRAN )
         FIRST = .FALSE. 
      ENDIF 
      
      
C     *  Redefine input argument 
      NNOUT = NOUT
      IF( NNOUT .LE. 0 )THEN 
         CTEXT='DGPTQ error==> NOUT<0, set NOUT=1!'
         CALL MEMESC(912476,10,ILARGEST,CTEXT(1:LENOCC(CTEXT))) 
         NNOUT = 1 
      ENDIF  
      
      DO IOUT = 1 , NNOUT 
         IRAN = IRAN + 1
         IF( IRAN .GT. NRAN .OR. IRAN .LE. 0 ) THEN
C     *  Generate the random vector once more 
            CALL F0NLUX( SPACE( 1 ) , RAN , NRAN )
            IRAN = 1 
         ENDIF 
         ROUT( IOUT ) = RAN( IRAN ) 
      ENDDO 
      
      RETURN 
      END 
      
