/*
*************************************************************************
**
** Description: header file for EvdirSql common structures and values
**
** Created: 20 Aug 97
** Updated:  
**
** Author:  Sergey Moukhine
**          HERA-B Collaboration
**          JINR-Dubna
**          Russia
**
*************************************************************************
*/
#ifndef EVDIR_SQL_COMMON_H
#define EVDIR_SQL_COMMON_H

#ifndef	MAX
#define MAX(n1, n2)	(((n1) < (n2)) ? (n2) :  (n1))
#endif

#ifndef	MIN
#define	MIN(n1, n2)	(((n1) > (n2)) ? (n2) :  (n1))
#endif

#ifndef NULL
#define NULL 		0
#endif

#define TRUE	    	1
#define	FALSE	    	0
#define DUMMY		(-1)
#define OK		0
#define FAILURE		(-1)

/*--------------------------------------------------------------   
**	Compiler types 
**--------------------------------------------------------------
*/

#define MAX_LEN_OF_NAME	  30
				/* Table of numbers */
typedef enum {
  ValInt,
  ValDouble,
  ValTime,
  ValDate,
  ValBool,
  ValBit,
  ValUnknown
} ValueKind;

typedef union {
  int		intV;		/* ValInt	*/
  double	doubleV;	/* ValDouble	*/
  double	timeV;		/* ValTime	*/
  int		dateV;		/* ValDate	*/
} ValueUnion;

typedef struct {
  ValueKind	kind;
  ValueUnion	v;
} ValueT;

typedef union {
   int i;
   double d;
} NumberValueT;

typedef enum {
   NuInt,
   NuDouble
} NumberKind;

typedef struct {
   NumberKind	kind;
   NumberValueT value;
} NumberT;

				/* Table of names  */
typedef struct {
   char  *text;
} NameT;

				/* Table of dates  */
typedef struct {
   int	day;
   int	month;
   int	year;
} DateT;

				/* Table of times  */
typedef struct {
   int		hours;
   int		minutes;
   double	seconds;
} TimeT;

				/* Table of fields  */
typedef struct {
   int		field;	/*  link to TbName	*/
   ValueKind	kind;
} FieldT;

				/* Table of Primary */
typedef enum
{
  PrNumber,
  PrDate,
  PrTime,
  PrField,
  PrUnknown
} PrimaryKind;

typedef struct
{
   PrimaryKind		kind;
   int			link;
} PrimaryT;
				/* Table of Expression */
typedef enum
{
  OpUminus,
  OpUplus,
  OpNot,
  OpBit,
  OpPlus,
  OpMinus,
  OpMult,
  OpDiv,
  OpLt,
  OpLe,
  OpGt,
  OpGe,
  OpEq,
  OpNe,
  OpAnd,
  OpOr,
  OpBetween,
  OpNotBetween,
  OpIn,
  OpNotIn,
  OpPrimary
} OperationKind;

typedef struct
{
   OperationKind	op;
   int			link;
   ValueKind		kind;
} ExprT;
				/* Table of Runs */
typedef struct
{
   int			beg;
   int			end;
} RunPairT;
				/* NameAndValue */
typedef struct {
  ValueKind	kind;
  ValueUnion	v;
  char		name[MAX_LEN_OF_NAME+1];
} NameAndValueT;

#endif

