/*
*************************************************************************
**
** Description: header file for class EvdirSql
**
** Created: 20 Aug 97
** Updated:  
**
** Author:  Sergey Moukhine
**          HERA-B Collaboration
**          JINR-Dubna
**          Russia
**
*************************************************************************
*/
#ifndef EVDIR_SQL_H
#define EVDIR_SQL_H

#include <setjmp.h>
#include <evdir/EvdCommon.h>
#include <evdir/EvdTable.h>
#include <evdir/EvdCompiler.h>


#define ERR_MESS_LEN	500

class EvdirSql {
// TEMP!
//private:
public:
   EvdirSqlCompiler compiler;
   
   char		*tcode;		// T-code (after Compiler)

   int		nOfNames;
   NameT 	*nameTab;	// array of names from Compiler

   int		nOfNumbers;
   ValueT	*numberTab;	// array of numbers from Compiler

   int		nOfDates;
   ValueT	*dateTab;	// array of dates from Compiler

   int		nOfTimes;
   ValueT	*timeTab;	// array of times from Compiler

   int		nOfFields;
   FieldT	*fieldTab;	// array of fields from Compiler
   ValueT	*fieldValue;	// array of field values from outside

   int		nOfPrimary;
   PrimaryT	*primTab;	// array of primaries from Compiler

   int		nOfExpr;
   ExprT	*exprProg;	// expression program from Compiler
   int		compFlag;	// TRUE if successfully compiled

   int		nOfRun;
   RunPairT	*runTab;	// array of run pairs from Compiler
   int          runInd;		// current index of run pair
   int		run;		// current run

   int		nOfValues;
   int		cur;
   ValueT	*valueStk;	// stack of values to evaluate exprProg


   jmp_buf			longJmpBuf;

   void Error(int er);		// return error message and exit
   void Drop();
   void GetTcode();
   double GetStkValue(ValueKind kind, int ind); // get value of kind "kind"

public:

   char errMess[ERR_MESS_LEN];
   
   int Compile(
              char*  sqlText,	// text to be compiled
              int    nFields,	// n of fields in "fields"
              NameAndValueT *fields, // names and types of fields
              char** error	// error message 
              ); 
   				/* 
   				** Compile "sqlText"
   				** return OK or FAILURE 
   				** if FAILURE then "error" contains
   				** address of error message 
   				*/

   int Evaluate(
               int    nFields,	// n of fields in "fields"
               NameAndValueT *fields, // names, types & values of fields
               char** error	// error message 
               ); 
   				/* 
   				** Evaluate expression for "fields"
   				** return TRUE, FALSE or FAILURE  
   				** if FAILURE then "error" contains
   				** address of error message 
   				*/

   int NextRun(void);		// Return next run number; 
   				//        FAILURE if no more runs

   int Print(char *text);		// Debug output 

   EvdirSql();			// Constructor
   ~EvdirSql();			// Destructor

};

#endif
