/*
*************************************************************************
**
** Description: class EvdirSqlPrint:
**	        implementation
**
** Language:	C++
**
** Created: 20 Aug 97
** Updated:  
**
** Author:  Sergey Moukhine
**          HERA-B Collaboration
**          JINR-Dubna
**          Russia
**
*************************************************************************
*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <evdir/EvdCommon.h>
#include <evdir/EvdPrint.h>

//////////////////////////////////////////////////////////////////////////

char* EvdirSqlPrint::NameOfValueKind(ValueKind kind)
{
static char text[30];
switch (kind)
   {
   case ValInt:
      strcpy(text,"Int");
      break;
      
   case ValDouble:
      strcpy(text,"Double");
      break;
      
   case ValTime:
      strcpy(text,"Time");
      break;
      
   case ValDate:
      strcpy(text,"Date");
      break;
      
   case ValBool:
      strcpy(text,"Bool");
      break;
      
   case ValBit:
      strcpy(text,"BitNumber");
      break;
      
   case ValUnknown:
      strcpy(text,"Unknown");
      break;
      
   default:
      strcpy(text,"Wrong");
      break;
      
   }
return(text);
}

//////////////////////////////////////////////////////////////////////////

char* EvdirSqlPrint::NameOfPrimaryKind(PrimaryKind kind)
{
static char text[30];
switch (kind)
   {
   case PrNumber:
      strcpy(text,"Number");
      break;
	    
   case PrDate:
      strcpy(text,"Date");
      break;
	    
   case PrTime:
      strcpy(text,"Time");
      break;
	    
   case PrField:
      strcpy(text,"Field");
      break;
	    
   case PrUnknown:
      strcpy(text,"Unknown");
      break;
   }
return(text);
}

//////////////////////////////////////////////////////////////////////////

char* EvdirSqlPrint::NameOfOperationKind(OperationKind kind)
{
static char text[30];
switch (kind)
   {
   case OpUminus:
      strcpy(text,"U-     ");
      break;

   case OpUplus:
      strcpy(text,"U+     ");
      break;

   case OpNot:
      strcpy(text,"Not    ");
      break;

   case OpBit:
      strcpy(text,"#      ");
      break;

   case OpPlus:
      strcpy(text,"+      ");
      break;

   case OpMinus:
      strcpy(text,"-      ");
      break;

   case OpLt:
      strcpy(text,"Lt     ");
      break;

   case OpLe:
      strcpy(text,"Le     ");
      break;

   case OpGt:
      strcpy(text,"Gt     ");
      break;

   case OpGe:
      strcpy(text,"Ge     ");
      break;

   case OpEq:
      strcpy(text,"Eq     ");
      break;

   case OpNe:
      strcpy(text,"Ne     ");
      break;

   case OpAnd:
      strcpy(text,"And    ");
      break;

   case OpOr:
      strcpy(text,"Or     ");
      break;

   case OpPrimary:
      strcpy(text,"Primary");
      break;

   case OpMult:
      strcpy(text,"*      ");
      break;

   case OpDiv:
      strcpy(text,"/      ");
      break;

   case OpBetween:
      strcpy(text,"Between");
      break;

   case OpNotBetween:
      strcpy(text,"NotBetween");
      break;

   case OpIn:
      strcpy(text,"In     ");
      break;

   case OpNotIn:
      strcpy(text,"NotIn  ");
      break;

   default:
      strcpy(text,"OpUnknown");
      break;
   
   }
return(text);
}

//////////////////////////////////////////////////////////////////////////

char* EvdirSqlPrint::Value(ValueT* v)
{
static char text[100];
strcpy(text,NameOfValueKind(v->kind));
switch (v->kind)
   {
   case ValInt:
      sprintf(text+strlen(text)," %d", v->v.intV);
      break;
      
   case ValDouble:
      sprintf(text+strlen(text)," %g", v->v.doubleV);
      break;
      
   case ValTime:
      {
      int h = (int)(v->v.doubleV/3600.);
      int m = (int)((v->v.doubleV - h*3600.)/60.);
      double s = v->v.doubleV - h*3600. - m*60;
      sprintf(text+strlen(text)," %02d:%02d:%09.6f", h,m,s);
      }
      break;
      
   case ValDate:
      sprintf(text+strlen(text)," %d", v->v.intV);
      break;
      
   case ValBool:
      sprintf(text+strlen(text)," %d", v->v.intV);
      break;
      
   case ValBit:
      sprintf(text+strlen(text)," %d", v->v.intV);
      break;
      
   default:
      sprintf(text+strlen(text)," Wrong");
      break;      
   
   }
return(text);
}

