/*
*************************************************************************
**
** Description: class EvdirSqlCompiler:
**	        Compiler storage functions
**
** Language:	C++
**
** Created: 20 Aug 97
** Updated:  
**
** Author:  Sergey Moukhine
**          HERA-B Collaboration
**          JINR-Dubna
**          Russia
**
*************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>

#include <evdir/EvdTable.h>
#include <evdir/EvdCommon.h>
#include <evdir/EvdCompiler.h>
#include <evdir/EvdPrint.h>

static EvdirSqlPrint prt;

/*
**--------------------------------------------------------------
**	Compiler table initialization (creation)
**--------------------------------------------------------------
*/

void EvdirSqlCompiler::TableInit(TranTablesKind tabKind)

{
   switch (tabKind)
   {
   case TbNumber:
      if (tab->number.Init(sizeof(TyNumber), 0, 10))
         Error(1);
      break;

   case TbName:
      rec->name.text = NULL;
      if (tab->name.Init(sizeof(TyName), 0, 20))
         Error(1);
      break;

   case TbPrimary:
      if (tab->primary.Init(sizeof(TyPrimary), 0, 30))
         Error(1);
      break;

   case TbExpr:
      if (tab->expr.Init(sizeof(TyExpr), 0, 30))
         Error(1);
      break;

   case TbDate:
      if (tab->date.Init(sizeof(TyDate), 0, 10))
         Error(1);
      break;

   case TbTime:
      if (tab->time.Init(sizeof(TyTime), 0, 10))
         Error(1);
      break;

   case TbField:
      if (tab->field.Init(sizeof(TyField), 0, 10))
         Error(1);
      break;

   case TbRun:
      if (tab->run.Init(sizeof(TyRun), 0, 10))
         Error(1);
      break;

   case TbRunTab:
      if (rec->runTab.Init(sizeof(TyRunTab), 0, 10))
         Error(1);
      break;

   case TbAll:
      {
      int i;

      for (i=0; i<TbAll; i++)
         TableInit((TranTablesKind)i);
      }
      break;

   default:
      break;

   }

}

/*
**--------------------------------------------------------------
**	Index in Compiler tables
**--------------------------------------------------------------
*/

int EvdirSqlCompiler::GetIndByName(TranTablesKind tabKind, char *name)

{

/*
**	Get index of record with a given name
*/

int ind;

ind = IndName(name);
if (ind == FAILURE)
   return(FAILURE);
return(TableInd(tabKind, ind));
}

int EvdirSqlCompiler::TableInd(TranTablesKind tabKind, int name)

{

/*
**	Get index of record in which link to TbName == name
*/

   int  i;
   int  n;

   switch (tabKind)
   {
   case TbNumber:
      break;

   case TbName:
      return(name);

   case TbPrimary:
      break;

   case TbExpr:
      break;
      
   case TbDate:
      break;

   case TbTime:
      break;

   case TbField:
      n = tab->field.Nused();
      for (i=0; i<n; i++)
         {
         TyField *f;
         
         f = GET_RECORD(TyField,TbField,i);
         if (f->field == name)
            return(i);
         }
      return(FAILURE);

   default:
      break;
   }   
   ErrorS(7,"CompilerTableInd");
   return(FAILURE);
}

int EvdirSqlCompiler::TableFind(TranTablesKind tabKind)

{

/*
**	Get index of record the same as in Record
*/

   int  i;
   int  n;

   switch (tabKind)
   {
   case TbNumber:
      n = tab->number.Nused();
      for (i=0; i<n; i++)
         {
         TyNumber *ptr;
         
         ptr = GET_RECORD(TyNumber,TbNumber,i);
         if (ptr->kind == rec->number.kind)
            {
            switch(ptr->kind)
               {
               case NuInt:
                  if (ptr->value.i == rec->number.value.i)
                     return(i);
                  break;
               case NuDouble:
                  if (ptr->value.d == rec->number.value.d)
                     return(i);
                  break;
               }
            }
         }
      return(FAILURE);

   case TbName:
      return(IndName(rec->name.text));

   case TbPrimary:
      break;

   case TbExpr:
      break;
      
   case TbDate:
      n = tab->date.Nused();
      for (i=0; i<n; i++)
         {
         TyDate *ptr;
         
         ptr = GET_RECORD(TyDate,TbDate,i);
         if (ptr->day == rec->date.day &&
             ptr->month == rec->date.month &&
             ptr->year == rec->date.year
            )
            return(i);
         }
      return(FAILURE);

   case TbTime:
      n = tab->time.Nused();
      for (i=0; i<n; i++)
         {
         TyTime *ptr;
         
         ptr = GET_RECORD(TyTime,TbTime,i);
         if (ptr->hours == rec->time.hours &&
             ptr->minutes == rec->time.minutes &&
             ptr->seconds == rec->time.seconds
            )
            return(i);
         }
      return(FAILURE);

   case TbField:
      n = tab->field.Nused();
      for (i=0; i<n; i++)
         {
         TyField *ptr;
         
         ptr = GET_RECORD(TyField,TbField,i);
         if (ptr->field == rec->field.field)
            return(i);
         }
      return(FAILURE);

   default:
      break;
   }   
   ErrorS(7,"CompilerTableFind");
   return(FAILURE);
}

/*
**--------------------------------------------------------------
**	N of elements in Compiler tables
**--------------------------------------------------------------
*/

int EvdirSqlCompiler::TableN(TranTablesKind tabKind)

{
   switch (tabKind)
   {
   case TbNumber:
      return(tab->number.Nused());

   case TbName:
      return(tab->name.Nused());

   case TbPrimary:
      return(tab->primary.Nused());

   case TbExpr:
      return(tab->expr.Nused());
      
   case TbDate:
      return(tab->date.Nused());

   case TbTime:
      return(tab->time.Nused());

   case TbField:
      return(tab->field.Nused());

   case TbRun:
      return(tab->run.Nused());

   case TbRunTab:
      return(rec->runTab.Nused());

   default:
      break;
   }   
   ErrorS(7,"CompilerTableN");
   return(FAILURE);
}

/*
**--------------------------------------------------------------
**	Drop Compiler tables
**--------------------------------------------------------------
*/

void EvdirSqlCompiler::TableDrop(TranTablesKind tabKind)

{
int   i;
int   n;
char  *ptr;

   switch (tabKind)
   {
   case TbAll:
      for (i=0; i<TbAll; i++)
         TableDrop((TranTablesKind)i);
      break;

   case TbNumber:
      tab->number.Drop();
      break;

   case TbName:
      n = tab->name.Nused();
      for (i=0; i<n; i++)
	 {
	 ptr = (GET_RECORD(TyName,TbName,i))->text;
	 if (ptr)
	    free(ptr);
	 ptr = NULL;
         }
      tab->name.Drop();
      break;

   case TbPrimary:
      tab->primary.Drop();
      break;

   case TbExpr:
      tab->expr.Drop();
      break;
      
   case TbDate:
      tab->date.Drop();
      break;

   case TbTime:
      tab->time.Drop();
      break;

   case TbField:
      tab->field.Drop();
      break;

   case TbRun:
      n = tab->run.Nused();
      for (i=0; i<n; i++)
	 {
	 TyRun *t;
	 t = (GET_RECORD(TyRun,TbRun,i));
	 t->Drop();
         }
      tab->run.Drop();
      break;

   case TbRunTab:
      break;

   default:
      break;
   }   

}

/*
**--------------------------------------------------------------
**	Add to Compiler tables
**--------------------------------------------------------------
*/

int EvdirSqlCompiler::TableAdd(TranTablesKind tabKind, void *data)

{
int  len;
int  ind;
char *name;

name = (char *)data;
switch (tabKind)
   {
   case TbNumber:
      ind = TableFind(tabKind);
      if (ind != FAILURE)
         return(ind);
      ind = tab->number.Add(&rec->number);
      if (ind == FAILURE)
         Error(1);
      return(ind);

   case TbName:
      len = strlen(name);
      rec->name.text   = (char *) MyAlloc(len + 1);  
      strcpy(rec->name.text, name);
      ind = IndName(name);
      if (ind == FAILURE)
         ind = tab->name.Add(&rec->name);
      if (ind == FAILURE)
         Error(1);
      return(ind);

   case TbPrimary:
      ind = tab->primary.Add(&rec->primary);
      if (ind == FAILURE)
         Error(1);
      return(ind);

   case TbExpr:
      ind = tab->expr.Add(&rec->expr);
      if (ind == FAILURE)
         Error(1);
      return(ind);

   case TbDate:
      ind = TableFind(tabKind);
      if (ind != FAILURE)
         return(ind);
      ind = tab->date.Add(&rec->date);
      if (ind == FAILURE)
         Error(1);
      return(ind);

   case TbTime:
      ind = TableFind(tabKind);
      if (ind != FAILURE)
         return(ind);
      ind = tab->time.Add(&rec->time);
      if (ind == FAILURE)
         Error(1);
      return(ind);

   case TbField:
      ind = TableFind(tabKind);
      if (ind != FAILURE)
         return(ind);
      ind = tab->field.Add(&rec->field);
      if (ind == FAILURE)
         Error(1);
      return(ind);

   case TbRun:
      ind = tab->run.Add(&rec->runTab);
      if (ind == FAILURE)
         Error(1);
      TableInit(TbRunTab);
      return(ind);

   case TbRunTab:
      ind = rec->runTab.Add(&rec->run);
      if (ind == FAILURE)
         Error(1);
      return(ind);

   default:
      break;
   }   

   ErrorS(7,"CompilerTableAdd");
   return(FAILURE);
}

/*
**--------------------------------------------------------------
**	Get from Compiler tables
**--------------------------------------------------------------
*/

void* EvdirSqlCompiler::TableGet(TranTablesKind tabKind, int ind)

{
void*  adr;

   switch (tabKind)
   {
   case TbNumber:
      adr = tab->number.Get(ind);
      if (adr)
         return (adr);
      break;

   case TbName:
      adr = tab->name.Get(ind);
      if (adr)
         return (adr);
      break;

   case TbExpr:
      adr = tab->expr.Get(ind);
      if (adr)
         return (adr);
      break;

   case TbPrimary:
      adr = tab->primary.Get(ind);
      if (adr)
         return (adr);
      break;

   case TbDate:
      adr = tab->date.Get(ind);
      if (adr)
         return (adr);
      break;

   case TbTime:
      adr = tab->time.Get(ind);
      if (adr)
         return (adr);
      break;

   case TbField:
      adr = tab->field.Get(ind);
      if (adr)
         return (adr);
      break;

   case TbRun:
      adr = tab->run.Get(ind);
      if (adr)
         return (adr);
      break;

   case TbRunTab:
      adr = rec->runTab.Get(ind);
      if (adr)
         return (adr);
      break;

   default:
      break;
   }   

ErrorS(7,"*CompilerTableGet");
return(NULL);
}

/*
**--------------------------------------------------------------
**	Get name of element from Compiler tables
**--------------------------------------------------------------
*/

char* EvdirSqlCompiler::TableName(TranTablesKind tabKind, int ind)

{

   switch (tabKind)
   {
   case TbNumber:
      break;
      
   case TbName:
      return(GET_RECORD(TyName,TbName,ind)->text);
      
   case TbExpr:
      break;

   case TbPrimary:
      break;

   case TbDate:
      break;

   case TbTime:
      break;

   case TbField:
      break;

   default:
      break;
   }   

ErrorS(7,"*CompilerTableName");
return(NULL);
}


/*
**--------------------------------------------------------------
**	Print Compiler tables
**--------------------------------------------------------------
*/


void EvdirSqlCompiler::TablePrint(TranTablesKind tabKind)

{
int  i;
int  n;

switch (tabKind)
   {
   case TbNumber:
      n = tab->number.Nused();
      printf("=================== TbNumber:\t\t n = %d\n", n);
      for (i=0; i<n; i++)
         {
         TyNumber *nu;
         
         nu = GET_RECORD(TyNumber,TbNumber,i);
         if (nu->kind == NuInt)
	    printf("%15d\n", nu->value.i);
	 else
	    printf("%g\n", nu->value.d);
         }
      break;
      
   case TbName:
      n = tab->name.Nused();
      printf("=================== TbName:\t\t n = %d\n", n);
      for (i=0; i<n; i++)
         {
	 printf("%s\n",GET_NAME(TbName,i));
         }
      break;

   case TbPrimary:
      n = tab->primary.Nused();
      printf("=================== TbPrimary:\t\t n = %d\n", n);
      for (i=0; i<n; i++)
         {
         TyPrimary *pr;
         
         pr = GET_RECORD(TyPrimary,TbPrimary,i);
         printf("%s\t%d\n", prt.NameOfPrimaryKind(pr->kind),pr->link);
	 }
      break;

   case TbExpr:
      {
      ExprT *ex;
      PrimaryT *pr;
      
      n = tab->expr.Nused();
      printf("=================== TbExpr:\t\t n = %d\n", n);
      for (i=0; i<n; i++)
         {
	 ex = GET_RECORD(TyExpr,TbExpr,i);
	 if (ex->op == OpPrimary)
	    {
            printf("%s", prt.NameOfOperationKind(ex->op));
	    pr = GET_RECORD(TyPrimary,TbPrimary,ex->link);
	    printf("\t%s\t", prt.NameOfPrimaryKind(pr->kind));
	    if (pr->kind == PrField)
	       {
               TyField   *fi;
         
               fi = GET_RECORD(TyField,TbField,pr->link);
	       printf("%s", GET_NAME(TbName,fi->field));
	       }            
	    printf("\n");
	    }
	 else
            printf("%s\n", prt.NameOfOperationKind(ex->op));
         }
      }
      break;

   case TbDate:
      n = tab->date.Nused();
      printf("=================== TbDate:\t\t n = %d\n", n);
      for (i=0; i<n; i++)
         {
         TyDate *ptr;
         
         ptr = GET_RECORD(TyDate,TbDate,i);
	 printf("%2d - %2d - %4d\n", ptr->day, ptr->month, ptr->year);
         }
      break;

   case TbTime:
      n = tab->time.Nused();
      printf("=================== TbTime:\t\t n = %d\n", n);
      for (i=0; i<n; i++)
         {
         TyTime   *ti;
         
         ti = GET_RECORD(TyTime,TbTime,i);
	 printf("%2d : %2d : %g\n",
	        ti->hours, ti->minutes, ti->seconds);
         }
      break;

   case TbField:
      n = tab->field.Nused();
      printf("=================== TbField:\t\t n = %d\n", n);
      for (i=0; i<n; i++)
         {
         TyField   *fi;
         
         fi = GET_RECORD(TyField,TbField,i);
	 printf("%s\n", GET_NAME(TbName,fi->field));
         }
      break;

   case TbRun:
      n = tab->run.Nused();
      printf("=================== TbRun:\t\t n = %d\n", n);
      for (i=0; i<n; i++)
         {
         TyRun    *t;
         TyRunTab *p;
         
         if (i != 0)
            printf("---------------------------\n");
         t = GET_RECORD(TyRun,TbRun,i);
         int nPair = t->Nused();
         memcpy(&rec->runTab,t,sizeof(TyRun));
         for (int iPair=0; iPair<nPair; iPair++)
            {
            p = GET_RECORD(TyRunTab,TbRunTab,iPair);
	    printf("%10d %10d\n", p->beg, p->end);
	    }
         }
      break;

   case TbRunTab:
      n = rec->runTab.Nused();
      printf("=================== TbRunTab:\t\t n = %d\n", n);
      for (i=0; i<n; i++)
         {
         TyRunTab *p;
         p = GET_RECORD(TyRunTab,TbRunTab,i);
	 printf("%10d %10d\n", p->beg, p->end);
         }
      break;

   default:
      break;
   }   
printf("========================================================\n");
}


/******************	Others	******************/

/*--------------------------------------------------------------*/
/*	  Get index in TbName by name. 				*/
/*--------------------------------------------------------------*/

/****************************************************************
 * IndName:			*  name				*
 ****************************************************************/

int EvdirSqlCompiler::IndName(char *name)

{
  int  i;
  int  n;

  n = CompilerData.table.name.Nused();
  for (i=0; i<n; i++)
    {
      if (strcmp(name,GET_RECORD(TyName,TbName,i)->text) == 0)
         return(i);
    }

  return(FAILURE);
}

/****************************************************************
 * MyAlloc:		my maloc function
 ****************************************************************/

void* EvdirSqlCompiler::MyAlloc(int size)

{
  void *adr;

  adr = malloc(size);
  if (!adr)
    Error(1);
  return(adr);
}
