/*
*************************************************************************
**
** Description: class EvdirSqlTable:
**	        Compiler Table manipulations
**
** Language:	C++
**
** Created: 20 Aug 97
** Updated:  
**
** Author:  Sergey Moukhine
**          HERA-B Collaboration
**          JINR-Dubna
**          Russia
**
*************************************************************************
*/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <evdir/EvdTable.h>
#include <evdir/EvdCommon.h>

/*
**----------------------------------
*/
int EvdirSqlTable::Init(

int     recSize,	/*  record size		*/
int     maxRec,		/*  total n of records	*/
int     ext 		/*  extention		*/

)

{
  record   = NULL;
  nTotal   = 0;
  nExt     = ext;
  nUsed    = 0;
  maxUsed  = 0;
  size     = recSize;

  if (recSize <= 0 || nExt < 0 || maxRec < 0)
     return (FAILURE);

  if (maxRec == 0)
    return(OK);
  record   = malloc(recSize * maxRec);
  if (record)
    {
      nTotal = maxRec;
      return(OK);
    }
  else
    {
      nTotal = 0;
      return(FAILURE);
    }
}

/*
**----------------------------------
*/
void EvdirSqlTable::Print()

{
  printf ("Table: size=%d used=%d total=%d ext=%d max=%d adr=%d\n",
	  size, nUsed, nTotal, nExt,
	  maxUsed, (int)record);
}

/*
**----------------------------------
*/
int EvdirSqlTable::Add(

void    *rec	/*  record	*/

)

{
  int  index;

  if (nTotal < 0)
     return(FAILURE);

  index = nUsed * size;

  if (nUsed >= nTotal)  /* not enought space */
    {
      if (nExt <= 0)
         return(FAILURE);

      nTotal += nExt;
      if (!record)
         record   = malloc(nTotal * size);
      else
         record = realloc(record, nTotal * size);
      if (!record)
        {
          nTotal = 0;
          nUsed  = 0;
          return(FAILURE);
        }
    }

  memmove((void *)((char *)record + index), rec, size);
  nUsed++;
  if (nUsed > maxUsed)
    maxUsed = nUsed;
  return(nUsed - 1);
}

/*
**----------------------------------
*/
int EvdirSqlTable::Replace(

void    *rec,	/*  record to put		*/
int     index	/*  index of existing record	*/

)

{
  if (!record)
     return(FAILURE);
  if (index < 0 || index >= nUsed)
     return(FAILURE);

  memmove((void *)((char *)record + index*size),
          rec, size);
  return(index);
}

/*
**----------------------------------
*/
void *EvdirSqlTable::Get(

int     index	/*  index of existing record	*/

)

{
  if (!record)
     return(NULL);
  if (index < 0 || index >= nUsed)
     return(NULL);

  return((void *)((char *)record + index*size));
}

/*
**----------------------------------
*/
void *EvdirSqlTable::Pop()

{
  int i = nUsed-1;
  if (!record)
     return(NULL);
  if (i < 0)
     return(NULL);
  nUsed--;
  return((void *)((char *)record + i*size));
}

/*
**----------------------------------
*/
int EvdirSqlTable::Nused()

{
  return(nUsed);
}

/*
**----------------------------------
*/
void EvdirSqlTable::Drop()

{
  if (record)
     free(record);
  record   = NULL;
  nTotal   = 0;
  nExt     = 0;
  nUsed    = 0;
  maxUsed  = 0;
  size     = 0;
}

/*
**----------------------------------
*/
void EvdirSqlTable::Empty()

{
  nUsed = 0;
}

