/*************************************************************************
*
* CR_ARTE
* written by Siegmund Nowak
*
* This routines defines geometry of the Inner Tracker
*
*       called from G..MAIN
*
*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/Arte.hh"
#include "arte/ArteUtil.h"
#include "arte/ArteMessage.hh"
#include "arte/Pointer.hh"
#include "gean/gclink.h"
#include "gean/gcvolu.h"
#include "gean/zgeom.h"
#include "auto/GEO.hh"
#include "gean/ZGESL.h"

extern "C" {
    void crlvbk_(int*,int*,int*,int*,char*,int[]);
    void glvolu_(int*,char*,int[],int*);
    void crvtrm_(int*,char*,int[],float[],float[]);    
    void mmbrel_(int&,int*,int*);
    void cr_arte_user_(const int,const int,const char*,const int[],const int);}


void cr_arte(int nr_sl) {
/***********************/
    struct GEDE gede;
    struct GWAL gwal;
    zgvol ZGVOL;
    zgpos ZGPOS;
    zgdet ZGDET;
    int ivos[15],in[15],nin[15],lcopy[15],ltype[15],nlevsl,iuser,indt,nr_vol;
    int nlev =0; int module =0;
    float pos[11], rotm[9];
    char  lvols[60];
    union { int ji; float jx;}; 
    union  { int ii; char volname[4];};

//  check whether the superlayer volume already exists
    strncpy(volname,ZGVOL.Get_volname(nr_sl),4);
    int Gvolnum = ZGVOL.Check_Gvol(volname);
    if (Gvolnum >0){
       int found =0; 
       for(unsigned int isl=1;isl<=ArtePointer<GESL>::nrow();isl++) {
           ArtePointer<GESL> ge(isl);
           if(!strncmp(ge->name,volname,4)) { found = isl; break;}}
       if(!found)   { arteMsg << "cr_arte - nonvalid superlayer volume";
                      arteMsg.error();} 
       crlvbk_(&ii,&found,&indt,&nlevsl,lvols,lcopy);
       nlevsl = nlevsl -1;
       nlev = nlevsl;
       ivos[nlev] = Gvolnum;
       in[nlev] = 0; }
    else {
       arteMsg << "cr_arte -nonvalid superlayer volume";   arteMsg.error();}

L10:       //modules and components
//  define the number of subcomponents nin(nlev) for the new level nlev 
    int jvos = *(gcbank_.lq + gclink_.jvolum -1 - ivos[nlev]);
    ji =*(gcbank_.iq + jvos + 2);
    nin[nlev] = int(jx);     

L15:
//  go to the next (first) component of level nlev          
    in[nlev]++;
//  if all components of this level are handled go to one level above or return 
    if(in[nlev] > nin[nlev]) { 
       nlev--;
       if (nlev >= nlevsl) goto L10;
       else
         gcvolu_.nlevel = 0;
         return;           }
//  get the gean banks of the component ivos(nlev) and the name of next subcomp
//  volname or as integer ii
    jvos = *(gcbank_.lq + gclink_.jvolum -1 - ivos[nlev]);
    int jin = *(gcbank_.lq + jvos -1 -in[nlev]);
    ji = *(gcbank_.iq + jin +1);
    int ivot = int(jx);
    ii = *(gcbank_.iq+gclink_.jvolum -1 + ivot);      
    ji = *(gcbank_.iq + jin +2);
    iuser = int(jx);
    int type = iuser>>16;
    if((nlev-nlevsl) == 0) module = iuser&65535; 
    nr_vol = ZGVOL.Get_volnr(volname,type);
    if(nr_vol < 0) goto L15;    

//  increase the level and put the new name and user number in lvols/lcopy
    nlev++;
    strncpy(&lvols[4*nlev],volname,4);
    ltype[nlev] = type;
    lcopy[nlev] = iuser;
//  define the number of subcomponents nin(nlev)
    int jvot = *(gcbank_.lq + gclink_.jvolum -1 -ivot);
    ji = *(gcbank_.iq + jvot +2);
    nin[nlev] = int(jx);

    if(nin[nlev] != 0) {                //    modules only 
       in[nlev] = 0;
       ivos[nlev] = ivot;
       module = iuser&65535;
       goto L10;            }

//  components only
    ji = *(gcbank_.iq + jvot+3); 
    int numed = int(jx);
    int jmd = *(gcbank_.lq + gclink_.jtmed -1 - numed);
    ji = *(gcbank_.iq + jmd + 5);
    int nmat = int(jx);
    int jmt = *(gcbank_.lq + gclink_.jmate -1 - nmat);
    ji = *(gcbank_.iq + jmt +8);
    float xms = jx;

//  prepare and transform positions and rotation matrces
    int nl = nlev+1;
    int ier;
    gcvolu_.nlevel = 0;
    glvolu_(&nl,lvols,lcopy,&ier);
    if (ier != 0) {
       arteMsg << "glvolu-ier";
       arteMsg.error();}
    crvtrm_(&nl,lvols,lcopy,pos,rotm);
    xms = (pos[5]-pos[4])/xms; 

    ArtePointer<GESL> curgesl(zgmoth_.CURRSL);
//  fill GEDE table
    if(ZGVOL.Get_sens_flag(nr_vol) > 2) {
       int ngede = ArtePointer<GEDE>::nrow() + 1;
       memset(&gede,0,sizeof(gede));  
       if(!curgesl->fpl) curgesl->fpl =  ngede;
       curgesl->npl = curgesl->npl +1 ;
       gede.cmp = curgesl->cmp;
       ji = *(gcbank_.iq + jvot+1);       
       gede.shp = int(jx);
       int in = iuser&65535;
       in = iuser<<16;
       gede.lay = in + ZGPOS.Get_layer(nr_sl,module);
       in = module<<16;
       gede.sec = in + ZGPOS.Get_sector(nr_sl,module);

       for (int m=0;m<4;m++) gede.gvol[m] = volname[m];
       float *ptpos = &gede.xmin; 
       for(int k=0; k<6; k++)  *(ptpos+k) = pos[k]; 
       ptpos = &gede.xmad;
       for(int k=0; k<3; k++)  *(ptpos+k) = pos[k+7]; 
       ptpos = &gede.rot[0];
       for(int k=0; k<9; k++)  *(ptpos+k) = rotm[k];
       int n_wire[4]; 
       float sens_dx, sens_dy, pitch[4];
       float x_wire[4], y1_wire[4], y2_wire[4], z_wire[4], wire_or[4] ;
       gede.nsl = ZGDET.Get_sides(nr_vol); 
       ZGDET.Get_sens_dim(nr_vol,sens_dx,sens_dy);
       ZGDET.Get_pitch(nr_vol,pitch,gede.nsl);
       ZGDET.Get_z_wire(nr_vol,z_wire,gede.nsl);
       ZGDET.Get_x_wire(nr_vol,x_wire,gede.nsl);
       ZGDET.Get_n_wire(nr_vol,n_wire,gede.nsl);
       ZGDET.Get_y1_wire(nr_vol,y1_wire,gede.nsl);
       ZGDET.Get_y2_wire(nr_vol,y2_wire,gede.nsl);
       ZGDET.Get_wire_orientation(nr_vol,wire_or,gede.nsl); 

       float sxmn= 1.e+9;    float symn=1.e+9; 
       float sxmx= -1.e+9;   float symx= -1.e+9;
       for (int m=0;m<gede.nsl;m++) {
         if(!n_wire[m]) continue; 
         float cosx =cos(AR_DEGRAD*wire_or[m]);
         float sinx =sin(AR_DEGRAD*wire_or[m]);
         if(!m) {
            gede.cos  = cosx;
            gede.sin  = sinx; }
         gede.xfw[m] = x_wire[m]  + gede.xmin*cosx - gede.ymin*sinx;
         if(gede.xfw[m] < sxmn) sxmn = gede.xfw[m];
         if(gede.xfw[m] + (n_wire[m]-1)*pitch[0] > sxmx) 
                 sxmx = gede.xfw[m] +(n_wire[m]-0.5)*pitch[0] ; 
         gede.y1w[m] = y1_wire[m] + gede.xmin*sinx + gede.ymin*cosx;
         if(gede.y1w[m] < symn) symn = gede.y1w[m];
         gede.y2w[m] = y2_wire[m] + gede.xmin*sinx + gede.ymin*cosx ;
         if(gede.y2w[m] > symx) symx = gede.y2w[m];
         gede.zfw[m] = 0.5*(pos[4]+pos[5]) + z_wire[m]; 
         gede.nwi[m]=  n_wire[m];}

       gede.sxmn = sxmn; gede.sxmx = sxmx; gede.symn = symn; gede.symx = symx;

       gede.pit = pitch[0];
       gede.acc = ZGDET.Get_accuracy(nr_vol);
       gede.eff = ZGDET.Get_effectivity(nr_vol); 
       gede.dod = ZGDET.Get_double_hit_res(nr_vol);
       gede.ms = xms*pow(0.0136*(1.+ 0.038*log(double(xms))),2.);       

       ArtePointer<GEDE>::append(gede);      

       mmbrel_(ArtePointer<GSLX_GEDE>::getLink().getRef(),
               &zgmoth_.CURRSL,&ngede);
       mmbrel_(ArtePointer<GMAX_GEDE>::getLink().getRef(),&nmat,&ngede); 

//     call possibility to make special arrangements for a special detector
       cr_arte_user_(ngede,0,lvols,lcopy,nlev); }

//  fill wall table   
    else if(ZGVOL.Get_sens_flag(nr_vol) <=2) {
      if(xms <= 1.e-10) {nlev = nlev-1;   goto L15;} 

      int ngwal = ArtePointer<GWAL>::nrow() + 1;
      memset(&gwal,0,sizeof(gwal));  
      if(!curgesl->fwl) curgesl->fwl =  ngwal;
      curgesl->nwl =curgesl->nwl +1 ; 
      ji = *(gcbank_.iq + jvot+1);       
      gwal.shp = int(jx) ;
      for (int m=0;m<4;m++) gwal.gvol[m] = volname[m];
      float *ptpos = &gwal.xmin; 
      for(int k=0; k<6; k++)  *(ptpos+k) = pos[k]; 
      ptpos = &gwal.xmad;
      for(int k=0; k<3; k++)  *(ptpos+k) = pos[k]; 
      ptpos = &gwal.rot[0];
      for(int k=0; k<9; k++)  *(ptpos+k) = rotm[k];       
      gwal.nsl =1; 
      gwal.ms = xms*pow(0.0136*(1.+ 0.038*log(double(xms))),2.);       
    
      ArtePointer<GWAL>::append(gwal);      
      
      mmbrel_(ArtePointer<GSLX_GWAL>::getLink().getRef(),
              &zgmoth_.CURRSL,&ngwal);
      mmbrel_(ArtePointer<GMAX_GWAL>::getLink().getRef(),&nmat,&ngwal); 
//    possibility to change something in the automatically produced tables
      cr_arte_user_(0,ngwal,lvols,lcopy,nlev); }

    nlev = nlev-1;
    goto L15; 
       }
      
/* cr_arte_ */
