/************************************************************************
*
* CRKIBK
* written by Siegmund Nowak, 13/02/96 19.56.54
*
* This routine transforms the temporary GEANT vertex and kinematic banks
* into  ARTE banks MVER and MTRA
*
*       called from GUOUT
*
***********************************************************************/
#include <vector.h>
#include <stdio.h>
#include "arte/Arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "gean/gclink.h"
#include "gean/ZGMOTH.h"
#include "gean/zgkine.hh"
#include "auto/MC.hh"
#include "auto/TOPP.hh"


extern "C" {
  void crkibk_();
  int  tpgear_(int*);
  void mmbrel_(ArteLink *, int*, int*);
  void mmfree_(int&);}

void crkibk_() {
   extern vector<hgmtra> *hg_mtra;
   extern vector<int>    *hg_vert;

   vector<hgmtra>::iterator mt, mo;   
   vector<int>::iterator mv;     
   zgkine ZGKINE;
   union { int ji; float jx;};

   if(!arteflag_.mimpsw || hg_mtra->size() == 0) return;

// mark tracks with mimps or where daughters have mimps
   for(int itr=hg_mtra->size(); itr >0; itr--){
      mt = hg_mtra->begin() + itr -1;
      if(!mt->nhit) continue;                  //    no hits
      if(mt->mtra)  continue;                  //    track already booked
      int jtr = *(gcbank_.lq + gclink_.jkine -1 - itr);
      ji = *(gcbank_.iq + jtr +5 ); unsigned int tr_vertex=int(jx); 
      if(!tr_vertex) continue;
      int jvx = *(gcbank_.lq + gclink_.jvertx -1 - tr_vertex);
      ji = *(gcbank_.iq + jvx +4 );    int mo_track=int(jx); 
      if(!mo_track) continue;
      mo = hg_mtra->begin() + mo_track -1;        if(!mo->nhit) mo->nhit = -1;}

// install mtra and mver and relations

   for(unsigned int itr=1; itr <= hg_mtra->size(); itr++){
      mt = hg_mtra->begin() + itr -1;
      if(mt->mtra) {                              // track already booked
         ArtePointer<MTRA> map(mt->mtra);   
         ArtePointer<MTRX> mxp(mt->mtra);   
         map->ml   = mt->len;
         mxp->leng = mt->len;
         map->flag = mt->flag;
         continue;}

      if(!mt->nhit) continue;                    // no hits

//    define geant particle code, geant vertex origin and geant mother track
      int jtr = *(gcbank_.lq + gclink_.jkine -1 - itr);
      ji = *(gcbank_.iq + jtr +4 );      int ipart =int(jx);
      ji = *(gcbank_.iq + jtr +5 );      unsigned int tr_vertex=int(jx); 
      int jvx = *(gcbank_.lq + gclink_.jvertx -1 - tr_vertex);
      ji = *(gcbank_.iq + jvx +4 );      int mo_track=int(jx); 

//    book and fill track bank
      MTRA ma;
      ji = *(gcbank_.iq + jtr    );   ma.px = jx;
      ji = *(gcbank_.iq + jtr +1 );   ma.py = jx;
      ji = *(gcbank_.iq + jtr +2 );   ma.pz = jx;
      ma.ml   = mt->len;
      ma.flag = mt->flag;
      int itopp = tpgear_(&ipart); //transform particle id from geant to arte
      if(!itopp) {           arteMsg << "crkibk: particle code = 0";        
      int idcount = 8762135;  arteMsg.warn(idcount);}
      ArtePointer<TOPP> tp(itopp);
      if(tp->wid > 1.e-9)      ma.flag = 2;              // it is a resonance
      ArtePointer<MTRA>::append(ma);

      // update temporary track bank and fill MTRX bank and rel topp - track
      int nmtra = ArtePointer<MTRA>::nrow();              
      mt->mtra = nmtra;
      ArtePointer<MTRA> map(nmtra);
      ArtePointer<MTRX> mxp(nmtra);
      mxp->leng = mt->len;
      if(tp->wid > 1.e-9)          mxp->mass = tp->mass;
      mmbrel_(&artelink_.jmtto,&itopp,&nmtra);

//    book and fill vertex bank
//    first look for the vertex which track origin is not a resonance
      int iorig = 0;
      if(mo_track > 0) {
         mo = hg_mtra->begin() + mo_track -1;
         mmbrel_(&artelink_.jmtmt,&mo->mtra,&nmtra);     // rel mtra-mtra
         //define particle id of mother 
         int mjtr = *(gcbank_.lq + gclink_.jkine -1 - mo_track);
         ji = *(gcbank_.iq + mjtr +4 );    int mpart =int(jx);
         int mtopp = tpgear_(&mpart);
         ArtePointer<TOPP> mtp(mtopp);                
         if(mtp->wid > 1.e-9) {            // mother is a resonance 
            iorig = -mo->mtra;
            // as long as mother = resonance go to the previous mother track
            while (mo_track &&  mtp->wid > 1.e-9) {  
               mjtr = *(gcbank_.lq + gclink_.jkine -1 - mo_track);
               ji = *(gcbank_.iq + mjtr +5 );   unsigned int tr_vertex=int(jx); 
               int mjvx = *(gcbank_.lq + gclink_.jvertx -1 - tr_vertex);
               ji = *(gcbank_.iq + mjvx +4 );    mo_track=int(jx); 
               if(mo_track) {
                  mjtr = *(gcbank_.lq + gclink_.jkine -1 - mo_track);
                  ji = *(gcbank_.iq + mjtr +5 );     mpart=int(jx);
                  mtopp = tpgear_(&mpart);
                  mtp = ArtePointer<TOPP>(mtopp);}}}}

//    no more mother or mother is not a resonance - look for the vertex origin
      mv = hg_vert->begin() + tr_vertex - 1;
      if(!*mv || tr_vertex > hg_vert->size()) { // vertex not yet  booked
         MVER mver;                         // book and fill hg_vert and mver
         int jvx = *(gcbank_.lq + gclink_.jvertx -1 - tr_vertex);
         ji = *(gcbank_.iq + jvx    );     mver.x=jx; 
         ji = *(gcbank_.iq + jvx + 1);     mver.y=jx; 
         ji = *(gcbank_.iq + jvx + 2);     mver.z=jx - zgmoth_.ZZERO ; 
         ArtePointer<MVER>::append(mver);
         int nmver = ArtePointer<MVER>::nrow();                 
         ZGKINE.mvpush(tr_vertex,nmver);
         ArtePointer<MVER> mv(nmver);
         if(!mo_track) {}                       
//        hier muss gecheckt werden
         else {
            mv->flag = 4;
            mo = hg_mtra->begin() + mo_track -1;         
            int nmoth = mo->mtra;
            ArtePointer<MTRA> moth(nmoth);
            if(iorig < 0 || moth->flag == 3) mv->flag = 2;
            mmbrel_(&artelink_.jmtmv,&nmoth,&nmver); }}

//    fill mtra_orig and relation mtmv
      mv = hg_vert->begin() + tr_vertex -1;
      int nmver = *mv;

      if(iorig < 0) {
           map->orig = iorig; 
           mmbrel_(&artelink_.jmvmt,&nmver,&nmtra);}
      else {
           map->orig = nmver;
           mmbrel_(&artelink_.jmvmt,&nmver,&nmtra);}
    }                                                     // end track loop

   mmfree_(ArtePointer<MTRA>::getLink().getRef());
   mmfree_(ArtePointer<MVER>::getLink().getRef());  }


