
      SUBROUTINE CRVTRM(NLEV,LVOLS,LCOPY,POS,RMAT)

************************************************************************
* CRVTRM
* written by Siegmund Nowak
*
* This routine returns the geometric parameters POS and rotation
* matrix RMAT translated from the different GEANT daughter systems
* into the HERA-B detector system 
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* var1        in           this variable will only be read
************************************************************************
      IMPLICIT NONE

      INTEGER IUCOMP,I,IVOS,JVOS,NPAR,NIN,JIN,IER,IVO,INR,
     +        JMOTH,LVOLS(15),LCOPY(15),NLEV,IMOTH,JS,ISHAP
      REAL POS(11),POSM(3),RMAT(9),SINANG,COSANG,TANANG
      CHARACTER NAME*4,CNAME*4
#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/gcvolu.fi"
#include "gean/ZGMOTH.fi"
#include "arte/Arte.ff" 

      IF(NLEV.LE.0) CALL MEERRO('CRVTRM: NLEV < 0 ')

C     get detector parameters SHAPE, IVOL and relation gmat-gede
      CALL UHTOC(LVOLS(NLEV),4,NAME,4)
      IVOS = IUCOMP (NAME, IQ(JVOLUM+1), IQ(JVOLUM-1))
      IF(IVOS.EQ.0) CALL MEERRO('CRVTRM >> NONVALID VOLUME')
      JVOS = LQ(JVOLUM-IVOS)
      ISHAP = Q(JVOS+2)

c     to get the parameters in MASTER  one has to define the volume tree

      CALL GLVOLU(NLEV,LVOLS,LCOPY,IER)
      IF(IER.NE.0) CALL MEERRO('glvolu-ier = ')
      CALL UHTOC(LVOLS(NLEV-1),4,CNAME,4)
      IMOTH=IUCOMP(CNAME,IQ(JVOLUM+1),IQ(JVOLUM-1))
      JMOTH = LQ(JVOLUM -IMOTH)
      NIN = Q(JMOTH+3)
      IF(NIN.LE.0) CALL MEERRO('CRVTRM: Mother has NIN .le. 0'//CNAME)
      DO I=1,NIN
      JIN  = LQ(JMOTH-I)
      IVO = Q(JIN+2)
      INR = Q(JIN+3) 
      IF(IVOS.EQ.IVO.AND.LCOPY(NLEV).EQ.INR) GO TO 10
      ENDDO
      JIN = 0
      CALL MEERRO('CRVTRM: VOLUME not found in JIN-bank') 

C     get the parameters in MASTER system
 10   CALL VZERO(POS,11)
      POS(3) = zzero 
      CALL GMTOD(POS,POSM,1)

      NPAR = Q(JVOS+5)
      IF(NPAR.EQ.0) GO TO 20
      JS = JVOS+6                !  volume positioned by GSPOS
      GO TO 30

 20   NPAR = Q(JIN+9)
      IF(NPAR.EQ.0) CALL MEERRO( 'CRVTRM >>  no parameters found')
      JS = JIN+9

 30   SINANG = 0.
      COSANG = 1.
      TANANG = 0.

      IF(ISHAP.EQ.1) THEN	 ! box 
         POS(1) = -POSM(1) - Q(JS+1)
         POS(2) = -POSM(1) + Q(JS+1) 
         POS(3) = -POSM(2) - Q(JS+2)
         POS(4) = -POSM(2) + Q(JS+2)
         POS(5) = -POSM(3) - Q(JS+3) 
         POS(6) = -POSM(3) + Q(JS+3) 

      ELSEIF(ISHAP.EQ.2) THEN  ! trapezoid x varying
         POS(1) = -POSM(1) - AMAX1(Q(JS+1),Q(JS+2))
         POS(2) = -POSM(1) + AMAX1(Q(JS+1),Q(JS+2))
         POS(3) = -POSM(2) - Q(JS+3)
         POS(4) = -POSM(2) + Q(JS+3)
         POS(5) = -POSM(3) - Q(JS+4) 
         POS(6) = -POSM(3) + Q(JS+4) 
         POS(7) = 0.5 * (Q(JS+2) - Q(JS+1)) / Q(JS+4)

      ELSEIF(ISHAP.EQ.3) THEN                         ! trapezoid x/y varying
         POS(1) = -POSM(1) - AMAX1(Q(JS+1),Q(JS+2))
         POS(2) = -POSM(1) + AMAX1(Q(JS+1),Q(JS+2))
         POS(3) = -POSM(2) - AMAX1(Q(JS+3),Q(JS+4))
         POS(4) = -POSM(2) + AMAX1(Q(JS+3),Q(JS+4))
         POS(5) = -POSM(3) - Q(JS+5) 
         POS(6) = -POSM(3) + Q(JS+5) 
         POS(7) = 0.5 * (Q(JS+2) - Q(JS+1)) / Q(JS+5)
         POS(8) = 0.5 * (Q(JS+4) - Q(JS+3)) / Q(JS+5)

      ELSEIF(ISHAP.EQ.5) THEN                         ! tube
c        inner radius, outer radius, dz
         POS(1) = Q(JS+1)
         POS(2) = Q(JS+2)
         POS(3) = -POSM(1)
         POS(4) = -POSM(2)
         POS(5) = -POSM(3) - Q(JS+3) 
         POS(6) = -POSM(3) + Q(JS+3)

      ELSEIF(ISHAP.EQ.6) THEN                         ! tubs  
c        inner radius, outer radius, x, y, zmin ,zmax, phi1, phi2
         POS(1) = Q(JS+1)
         POS(2) = Q(JS+2)
         POS(3) = -POSM(1)
         POS(4) = -POSM(2)
         POS(5) = -POSM(3) - Q(JS+3) 
         POS(6) = -POSM(3) + Q(JS+3) 
         POS(7) = Q(JS+4)
         POS(8) = Q(JS+5)

      ELSEIF(ISHAP.EQ.7) THEN                         ! cone 
c        inner radius at small z, outer radius at small x,xpos,ypos
c        zmin, zmax, cos = delta ri, sin = delta ro  
         POS(1) = AMAX1(Q(JS+3),Q(JS+5))
         POS(2) = AMAX1(Q(JS+2),Q(JS+4))
         POS(3) = -POSM(1)
         POS(4) = -POSM(2)
         POS(5) = -POSM(3) - Q(JS+1) 
         POS(6) = -POSM(3) + Q(JS+1) 
         POS(7) = 0.5 * (Q(JS+5) - Q(JS+3))/ Q(JS+1)
         POS(8) = 0.5 * (Q(JS+4) - Q(JS+2))/ Q(JS+1)

      ELSEIF(ISHAP.EQ.10) THEN			! parallelepiped
         POS(1) = -POSM(1) - Q(JS+1)
         POS(2) = -POSM(1) + Q(JS+1) 
         POS(3) = -POSM(2) - Q(JS+2)
         POS(4) = -POSM(2) + Q(JS+2)
         POS(5) = -POSM(3) - Q(JS+3) 
         POS(6) = -POSM(3) + Q(JS+3) 
         IF(ABS(Q(JS+4)).GT.1.E-6) THEN
            TANANG = Q(JS+4)       
            COSANG = 1/SQRT(1.+TANANG**2)
            SINANG = TANANG*COSANG
            POS(1) = POS(1) - TANANG * Q(JIN+6)
            POS(2) = POS(2) - TANANG * Q(JIN+6)
         ELSEIF(ABS(Q(JS+5)).GT.1.E-6.AND.ABS(Q(JS+6)).LT.1.E-6) THEN
            POS(1) = POS(1) - ABS(Q(JS+5)) * Q(JS+3)
            POS(2) = POS(2) + ABS(Q(JS+5)) * Q(JS+3)
            POS(7) = Q(JS+5)
            POS(8) = 0.
         ELSEIF(ABS(Q(JS+4)).LT.1.E-6.AND.ABS(Q(JS+6)).LT.1.E-5  
     +                               .AND.ABS(Q(JS+6)).LT.1.E-6) THEN
c        take it as box - do nothing
         ELSE 
            CALL MEERRO('CRVTRM: check parameters for shape 10')
         ENDIF

      ELSE
         C78 = 'CRVTRM: ISHAP = '
         CALL CH78AI(ISHAP)
         CALL ME78ER
      ENDIF

      POS(10) = COSANG
      POS(11) = SINANG

C     get the rotation matrix
      IF(GRMAT(10,NLEV).EQ.0.0) THEN
         RMAT(1)  = 1.
         RMAT(2)  = 0.
         RMAT(3)  = 0.
         RMAT(4)  = 0.
         RMAT(5)  = 1.
         RMAT(6)  = 0.
         RMAT(7)  = 0.
         RMAT(8)  = 0.
         RMAT(9)  = 1.
      ELSE
         RMAT(1) = GRMAT(1,NLEV)
         RMAT(2) = GRMAT(4,NLEV)
         RMAT(3) = GRMAT(7,NLEV)
         RMAT(4) = GRMAT(2,NLEV)
         RMAT(5) = GRMAT(5,NLEV)
         RMAT(6) = GRMAT(8,NLEV)
         RMAT(7) = GRMAT(3,NLEV)
         RMAT(8) = GRMAT(6,NLEV)
         RMAT(9) = GRMAT(9,NLEV)
      ENDIF

      NLEVEL = 0
 999  RETURN
      END




