
      SUBROUTINE GGCKOV
C.
C.    ******************************************************************
C.    *                                                                *
C.    *   This routine is called for each tracking step of a charged   *
C.    *   particle in a radiator. A Poisson-distributed number of      *
C.    *   photons is generated according to the Cherenkov formula,     *
C.    *   distributed evenly along the track segment and uniformly     *
C.    *   azimuth w.r.t. the particle direction. The parameters are    *
C.    *   then transformed into the Master Reference System, and they  *
C.    *   are added to the particle stack.                             *
C.    *                                                                *
C.    *   ==>Called by : GTMUON, GTHADR, GTELEC                        *
C.    *      Authors     R.Jones, F.Carminati ********                 *
C.    *                                                                *
C.    ******************************************************************
C.
      REAL RPHOT(3),XXPHOT(11)
      LOGICAL ROTATE
      PARAMETER (RFACT=369.81E9)


#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/gcjloc.fi"
#include "gean/gctmed.fi"
#include "gean/gcunit.fi"
#include "gean/gctrak.fi"
#include "gean/gckine.fi"
#include "gean/gcking.fi"
#include "gean/ZGMOTH.fi"
#include "arte/Arte.ff"
*
*   ------------------------------------------------------------------
*
* *** See whether we generate at least one photon
*
      IF(ITRTYP.NE.4.AND.ITRTYP.NE.8) CALL GNCKOV
      CALL GPOISS(DNDL*STEP,NGPHOT,1)
      IF(NGPHOT.EQ.0) GO TO 999
*
* ***  Set up rotation to Particle frame
*
      CALL GFANG(VECT(4),COSTH,SINTH,COSPH,SINPH,ROTATE)
*
* ***  Distribute the photons in origin, direction, momentum
      COSMX  = THRIND/Q(JINDEX+NPCKOV)
      SINMX2 = (1.-COSMX)*(1.+COSMX)
      DO 40 J=1,NGPHOT
         CALL GRNDM(RPHOT, 1)
         IF(IGNEXT.NE.0) THEN
            DS=(STEP-PREC)*RPHOT(1)+PREC
         ELSE
            DS = STEP*RPHOT(1)
         ENDIF
         XXPHOT(1) = VECT(1)-VECT(4)*DS
         XXPHOT(2) = VECT(2)-VECT(5)*DS
         XXPHOT(3) = VECT(3)-VECT(6)*DS
         XXPHOT(11)= TOFG+(STEP-DS)*GETOT/(VECT(7)*CLIGHT)
*
* *** Sample the momentum of the photon
   20    CALL GRNDM(RPHOT, 3)
         PPHOT=PMIN+RPHOT(1)*DP
*
* *** Find in which bin we are
         KMIN = JMIN
         KMAX = NPCKOV
   30    KMED = (KMIN+KMAX)/2
         IF(Q(JTCKOV+1+KMED).LT.PPHOT) THEN
            KMIN = KMED
         ELSE
            KMAX = KMED
         ENDIF
         IF(KMAX-KMIN.GT.1) GOTO 30
         RATIO = (PPHOT-Q(JTCKOV+1+KMIN))/
     +           (Q(JTCKOV+KMIN+2)-Q(JTCKOV+1+KMIN))
         RATI1  = (1.-RATIO)
*
* *** Find the density function value corresponding to the
* *** momentum sampled
         RINDEX = Q(JINDEX+KMIN)*RATI1+Q(JINDEX+KMIN+1)*RATIO
         COST   = THRIND/RINDEX
         SINT2  = (1.-COST)*(1.+COST)
*
* *** Perform hit-and-miss
         IF(RPHOT(2)*SINMX2.GT.SINT2) GO TO 20
         SINT = SQRT(SINT2)
         PHI  = TWOPI*RPHOT(3)
         SINP = SIN(PHI)
         COSP = COS(PHI)
         XXPHOT(4) = SINT*COSP
         XXPHOT(5) = SINT*SINP
         XXPHOT(6) = COST
         XXPHOT(7) = PPHOT
         XXPHOT(8) = COST*COSP
         XXPHOT(9) = COST*SINP
         XXPHOT(10) = -SINT
*
         IF(ROTATE) THEN
            CALL GDROT(XXPHOT(8),COSTH,SINTH,COSPH,SINPH)
            CALL GDROT(XXPHOT(4),COSTH,SINTH,COSPH,SINPH)
         ENDIF
C
C     store as MIMP

         CALL CRRIHI(XXPHOT)

   40 CONTINUE


  999 END
