*-- Author : A.Spiridonov
************************************************************
*                                                          *
      subroutine gufld(x0,b)
*
*     Define field of HERA-B magnet for HBGEAN
*              in ARTE environment
*
*            Version  3.00  Produced at 5/12/96
*
*
*      Input  :
*
*      x0(3) -  coordinates of the point in cm
*
*
*      Output :
*
*      b(3) -  componentes of the magnetic field in kG
*
*
*     Author :  Alexander Spiridonov
*     E-mail :  spiridon@ifh.de
*                                                          *
*                                                          *
************************************************************
*
      implicit none
*
#include "arte/Arte.fi"
#include "gean/ZGESL.fi"
#include "gean/ZGDETV.fi"
#include "gean/ZGMOTH.fi"
#include "auto/GESL.fi"
#include "auto/MF.fi"
*
      integer S W I T C H  
      integer Q I N T E R 

      save switch
*
      integer j,J_P,N_P3,ix,iy,iz,isl
      integer ID_reg,ID_reg_1,ID_reg_2
*
      real x(3),x1(3),x0(3),b(3)
      real xposmg,yposmg,zposmg,field,rad2
      real c_c(20)
*
      real e
      parameter (e=1.e-4)
*
      integer MAX_reg
      parameter ( MAX_reg = 21 )
      integer LOCF , LOC_MF (MAX_reg)
      integer JMFPA(1)
      equivalence ( JMF01 , JMFPA(1) )
*
      integer nx,ny,nz,N_knot
      integer lk000,lk001,lk002,lk010,lk011,lk012,lk020,lk021,lk022
      integer lk100,lk101,lk102,lk110,lk111,lk112,lk120,lk121,lk122
      integer lk200,lk201,lk202,lk210,lk211,lk212,lk220,lk221,lk222
*
      real r00,r01,r02,r10,r11,r12,r20,r21,r22,rr0,rr1,rr2
      real xx0,xx1,xx2,yy0,yy1,yy2,zz0,zz1,zz2
      real xx_0,xx_1,xx_2,xx_3,y_1,y_2,y_3
      real l_inter,q_inter
*
      save ID_reg_2,LOC_MF
      save xposmg,yposmg,zposmg
      data S W I T C H  /0/, Q I N T E R  /1/,ID_reg_2/2/         
*
#include "arte/Arte.ff"
#include "auto/GESL.ff"
#include "auto/MF.ff"
*
*                                                location of knot
*
*     lk(ix,iy,iz)=(ix-1)*ny*nz+(iy-1)*nz+iz
*
*                                                linear interpolation
*
      l_inter(xx_1,xx_2,y_1,y_2,xx_0) =
     + ( (xx_0-xx_2)*y_1+(xx_1-xx_0)*y_2 )/(xx_1-xx_2)
*
*                                                quadratic interpolation
*
      q_inter ( xx_1,xx_2,xx_3,y_1,y_2,y_3,xx_0 ) =
     + (((xx_0-xx_3)  *( ((xx_0-xx_2)*y_1+(xx_1-xx_0)*y_2)
     + / (xx_1-xx_2) ) )
     + + ( (xx_1-xx_0)*( ((xx_0-xx_3)*y_2+(xx_2-xx_0)*y_3) 
     + / (xx_2-xx_3) ) ) )
     + / (xx_1-xx_3)
*
*==================================================================
*
*
*                                             Initialization
      if ( SWITCH .eq. 0 )  then
*
         LOC_MF( 1) = LOCF ( JMF01 ) - LOCF ( JMF01) + 1
         LOC_MF( 2) = LOCF ( JMF02 ) - LOCF ( JMF01) + 1
         LOC_MF( 3) = LOCF ( JMF03 ) - LOCF ( JMF01) + 1
         LOC_MF( 4) = LOCF ( JMF04 ) - LOCF ( JMF01) + 1
         LOC_MF( 5) = LOCF ( JMF05 ) - LOCF ( JMF01) + 1
         LOC_MF( 6) = LOCF ( JMF06 ) - LOCF ( JMF01) + 1
         LOC_MF( 7) = LOCF ( JMF07 ) - LOCF ( JMF01) + 1
         LOC_MF( 8) = LOCF ( JMF08 ) - LOCF ( JMF01) + 1
         LOC_MF( 9) = LOCF ( JMF09 ) - LOCF ( JMF01) + 1
         LOC_MF(10) = LOCF ( JMF10 ) - LOCF ( JMF01) + 1
         LOC_MF(11) = LOCF ( JMF11 ) - LOCF ( JMF01) + 1
         LOC_MF(12) = LOCF ( JMF12 ) - LOCF ( JMF01) + 1
         LOC_MF(13) = LOCF ( JMF13 ) - LOCF ( JMF01) + 1
         LOC_MF(14) = LOCF ( JMF14 ) - LOCF ( JMF01) + 1
         LOC_MF(15) = LOCF ( JMF15 ) - LOCF ( JMF01) + 1
         LOC_MF(16) = LOCF ( JMF16 ) - LOCF ( JMF01) + 1
         LOC_MF(17) = LOCF ( JMF17 ) - LOCF ( JMF01) + 1
         LOC_MF(18) = LOCF ( JMF18 ) - LOCF ( JMF01) + 1
         LOC_MF(19) = LOCF ( JMF19 ) - LOCF ( JMF01) + 1
         LOC_MF(20) = LOCF ( JMF20 ) - LOCF ( JMF01) + 1
         LOC_MF(21) = LOCF ( JMF21 ) - LOCF ( JMF01) + 1
*
*                                              Look for superlayer with
*                                              coordinates of magnet
         xposmg = FGESL_X1(FELDSL)
         yposmg = FGESL_Y1(FELDSL)
         zposmg = FGESL_Z1(FELDSL) + ZZERO    ! ZZERO from sequence ZGMOTH
         rad2   = FGESL_X2(FELDSL)**2
         field  = FGESL_Y2(FELDSL)
         if(DETV(LGESL_FLD).eq.'FLDS') then
            switch = 1
         elseif(DETV(LGESL_FLD).eq.'FLDL') then
            switch = 2
         end if

 1    if(switch.eq.0) go to 996
      end if
*
*                                              Homogeneous magnetic field
*
      if ( SWITCH .eq. 1 ) then
*
        x1(1) = x0(1) - xposmg
        x1(2) = x0(2) - yposmg
        x1(3) = x0(3) - zposmg
*
        b(1) = 0.
        b(3) = 0.
        if(x1(1)**2+x1(3)**2.lt.rad2) then
          b(2) = field
        else
          b(2) = 0.
        end if

*
*                                              Inhomogeneous magnetic field
*
      else if ( SWITCH .eq. 2 ) then
*
      x1(1) = x0(1) - xposmg
      x1(2) = x0(2) - yposmg
      x1(3) = x0(3) - zposmg
*
      if(x1(1).lt.0.) then
        if(-x1(1).lt.1.e-10) x1(1)=0.
      else
        if( x1(1).lt.1.e-10) x1(1)=0.
      end if
      if(x1(2).lt.0.) then
        if(-x1(2).lt.1.e-10) x1(2)=0.
      else
        if( x1(2).lt.1.e-10) x1(2)=0.
      end if
      if(x1(3).lt.0.) then
        if(-x1(3).lt.1.e-10) x1(3)=0.
      else
        if( x1(3).lt.1.e-10) x1(3)=0.
      end if
*
*
*                                               Test if  point is inside
*                                             region treated during previous
*                                                    call of GUFLD
*
      if (
     + x1(1).ge.FMFLD_Xmin(ID_reg_2).and.
     + x1(1).le.FMFLD_Xmax(ID_reg_2).and.
     + x1(2).ge.FMFLD_Ymin(ID_reg_2).and.
     + x1(2).le.FMFLD_Ymax(ID_reg_2).and.
     + x1(3).ge.FMFLD_Zmin(ID_reg_2).and.
     + x1(3).le.FMFLD_Zmax(ID_reg_2)       ) then
*
        if( IMFLD_pipe(ID_reg_2).ne.0) then
*                                                                  e-pipe



          if (
     +      x1(3).ge.FMFLD_Zmin(1).and.
     +      x1(3).le.FMFLD_Zmax(1).and.
     +      x1(1).ge.FMFLD_Xmin(1).and.
     +      x1(1).le.FMFLD_Xmax(1).and.
     +      x1(2).ge.FMFLD_Ymin(1).and.
     +      x1(2).le.FMFLD_Ymax(1)       )   then
            ID_reg=1
            go to 7777
          else
            ID_reg=ID_reg_2
            go to 7777
          end if
*

        else
          ID_reg=ID_reg_2
          go to 7777
        end if
*
      end if
*
        do id_reg_1=2,NMFLD()                ! Search of region where point is
*
          if (    ID_reg_2.ne.ID_reg_1 .and.
     +    x1(1).ge.FMFLD_Xmin(ID_reg_1).and.
     +    x1(1).le.FMFLD_Xmax(ID_reg_1)      )    then
*
          if (
     +    x1(2).ge.FMFLD_Ymin(ID_reg_1).and.
     +    x1(2).le.FMFLD_Ymax(ID_reg_1).and.
     +    x1(3).ge.FMFLD_Zmin(ID_reg_1).and.
     +    x1(3).le.FMFLD_Zmax(ID_reg_1)       )   then
*
            ID_reg_2=ID_reg_1
*
            if( IMFLD_pipe(ID_reg_2).ne.0) then
*                                                                   e-pipe
              if (
     +        x1(3).ge.FMFLD_Zmin(1).and.
     +        x1(3).le.FMFLD_Zmax(1).and.
     +        x1(1).ge.FMFLD_Xmin(1).and.
     +        x1(1).le.FMFLD_Xmax(1).and.
     +        x1(2).ge.FMFLD_Ymin(1).and.
     +        x1(2).le.FMFLD_Ymax(1)       )   then
                ID_reg=1
                go to 7777
              else
                ID_reg=ID_reg_2
                go to 7777
              end if
*
            else
              ID_reg=ID_reg_2
              go to 7777
*
            end if
*
          end if                              ! end if on x,y test
          end if                              ! end if on z   test
*
        end do
*                                             Point is outside of any region
        b(1)=0.
        b(2)=0.
        b(3)=0.
*
        go to 999
*                                             Field evaluation
*
 7777 J_P  = JMFPA ( LOC_MF ( ID_reg ) )
*                                             Ordinary polinimial
      if(IMFLD_meth(ID_reg).eq.1) then
*
      N_P3 = IQ(J_P + 1)
*
      ix = int((x1(1)-FMFLD_Xmin(ID_reg)-e) / FMFLD_DR_x(ID_reg)) + 1
      iy = int((x1(2)-FMFLD_Ymin(ID_reg)-e) / FMFLD_DR_y(ID_reg)) + 1
      iz = int((x1(3)-FMFLD_Zmin(ID_reg)-e) / FMFLD_DR_z(ID_reg)) + 1
*
      x(1) = x1(1) * FMFLD_scal ( ID_reg )
      x(2) = x1(2) * FMFLD_scal ( ID_reg )
      x(3) = x1(3) * FMFLD_scal ( ID_reg )
*
      j = ( (ix-1) * IMFLD_nr_y(ID_reg) * IMFLD_nr_z(ID_reg)
     +    + (iy-1) * IMFLD_nr_z(ID_reg)
     +    + (iz-1) ) * N_P3 + 2 + J_P
*
      if ( N_P3 .eq. 60 ) then                 !    3rd power
*
        b(1) =  Q(j+1)
     +  +x(1)*( Q(j+2)+x(2)*( Q(j+8)+Q(j+16)*x(2)+Q(j+20)*x(3) )
     +  +x(3)*( Q(j+9)+Q(j+18)*x(3) )
     +  +x(1)*( Q(j+5)+Q(j+11)*x(1)+Q(j+14)*x(2)+Q(j+15)*x(3) ))
     +  +x(2)*( Q(j+3)+x(3)*( Q(j+10)+Q(j+19)*x(3) )
     +  +x(2)*( Q(j+6)+Q(j+12)*x(2)+Q(j+17)*x(3) ))
     +  +x(3)*( Q(j+4)+x(3)*( Q(j+7)+Q(j+13)*x(3) ))
*
        b(2) =  Q(j+21)
     +  +x(1)*( Q(j+22)+x(2)*( Q(j+28)+Q(j+36)*x(2)+Q(j+40)*x(3) )
     +  +x(3)*( Q(j+29)+Q(j+38)*x(3) )
     +  +x(1)*( Q(j+25)+Q(j+31)*x(1)+Q(j+34)*x(2)+Q(j+35)*x(3) ))
     +  +x(2)*( Q(j+23)+x(3)*( Q(j+30)+Q(j+39)*x(3) )
     +  +x(2)*( Q(j+26)+Q(j+32)*x(2)+Q(j+37)*x(3) ))
     +  +x(3)*( Q(j+24)+x(3)*( Q(j+27)+Q(j+33)*x(3) ))
*
        b(3) =  Q(j+41)
     +  +x(1)*( Q(j+42)+x(2)*( Q(j+48)+Q(j+56)*x(2)+Q(j+60)*x(3) )
     +  +x(3)*( Q(j+49)+Q(j+58)*x(3) )
     +  +x(1)*( Q(j+45)+Q(j+51)*x(1)+Q(j+54)*x(2)+Q(j+55)*x(3) ))
     +  +x(2)*( Q(j+43)+x(3)*( Q(j+50)+Q(j+59)*x(3) )
     +  +x(2)*( Q(j+46)+Q(j+52)*x(2)+Q(j+57)*x(3) ))
     +  +x(3)*( Q(j+44)+x(3)*( Q(j+47)+Q(j+53)*x(3) ))
*
      else if ( N_P3 .eq. 30 ) then            !    2nd power
*
        b(1) =  Q(j+1)
     +  +x(1)*( Q(j+2)+x(2)*Q(j+8)+x(3)*Q(j+9)+x(1)*Q(j+5))
     +  +x(2)*( Q(j+3)+x(3)* Q(j+10)+x(2)*Q(j+6) )
     +  +x(3)*( Q(j+4)+x(3)*Q(j+7) )
*
        b(2) =  Q(j+11)
     +  +x(1)*( Q(j+12)+x(2)*Q(j+18)+x(3)*Q(j+19)+x(1)*Q(j+15))
     +  +x(2)*( Q(j+13)+x(3)* Q(j+20)+x(2)*Q(j+16) )
     +  +x(3)*( Q(j+14)+x(3)*Q(j+17) )
*
        b(3) =  Q(j+21)
     +  +x(1)*( Q(j+22)+x(2)*Q(j+28)+x(3)*Q(j+29)+x(1)*Q(j+25))
     +  +x(2)*( Q(j+23)+x(3)* Q(j+30)+x(2)*Q(j+26) )
     +  +x(3)*( Q(j+24)+x(3)*Q(j+27) )
*
      else if ( N_P3 .eq. 12 ) then             !    1st  power
*
        b(1) = Q(j+1) + x(1)*Q(j+ 2) + x(2)*Q(j +3) + x(3)*Q(j +4)
*
        b(2) = Q(j+5) + x(1)*Q(j+ 6) + x(2)*Q(j +7) + x(3)*Q(j +8)
*
        b(3) = Q(j+9) + x(1)*Q(j+10) + x(2)*Q(j+11) + x(3)*Q(j+12)
*
      else if ( N_P3 .eq. 3 ) then             !    0 power
*
      b(1) = Q(j+1)
      b(2) = Q(j+2)
      b(3) = Q(j+3)
*
      else                                    ! Error number of coefficients
        go to 997
      end if
*                                             Polinomial constrained by
*                                                Maxwell equation
*
      else if(IMFLD_meth(ID_reg).eq.4) then
*
      N_P3 = IQ(J_P + 1)
*
      ix = int((x1(1)-FMFLD_Xmin(ID_reg)-e) / FMFLD_DR_x(ID_reg)) + 1
      iy = int((x1(2)-FMFLD_Ymin(ID_reg)-e) / FMFLD_DR_y(ID_reg)) + 1
      iz = int((x1(3)-FMFLD_Zmin(ID_reg)-e) / FMFLD_DR_z(ID_reg)) + 1
*
      x(1) = x1(1) * FMFLD_scal ( ID_reg )
      x(2) = x1(2) * FMFLD_scal ( ID_reg )
      x(3) = x1(3) * FMFLD_scal ( ID_reg )
*
      j = ( (ix-1) * IMFLD_nr_y(ID_reg) * IMFLD_nr_z(ID_reg)
     +    + (iy-1) * IMFLD_nr_z(ID_reg)
     +    + (iz-1) ) * N_P3 + 2 + J_P
*
*     c_c( 1) = Q(j+1)
      c_c( 2) = 2.*Q(j+4)
*     c_c( 3) = Q(j+6)
*     c_c( 4) = Q(j+7)
      c_c( 5) = 3.*Q(j+9)
*     c_c( 6) = Q(j+14)
      c_c( 7) = -3.*Q(j+9)-Q(j+14)
      c_c( 8) = 2.*Q(j+12)
      c_c( 9) = 2.*Q(j+13)
*     c_c(10) = Q(j+15)
      c_c(11) = 4.*Q(j+16)
*     c_c(12) = Q(j+21)
*     c_c(13) = Q(j+23)
      c_c(14) = 3.*Q(j+19)
      c_c(15) = 3.*Q(j+20)
      c_c(16) = -6.*(Q(j+16)+Q(j+17)-Q(j+18))
      c_c(17) = -3.*(Q(j+20)+Q(j+23))
      c_c(18) = -6.*(Q(j+16)-Q(j+17)+Q(j+18))
      c_c(19) = -3.*(Q(j+19)+Q(j+21))
      c_c(20) = -6.*(Q(j+22)+Q(j+24))
*
      b(1) = Q(j+1)
     + +x(1)*( c_c(2)+x(2) *( c_c(8)+c_c(16)*x(2)+c_c(20)*x(3) )
     + +x(3)*( c_c(9)+c_c(18)*x(3) )
     + +x(1)*( c_c(5)+c_c(11)*x(1)+c_c(14)*x(2)+c_c(15)*x(3) ))
     + +x(2)*( Q(j+6)  +x(3) *( Q(j+15)+c_c(19)*x(3) )
     + +x(2)*( Q(j+14) +Q(j+21)  *x(2)+c_c(17)*x(3) ))
     + +x(3)*( Q(j+7)  +x(3) *( c_c(7)+Q(j+23)*x(3) ))
*
*                                                           By
*     c_c( 1) = Q(j+2)
*     c_c( 2) = Q(j+6)
      c_c( 3) = 2.*Q(j+5)
*     c_c( 4) = Q(j+8)
*     c_c( 5) = Q(j+12)
      c_c( 6) = 3.*Q(j+10)
      c_c( 7) = -3.*Q(j+10)-Q(j+12)
      c_c( 8) = 2.*Q(j+14)
*     c_c( 9) = Q(j+15)
      c_c(10) = -6.*Q(j+11)-2.*Q(j+13)
*     c_c(11) = Q(j+19)
      c_c(12) = 4.*Q(j+17)
*     c_c(13) = Q(j+24)
      c_c(14) = 6.*(-Q(j+16)-Q(j+17)+Q(j+18))
      c_c(15) = -3.*(Q(j+22)+Q(j+24))
      c_c(16) = 3.*Q(j+21)
      c_c(17) = 3.*Q(j+22)
      c_c(18) = -3.*(Q(j+19)+Q(j+21))
      c_c(19) = 6.*(Q(j+16)-Q(j+17)-Q(j+18))
      c_c(20) = -6.*(Q(j+20)+Q(j+23))
*
      b(2)  = Q(j+2)
     + +x(1)*( Q(j+6)+x(2)*(c_c(8)+c_c(16)*x(2)+c_c(20)*x(3))
     + +x(3)*( Q(j+15) +c_c(18)*x(3) )
     + +x(1)*( Q(j+12) +Q(j+19)*x(1)+c_c(14)*x(2)+c_c(15)*x(3)))
     + +x(2)*( c_c(3)+x(3) *( c_c(10)+c_c(19)*x(3) )
     + +x(2)*( c_c(6)+c_c(12)*x(2)+c_c(17)*x(3) ))
     + +x(3)*( Q(j+8)  +x(3) *( c_c(7)+Q(j+24) *x(3) ))
*
*                                                             Bz
*     c_c( 1) = Q(j+3)
*     c_c( 2) = Q(j+7)
*     c_c( 3) = Q(j+8)
      c_c( 4) = -2.*(Q(j+4)+Q(j+5))
*     c_c( 5) = Q(j+13)
      c_c( 6) = -3.*Q(j+11)-Q(j+13)
      c_c( 7) = 3.*Q(j+11)
*     c_c( 8) = Q(j+15)
      c_c( 9) = -6.*Q(j+9)-2.*Q(j+14)
      c_c(10) = -6.*Q(j+10)-2.*Q(j+12)
*     c_c(11) = Q(j+20)
*     c_c(12) = Q(j+22)
      c_c(13) = 4.*Q(j+18)
      c_c(14) = -3.*(Q(j+22)+Q(j+24))
      c_c(15) = -6.*(Q(j+16)-Q(j+17)+Q(j+18))
      c_c(16) = -3.*(Q(j+23)+Q(j+20))
      c_c(17) = 6.*(Q(j+16)-Q(j+17)-Q(j+18))
      c_c(18) = 3.*Q(j+23)
      c_c(19) = 3.*Q(j+24)
      c_c(20) = -6.*(Q(j+19)+Q(j+21))
*
      b(3)  = Q(j+3)
     + +x(1)*( Q(j+7)  +x(2) *( Q(j+15)+c_c(16)*x(2)+c_c(20)*x(3) )
     + +x(3)*( c_c(9)+c_c(18)*x(3) )
     + +x(1)*( Q(j+13) +Q(j+20)  *x(1)+c_c(14)*x(2)+c_c(15)*x(3) ))
     + +x(2)*( Q(j+8)  +x(3) *( c_c(10)+c_c(19)*x(3) )
     + +x(2)*( c_c(6)+Q(j+22)  *x(2)+c_c(17)*x(3) ))
     + +x(3)*( c_c(4)+x(3) *( c_c(7)+c_c(13)*x(3) ))
*
*
*                                             Interpolation
*
      else if(IMFLD_meth(ID_reg).eq.6) then
*
      N_knot = IQ(J_P + 2)/3
*
      ix = int((x1(1)-FMFLD_Xmin(ID_reg)-e) / FMFLD_DR_x(ID_reg)) + 1
      iy = int((x1(2)-FMFLD_Ymin(ID_reg)-e) / FMFLD_DR_y(ID_reg)) + 1
      iz = int((x1(3)-FMFLD_Zmin(ID_reg)-e) / FMFLD_DR_z(ID_reg)) + 1
*
      if ( Q I N T E R .ne. 0) then          ! Quadratic interpolation
*
      if(ix.gt.IMFLD_nr_x(ID_reg)-2) ix=IMFLD_nr_x(ID_reg)-2
      if(iy.gt.IMFLD_nr_y(ID_reg)-2) iy=IMFLD_nr_y(ID_reg)-2
      if(iz.gt.IMFLD_nr_z(ID_reg)-2) iz=IMFLD_nr_z(ID_reg)-2
*
      xx0 = FMFLD_Xmin(ID_reg) + float(ix-1) * FMFLD_DR_x(ID_reg)
      xx1 = xx0 + FMFLD_DR_x(ID_reg)
      xx2 = xx1 + FMFLD_DR_x(ID_reg)
*
      yy0 = FMFLD_Ymin(ID_reg) + float(iy-1) * FMFLD_DR_y(ID_reg)
      yy1 = yy0 + FMFLD_DR_y(ID_reg)
      yy2 = yy1 + FMFLD_DR_y(ID_reg)
*
      zz0 = FMFLD_Zmin(ID_reg) + float(iz-1) * FMFLD_DR_z(ID_reg)
      zz1 = zz0 + FMFLD_DR_z(ID_reg)
      zz2 = zz1 + FMFLD_DR_z(ID_reg)
*
      nx=IMFLD_nr_x(ID_reg)
      ny=IMFLD_nr_y(ID_reg)
      nz=IMFLD_nr_z(ID_reg)
*      
*      lk(ix,iy,iz)=(ix-1)*ny*nz+(iy-1)*nz+iz
*
*                                                                B_x
 
      lk000 = (ix-1) * ny*nz + (iy-1) * nz + iz + J_P + 2
      lk100 = lk000 + ny*nz
      lk200 = lk100 + ny*nz
*      
      lk001 = lk000 + 1
      lk002 = lk000 + 2
      lk010 = lk000 + nz
      lk011 = lk010 + 1
      lk012 = lk010 + 2
      lk020 = lk010 + nz
      lk021 = lk020 + 1
      lk022 = lk020 + 2
* 
      lk101 = lk100 + 1
      lk102 = lk100 + 2
      lk110 = lk100 + nz
      lk111 = lk110 + 1
      lk112 = lk110 + 2
      lk120 = lk110 + nz
      lk121 = lk120 + 1
      lk122 = lk120 + 2
*
      lk201 = lk200 + 1
      lk202 = lk200 + 2
      lk210 = lk200 + nz
      lk211 = lk210 + 1
      lk212 = lk210 + 2
      lk220 = lk210 + nz
      lk221 = lk220 + 1
      lk222 = lk220 + 2
*
      r00=q_inter(xx0,xx1,xx2,Q(lk000),Q(lk100),Q(lk200),x1(1))     
      r01=q_inter(xx0,xx1,xx2,Q(lk001),Q(lk101),Q(lk201),x1(1))     
      r02=q_inter(xx0,xx1,xx2,Q(lk002),Q(lk102),Q(lk202),x1(1))
      r10=q_inter(xx0,xx1,xx2,Q(lk010),Q(lk110),Q(lk210),x1(1))     
      r11=q_inter(xx0,xx1,xx2,Q(lk011),Q(lk111),Q(lk211),x1(1))     
      r12=q_inter(xx0,xx1,xx2,Q(lk012),Q(lk112),Q(lk212),x1(1))     
      r20=q_inter(xx0,xx1,xx2,Q(lk020),Q(lk120),Q(lk220),x1(1))     
      r21=q_inter(xx0,xx1,xx2,Q(lk021),Q(lk121),Q(lk221),x1(1))     
      r22=q_inter(xx0,xx1,xx2,Q(lk022),Q(lk122),Q(lk222),x1(1))     
*
      rr0=q_inter(yy0,yy1,yy2,r00,r10,r20,x1(2))
      rr1=q_inter(yy0,yy1,yy2,r01,r11,r21,x1(2))
      rr2=q_inter(yy0,yy1,yy2,r02,r12,r22,x1(2))
*
      b(1)=q_inter(zz0,zz1,zz2,rr0,rr1,rr2,x1(3))
*                                                                B_y
      lk000 = lk000 + n_knot
      lk100 = lk100 + n_knot
      lk200 = lk200 + n_knot
*      
      lk001 = lk001 + n_knot
      lk002 = lk002 + n_knot
      lk010 = lk010 + n_knot
      lk011 = lk011 + n_knot
      lk012 = lk012 + n_knot
      lk020 = lk020 + n_knot
      lk021 = lk021 + n_knot
      lk022 = lk022 + n_knot
* 
      lk101 = lk101 + n_knot
      lk102 = lk102 + n_knot
      lk110 = lk110 + n_knot
      lk111 = lk111 + n_knot
      lk112 = lk112 + n_knot
      lk120 = lk120 + n_knot
      lk121 = lk121 + n_knot
      lk122 = lk122 + n_knot
*
      lk201 = lk201 + n_knot
      lk202 = lk202 + n_knot
      lk210 = lk210 + n_knot
      lk211 = lk211 + n_knot
      lk212 = lk212 + n_knot
      lk220 = lk220 + n_knot
      lk221 = lk221 + n_knot
      lk222 = lk222 + n_knot
*
      r00=q_inter(xx0,xx1,xx2,Q(lk000),Q(lk100),Q(lk200),x1(1))     
      r01=q_inter(xx0,xx1,xx2,Q(lk001),Q(lk101),Q(lk201),x1(1))     
      r02=q_inter(xx0,xx1,xx2,Q(lk002),Q(lk102),Q(lk202),x1(1))
      r10=q_inter(xx0,xx1,xx2,Q(lk010),Q(lk110),Q(lk210),x1(1))     
      r11=q_inter(xx0,xx1,xx2,Q(lk011),Q(lk111),Q(lk211),x1(1))     
      r12=q_inter(xx0,xx1,xx2,Q(lk012),Q(lk112),Q(lk212),x1(1))     
      r20=q_inter(xx0,xx1,xx2,Q(lk020),Q(lk120),Q(lk220),x1(1))     
      r21=q_inter(xx0,xx1,xx2,Q(lk021),Q(lk121),Q(lk221),x1(1))     
      r22=q_inter(xx0,xx1,xx2,Q(lk022),Q(lk122),Q(lk222),x1(1))     
*
      rr0=q_inter(yy0,yy1,yy2,r00,r10,r20,x1(2))
      rr1=q_inter(yy0,yy1,yy2,r01,r11,r21,x1(2))
      rr2=q_inter(yy0,yy1,yy2,r02,r12,r22,x1(2))
*
      b(2)=q_inter(zz0,zz1,zz2,rr0,rr1,rr2,x1(3))
*                                                                B_z 
      lk000 = lk000 + n_knot
      lk100 = lk100 + n_knot
      lk200 = lk200 + n_knot
*      
      lk001 = lk001 + n_knot
      lk002 = lk002 + n_knot
      lk010 = lk010 + n_knot
      lk011 = lk011 + n_knot
      lk012 = lk012 + n_knot
      lk020 = lk020 + n_knot
      lk021 = lk021 + n_knot
      lk022 = lk022 + n_knot
* 
      lk101 = lk101 + n_knot
      lk102 = lk102 + n_knot
      lk110 = lk110 + n_knot
      lk111 = lk111 + n_knot
      lk112 = lk112 + n_knot
      lk120 = lk120 + n_knot
      lk121 = lk121 + n_knot
      lk122 = lk122 + n_knot
*
      lk201 = lk201 + n_knot
      lk202 = lk202 + n_knot
      lk210 = lk210 + n_knot
      lk211 = lk211 + n_knot
      lk212 = lk212 + n_knot
      lk220 = lk220 + n_knot
      lk221 = lk221 + n_knot
      lk222 = lk222 + n_knot
*
      r00=q_inter(xx0,xx1,xx2,Q(lk000),Q(lk100),Q(lk200),x1(1))     
      r01=q_inter(xx0,xx1,xx2,Q(lk001),Q(lk101),Q(lk201),x1(1))     
      r02=q_inter(xx0,xx1,xx2,Q(lk002),Q(lk102),Q(lk202),x1(1))
      r10=q_inter(xx0,xx1,xx2,Q(lk010),Q(lk110),Q(lk210),x1(1))     
      r11=q_inter(xx0,xx1,xx2,Q(lk011),Q(lk111),Q(lk211),x1(1))     
      r12=q_inter(xx0,xx1,xx2,Q(lk012),Q(lk112),Q(lk212),x1(1))     
      r20=q_inter(xx0,xx1,xx2,Q(lk020),Q(lk120),Q(lk220),x1(1))     
      r21=q_inter(xx0,xx1,xx2,Q(lk021),Q(lk121),Q(lk221),x1(1))     
      r22=q_inter(xx0,xx1,xx2,Q(lk022),Q(lk122),Q(lk222),x1(1))     
*
      rr0=q_inter(yy0,yy1,yy2,r00,r10,r20,x1(2))
      rr1=q_inter(yy0,yy1,yy2,r01,r11,r21,x1(2))
      rr2=q_inter(yy0,yy1,yy2,r02,r12,r22,x1(2))
*
      b(3)=q_inter(zz0,zz1,zz2,rr0,rr1,rr2,x1(3))
*
      else                                         ! Linear interpolation
*
      if(ix.gt.IMFLD_nr_x(ID_reg)-1) ix=IMFLD_nr_x(ID_reg)-1
      if(iy.gt.IMFLD_nr_y(ID_reg)-1) iy=IMFLD_nr_y(ID_reg)-1
      if(iz.gt.IMFLD_nr_z(ID_reg)-1) iz=IMFLD_nr_z(ID_reg)-1
*
      xx0 = FMFLD_Xmin(ID_reg) + float(ix-1) * FMFLD_DR_x(ID_reg)
      xx1 = xx0 + FMFLD_DR_x(ID_reg)
*
      yy0 = FMFLD_Ymin(ID_reg) + float(iy-1) * FMFLD_DR_y(ID_reg)
      yy1 = yy0 + FMFLD_DR_y(ID_reg)
*
      zz0 = FMFLD_Zmin(ID_reg) + float(iz-1) * FMFLD_DR_z(ID_reg)
      zz1 = zz0 + FMFLD_DR_z(ID_reg)
*
      nx=IMFLD_nr_x(ID_reg)
      ny=IMFLD_nr_y(ID_reg)
      nz=IMFLD_nr_z(ID_reg)
*      
*      lk(ix,iy,iz)=(ix-1)*ny*nz+(iy-1)*nz+iz
*
*                                                               B_x
 
      lk000 = (ix-1) * ny*nz + (iy-1) * nz + iz + J_P + 2
      lk100 = lk000 + ny*nz
*      
      lk001 = lk000 + 1
      lk010 = lk000 + nz
      lk011 = lk010 + 1
* 
      lk101 = lk100 + 1
      lk110 = lk100 + nz
      lk111 = lk110 + 1
*
*
      r00=l_inter(xx0,xx1,Q(lk000),Q(lk100),x1(1))     
      r01=l_inter(xx0,xx1,Q(lk001),Q(lk101),x1(1))     
      r10=l_inter(xx0,xx1,Q(lk010),Q(lk110),x1(1))     
      r11=l_inter(xx0,xx1,Q(lk011),Q(lk111),x1(1))     
*
      rr0=l_inter(yy0,yy1,r00,r10,x1(2))
      rr1=l_inter(yy0,yy1,r01,r11,x1(2))
*
      b(1)=l_inter(zz0,zz1,rr0,rr1,x1(3))
*
*                                                               B_y
 
      lk000 = lk000 + n_knot
      lk100 = lk100 + n_knot
*      
      lk001 = lk001 + n_knot
      lk010 = lk010 + n_knot
      lk011 = lk011 + n_knot
* 
      lk101 = lk101 + n_knot
      lk110 = lk110 + n_knot
      lk111 = lk111 + n_knot
*
*
      r00=l_inter(xx0,xx1,Q(lk000),Q(lk100),x1(1))     
      r01=l_inter(xx0,xx1,Q(lk001),Q(lk101),x1(1))     
      r10=l_inter(xx0,xx1,Q(lk010),Q(lk110),x1(1))     
      r11=l_inter(xx0,xx1,Q(lk011),Q(lk111),x1(1))     
*
      rr0=l_inter(yy0,yy1,r00,r10,x1(2))
      rr1=l_inter(yy0,yy1,r01,r11,x1(2))
*
      b(2)=l_inter(zz0,zz1,rr0,rr1,x1(3))
*
*                                                               B_z 
      lk000 = lk000 + n_knot
      lk100 = lk100 + n_knot
*      
      lk001 = lk001 + n_knot
      lk010 = lk010 + n_knot
      lk011 = lk011 + n_knot
* 
      lk101 = lk101 + n_knot
      lk110 = lk110 + n_knot
      lk111 = lk111 + n_knot
*
*
      r00=l_inter(xx0,xx1,Q(lk000),Q(lk100),x1(1))     
      r01=l_inter(xx0,xx1,Q(lk001),Q(lk101),x1(1))     
      r10=l_inter(xx0,xx1,Q(lk010),Q(lk110),x1(1))     
      r11=l_inter(xx0,xx1,Q(lk011),Q(lk111),x1(1))     
*
      rr0=l_inter(yy0,yy1,r00,r10,x1(2))
      rr1=l_inter(yy0,yy1,r01,r11,x1(2))
*
      b(3)=l_inter(zz0,zz1,rr0,rr1,x1(3))
*
      end if                                  ! end of interpolation
*
      end if                                  ! END OF FIELD EVALUATION
*
      END IF                                  ! END of SWITCH=2
*
 999  return
*
 998  c78='GUFLD : TABLE  MFLD  IS  EMPTY'
      call ME78ER
      return
 997  c78='GUFLD : ERROR NUMBER OF COEFFICIENTS '
      call CH78BI(N_p3)
      call ME78ER
      return
 996  c78='GUFLD : NO MAGNET COORDINATES IN TABLE GESL  '
      call ME78ER
      return
*
      end
*
*.......................................................................
*     END OF GUFLD
*.......................................................................
