/************************************************************************
*
* ITSGEO
* written by Siegmund Nowak
*
* This routines defines geometry of the Inner Tracker
*
*       called from G..MAIN
*
************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/Arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "arte/ArteUtil.h"
#include "gean/zgeom.h"
#include "gean/zgvers.hh"
#include "gean/ZGESL.h"
#include "auto/GEO.hh"

  extern "C" {
    void itsgeo_(int &);
    void cggesl_();
    void gsatt_();
    void gsdet_();}

    static int first_geo[32]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

    int  read_dat(FILE*, int&); 
    void cr_arte(int);
    FILE  *utfilp(char*,char*);

void itsgeo_(int &) {
    char colo[5], mot[5], set[5], filnam[16];
    static FILE *GEO_inp[32];  
    zgrot ZGROT;
    zgmed ZGMED;
    zgvol ZGVOL;
    zgpos ZGPOS;
    zgvers ZGVERS; 

    strcpy(colo,"COLO");
    strcpy(mot ,"blnk");
    strcpy(set ,"ITST");

    ArtePointer<GESL> gesl(zgmoth_.CURRSL);
    int cmp = gesl->cmp;
   
    zgvol::Set_ind_pt(cmp);

    if(first_geo[cmp-1] == 0) {

//     select file name and  open the file
       strcpy(filnam,ZGVERS.Get_vers_datafile(cmp));         
//       strcpy(filnam,"its00.dat");           
       GEO_inp[cmp-1] = utfilp(filnam,"\n");
    
//     reserve storage for zgrot, zgvol and zgpos 
       zgdet::ini(5);
       zgrot::ini(4);   
       zgvol::ini(100);
       zgpos::ini(1000);
       first_geo[cmp-1] = 1;         }

    int icopy = 0;
    int imodule = 0;   
    
//  now the card handling begins

    for (;;)   {
//  read in one volume block
      int datagroup=0; int nr=0;
     datagroup = read_dat(GEO_inp[cmp-1],nr); 

//   no action 
     if (datagroup == 0) continue;                          

//   media
     if (datagroup == 1) {
       ZGMED.Create_Gmed();
       continue; }

//   rotations
     if (datagroup == 2) {
        int nrot=999;
        ZGROT.Create_Grot(nrot);
        continue; }

     char volname[5];
     int nr_vol=999;
     strncpy(volname,ZGVOL.Get_volname(nr_vol),4);
     volname[4] = '\0';
 
//   components or modules
     if (datagroup == 3 || datagroup==4 ) {   
       if(ZGVOL.Get_sens_flag(nr_vol) != 0 ) {
//      check whether the volume already exists
        int Gvolnum = ZGVOL.Check_Gvol(volname); 
        if (Gvolnum == 0) {
            ZGVOL.Create_Gvol(nr_vol);
            if(ZGVOL.Get_sens_flag(nr_vol) > 2 ) {
               int iset,idet; int i1=1, i20=20, i32=32, i50=50; 
               gsdet_(&set,&volname,&i1,&volname,&i32,&cmp,
                      &i50,&i20,&iset,&idet);}}}}

     else {
//      define the superlayer volume and posit it in gean 
        if(!strncmp(gesl->name,volname,4))  {
           if(gesl->lay == 0) {
              arteMsg << "itr-superlayer " << &volname <<" is not activated\n";
              arteMsg.warn();     
              break;}  
           if (gesl->mfl == 0)  strncpy(mot,"MOTH",4);
           else                 strncpy(mot,"MAGN",4); 
           int i0=0, color=-7; 
           cggesl_(&gesl->name,&mot,&i0);          
           gsatt_(&gesl->name,&colo,&color); }}

//    modules 
      if(datagroup == 4 )    {
         if(ZGVOL.Get_sens_flag(nr_vol)!=0) ZGPOS.Gposit_all(nr_vol,icopy); 
         icopy = 0;}

//    superlayers
      if(datagroup == 5 || datagroup == -99)    {
         if(ZGVOL.Get_sens_flag(nr_vol) != 0) {
            ZGPOS.Gposit_all(nr_vol,imodule); 
            cr_arte(nr_vol); } }

     if ( datagroup == -99 )  {                              //end of file
       arteMsg << " CLOSE FILE " << "its00.dat";
       arteMsg.warn();     
       zgdet::fin();
       zgrot::fin();
       zgvol::fin();
       zgpos::fin();
       break;  }                      

     if(datagroup==5 ) break;            }
   }
  
/* itsgeo_ */

