        subroutine otrgeo(isl)
******************************************************************
*
* Reads definition of OTR superlayer from text files
* and declare related Arte tables and Geant volumes.
*
* Called from g97main, g98main
*
* External references:
*   OTR-specific:     otrmod,otrmed,otropen,otrvol,wtldat
*   Reading routines: otrcomc,otrcomi,otrcomr,iread
*   HBGeant:          cggesl,crgwal,crrotm,gradln
*   Geant:            gsposp
*   Arte:             mmbook,mmbrel,atstop,ioclos,me*,ch*
*   Cernlib:          uhtoc,uctoh,spaces,lnblnk,cltou,cutol
*
* Arguments  in/out       Description
*
* isl        in           superlayer number (index for GESL table)
*
* Author: Alexander Lanyov     Oct 1997
*
******************************************************************
*
* Some superlayer values already filled in kagesl.F are just overwritten.
*
************************************************************************
c Problem:
c Wire begin and end fgede_y1w(ilay) and fgede_y2w(ilay) vary also with
c the wire number!
c One should define some special igede_shp=42,34 with fgede_xmad=tan(+-60),
c fgede_ymad=tan(+-5), fgede_zmad=tan(+-5)
************************************************************************

        implicit none

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/gcnum.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGBPIP.fi"
#include "gean/ZGOTR.fi"
#include "gean/ZGDETV.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"
#include "auto/CON.fi"

        integer maxwor,maxstereo
        parameter (maxwor=10,maxstereo=6)
        integer isl

#include "gean/otrgeo.fi"
        integer maxsl,maxrot
        parameter (maxsl=15,maxrot=20)
        character*4 side*5,modtype*16,slname,name
        character*120 cnamfil,modfile,dir
        character*80 buf1(maxpar)

        integer nsl,i,l,ic,ipl,imod,nmod,nlast,isec,ierr,offset
        integer iw,il,is,ip,iz,iside,sec1,sec2,nsec,iy,iy1,iy2
        logical lfirst,ebeam,lsldec,lhadec,lmodec,newgeom,cosmic
        save nsl,nmod,lfirst,cosmic

        integer iNaN
        real NaN,angle,angle_old,sinang,cosang,tanang
        real rotz,rotzmax,sinrot,cosrot,dalign(3),xref,yref,zref
        real xrot,yrot,z,xlab,ylab,xp,yp,zp,xp_rot,yp_rot
        real xp_mo,yp_mo,zp_mo,xp_,yp_,xnew,ynew
        real xmin,xmax,ymin,ymax,zmin,zmax,ymid
        real xmin_lab,xmax_lab,ymin_lab,ymax_lab
        real xmin_de,xmax_de,ymin_de,ymax_de,zmin_de,zmax_de
        real xmin_mo,xmax_mo,ymin_mo,ymax_mo,zmin_mo,zmax_mo
        real xmin_ge,xmax_ge,ymin_ge,ymax_ge,zmin_ge,zmax_ge
        real sxmn_de,sxmx_de,symn_de,symx_de
        real ydivul(motlay,6),ydivdl(motlay,6)
        real d,pit,pit1,ydg10,dyg10,x1,x2,x1_,x2_,dx(5),dx1,dx2,otdisd
        real zstereo(maxstereo),cov_th,cov_d1,cov_d2,foipin,angpar
        integer nwires,ncells,mcells,mcells_ge,dfi,dla,dfio,dlao
        logical nowires,ldiv,justloaded
        integer ilay,ilay1,ilay2,il0,il1,il2,jl1,jl2,jl
        integer ndiv,idiv,ndiv0,idiv0
        integer irot,jrot,ir,it,ir_g10
        save irot,jrot

        integer igede,igede1,nvol,ivo,jvo,numed,jtm,nmat,imat
        integer ityp,ityp0,ityp_g,ityp_ge,laytyp,imflag,npar
        integer ltyp_old,ltyp_new,ltyp_geo
        real par(10),xmsc,xms_tmp,tmp
        character*4 modshape,nam,matg10(2),matnam*20
        character*100 c100
c Function types
        integer lenocc,iucomp,matnum,otrsect,iread
        logical otrcomc,otrcomi,otrcomr
        character*80 spaces
        ! character*(*) spaces !???
           ! copied from /cern/96a/src/packlib/kernlib/kernbit/m433/spaces.F

c required for icdeci (M432)
        integer nd,ne,nf,ng,num
        real dummy
        common /slate/ nd,ne,nf,ng,num(2),dummy(34)

        data ydg10/1./
        data lfirst/.true./

#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/CON.ff"

c-------------------------------- initialization
        if (lfirst) then
           nmod=0       ! # of module definition loaded

C-- Define volume for chambers at first entry
           if (int(geomvers).eq.97) then
              call wtldat('otr97.dat')
           else
              call wtldat('otr.dat')
           end if

           call otrmed
           call otrvol

           matg10(1) = 'WTGT'
           matg10(2) = 'WTGM'

           cosmic = .false.

           newgeom = .false.
           if (int(geomvers).ne.97) newgeom = .true.

           lfirst=.false.
        end if    
c-------------------------------- End of initialization

        currsl = isl

        call uhtoc(iq(hqgesl_name(isl)),4,slname,4)
        if (lenocc(slname).le.0)
     +         call meerro('OTRGEO: SLNAME not initialized!')

        xmin_all = 1.e10 ! big positive number
        xmax_all =-xmin_all
        ymin_all = xmin_all
        ymax_all = xmax_all
        zmin_all = xmin_all
        zmax_all = xmax_all

        xmin_sl = fgesl_x1(isl)
        xmax_sl = fgesl_x2(isl)
        ymin_sl = fgesl_y1(isl)
        ymax_sl = fgesl_y2(isl)
        zmin_sl = fgesl_z1(isl)
        zmax_sl = fgesl_z2(isl)

        nsl = IGESL_NSL(ISL)
        if (nsl.le.0 .or. nsl .gt. maxsl) then
           c78 = 'Illegal SL number ='
           call ch78bi(nsl)
           call meerro(c78)
        endif

        cnamfil=slname(:lenocc(slname))//'.txt'
        call cutol(cnamfil)


        if (int(geomvers).ne.97) then
           dir = 'otr98'
        else
           dir = 'otr97'
        end if
        call otropen(lread,cnamfil,dir)

        if (cnamfil(:1).eq.' ') then
           if (slname(:2).eq.'MC' .and. int(geomvers).ne.97) then
              c78 = 'Superlayer ' // slname //
     @              ' geometry is not specified.'
              call me78wa
           else
              c78 = 'File'
              call ch78bc(cnamfil(:lenocc(cnamfil)))
              call ch78bc('was not opened')
              call meerro(c78)
           end if
           return
        end if

        lsldec = .false.
        lhadec = .false.
        lmodec = .false.

        iNaN = -10000000        ! number which should give an error if used
        NaN = -1.e35            ! number which should give an error if used
        angle = NaN

c default pin distance - normally should be specified in text file!
        if      (slname(:2).eq.'TC') then
           otdisd = 465.3
        else if (slname(:2).eq.'PC') then
           otdisd = 295.
        else if (slname.eq.'MC01') then
             otdisd = 94.8
        else if (slname.eq.'MC02') then
             otdisd = 110.4
        else
           call meerro('Default pin distance not defined for '//slname)
        end if
        do i=1,6
           zstereo(i) = 1500.
        end do

        rotzwt(nsl) = 0.
        rotzmax = 0.
        dalign(1) = 0.
        dalign(2) = 0.
        dalign(3) = 0.
        xref = 0.
        yref = 0.
        zref = 0.

        ltyp_geo = igesl_ltyp(isl)      ! layer types/sequences
        iq(iqgesl_ltyp(isl)) = 0        ! layer types/sequences

c================================================ begin of reading loop
 100    continue
        read (lread,'(a)',end=991) buf

        do i=1,len(buf)
           ic=ichar(buf(i:i))
c           if (ic.ge.ichar('A') .and. ic.le.ichar('Z'))
c     @        buf(i:i)=char(ic+ichar('a')-ichar('A'))
           if (ic.eq. 9) buf(i:i)=' '           ! Tab
           if (ic.eq.13) buf(i:i)=' '           ! CR
c           if (buf(i:i).ne.' ') nlast=i
        end do

        buf = spaces(buf,1)
        nlast = lenocc(buf)
        call cutol(buf(:nlast))
        if (nlast.le.0) goto 100
        if (buf(:1).eq.'*') goto 100
        if (buf(:5).eq.'free ') goto 100
        if (buf(:2).eq.'" ') goto 100


c------------------------------------------------ Ignored commands
             if (otrcomc('superlayer data - structure',cparam,-1)) then
        else if (otrcomc('stereo layer half'          ,cparam,-1)) then
        else if (otrcomc('stereo layer angle , mrad'  ,cparam,-1)) then
        else if (otrcomc('layer type'                 ,cparam,-1)) then
        else if (otrcomc('# of stereo layer halfs'    ,cparam,-1)) then
        else if (otrcomc('modules of stereo layer'    ,cparam,-1)) then

        else if (otrcomc('summary of coordinates and'  ,cparam,-8)) then
        else if (otrcomc('thickness of 5mm single'     ,cparam,-8)) then
        else if (otrcomc('thickness of 5mm double'     ,cparam,-8)) then
        else if (otrcomc('shift of pin-hole center of' ,cparam,-8)) then
        else if (otrcomc('shift of z-position of 10mm' ,cparam,-8)) then
        else if (otrcomc('thickness of distance piece' ,cparam,-8)) then
        else if (otrcomc('thickness of cover material' ,cparam,-8)) then
        else if (otrcomc('total thickness of'          ,cparam,-8)) then
        else if (otrcomc('length of modules around'    ,cparam,-8)) then
        else if (otrcomc('definition of pin-holder'    ,cparam,-8)) then
        else if (otrcomc('pin-position'                ,cparam,-8)) then
        else if (otrcomc('l2r2'                        ,cparam,-8)) then
        else if (otrcomc('l2r2,'                       ,cparam,-8)) then
        else if (otrcomc('l5r5,'                       ,cparam,-8)) then
        else if (otrcomc('l5r5'                        ,cparam,-8)) then
        else if (otrcomc('m.walter'                    ,cparam,-8)) then
        else if (otrcomc('m.walter.'                   ,cparam,-8)) then
        else if (otrcomc('m.walter,'                   ,cparam,-8)) then
        else if (otrcomc('(m.walter,'                  ,cparam,-8)) then
        else if (otrcomc('a.lanyov'                    ,cparam,-8)) then
        else if (otrcomc('a.lanyov.'                   ,cparam,-8)) then
        else if (otrcomc('a.lanyov,'                   ,cparam,-8)) then
        else if (otrcomc('(a.lanyov,'                  ,cparam,-8)) then
        else if (otrcomc('thickness of cap for 1'      ,cparam,-8)) then
        else if (otrcomc('thickness of cap for stereo' ,cparam,-8)) then
        else if (otrcomc('structure:'                  ,cparam,-8)) then

c       else if (otrcomc(''    ,cparam,-8)) then

c-------------------------------- Cosmic tests
        else if (otrcomc('cosmic',cparam,1)) then
           cosmic = .true.

c-------------------------------- Read alignment corrections (N.B. unit is cm!)
        else if (otrcomr('dx',dalign(1),1)) then
        else if (otrcomr('dy',dalign(2),1)) then
        else if (otrcomr('dz',dalign(3),1)) then
        else if (otrcomr('dphiz',rotzwt(nsl),1)) then
        else if (otrcomr('xref',xref,1)) then
        else if (otrcomr('yref',yref,1)) then
        else if (otrcomr('zref',zref,1)) then


c-------------------------------- Some distances for whole superlayer
        else if (otrcomr('z-position backside basisplate 1.module'//
     @              ' 1.stereo layer at y=0, mm:',zstereo(1),1)) then
           zstereo(1) = 0.1 * zstereo(1)
        else if (otrcomr('z-position backside basisplate 1.module'//
     @              ' 2.stereo layer at y=0, mm:',zstereo(2),1)) then
           zstereo(2) = 0.1 * zstereo(2)
        else if (otrcomr('z-position backside basisplate 1.module'//
     @              ' 3.stereo layer at y=0, mm:',zstereo(3),1)) then
           zstereo(3) = 0.1 * zstereo(3)
        else if (otrcomr('z-position backside basisplate 1.module'//
     @              ' 4.stereo layer at y=0, mm:',zstereo(4),1)) then
           zstereo(4) = 0.1 * zstereo(4)
        else if (otrcomr('z-position backside basisplate 1.module'//
     @              ' 5.stereo layer at y=0, mm:',zstereo(5),1)) then
           zstereo(5) = 0.1 * zstereo(5)
        else if (otrcomr('z-position backside basisplate 1.module'//
     @              ' 6.stereo layer at y=0, mm:',zstereo(6),1)) then
           zstereo(6) = 0.1 * zstereo(6)
        else if (otrcomr('distance between pin-hole centers'//
     @              ' for full length modules, mm',otdisd,1)) then
           otdisd = 0.1 * otdisd
        else if (otrcomr('thickness of cap material, mm',cov_th,-1))then
           cov_th = 0.1 * cov_th
        else if (otrcomr('gap between cap and 1.module in z, mm',
     @                                          cov_d1,1))then
           cov_d1 = 0.1 * cov_d1
        else if (otrcomr('gap between last module and cap in z, mm',
     @                                          cov_d2,1))then
           cov_d2 = 0.1 * cov_d2




c-- dummy at the moment
      else if (otrcomr('gap between cap and 1.module in z, mm, double',
     @                                          dummy,1))then
      else if (otrcomr('gap between cap and 1.module in z, mm, single',
     @                                          dummy,1))then





        else if (otrcomr('distance between foil end'//
     @            ' and pin-hole center, mm',foipin,1))then
           foipin = 0.1 * foipin

c----------------------------- What data could be used:
c thickness of 5mm double layer module, mm :                      12.5
c shift of pin-hole center of 2nd submodule in -x: 2.5*8.7=       21.75
c shift of z-position of 10mm module backside of basisplate, mm   13.5
c thickness of cap for 1 stereo layer in z, mm                    47.
c thickness of distance piece between caps including glue, mm      2.2
c thickness of cover material in z, mm                            20.
c total thickness of left superlayer half, mm                    185.6
c length of modules around p-beam(tc-101 - tc-105), mm          2161.5
c total thickness of superlayer half, mm                         185.6
c definition of pin-holder position for g10-strips (see module tables):
c-----------------------------

c----------------------------------------------------------- command SL
        else if (otrcomc('name',cparam,1)
     @      .or. otrcomc('superlayer',cparam,-4)) then

           name = cparam(1)
           if (name(4:4).eq.' ') then
              name(4:4) = name(3:3)
              name(3:3) = '0'
           end if
           call cltou(name)
           if(slname.ne.name) call meerro(
     @            'Encountered '//name//' while expecting : '//slname)

           if (lsldec) then
              c78 = 'SL has been already declared in file "'
              call ch78ac(cnamfil(:lenocc(cnamfil)))
              call ch78ac('"')
              call meerro(c78)
              goto 991
           end if

           ipl=0

c These are the coordinates of the whole SL in Lab system.
        else if (otrcomr('xmin',xmin_sl,1)) then

c give warning - also for other coordinates!
           if (fgesl_x1(isl).ne.0. .and. xmin_sl.lt.fgesl_x1(isl)) then
              c78 = 'New xmin is less than previous'
              call ch78bf(fgesl_x1(isl),-2)
              call me78wa
           end if

        else if (otrcomr('xmax',xmax_sl,1)) then
        else if (otrcomr('ymin',ymin_sl,1)) then
        else if (otrcomr('ymax',ymax_sl,1)) then

c
c S.Nowak: superlayer Z-sizes should be defined only in a special SL file.
c So ignore this information.
c

       else if (otrcomr('zmin',dummy,1)
     @     .or. otrcomr('minimal z-coord. of sl',dummy,1)) then

       else if (otrcomr('zmax',dummy,1)
     @     .or. otrcomr('maximum z-coord. of sl',dummy,1)) then


c----------------------------------------------------------- command Half
        else if (otrcomc('half',buf1,-5)) then


           angle_old = angle

           if (newgeom) then
              read (buf1(1),*,err=99) angle
              angle = angle/1000. * 180. / 3.14159      ! mrad --> degrees

              angle = -angle

              iside = iread(buf1(3))
              if (iside.eq.-1) then
                 side = 'left'
              else if (iside.eq.1) then
                 side = 'right'
              else
                 c78 = 'Wrong side in HALF command (98 geometry)'
                 call meerro (c78)
              end if
           else
              read (buf1(1),*,err=99) angle
              side = buf1(2)
              if (side(:4).eq.'left') then
                 iside = -1
              else if (side(:4).eq.'righ') then
                 iside = 1
              else
                 c78 = 'Wrong side in HALF command (97 geometry)'
                 call meerro (c78)
              end if
           end if

           if (abs(angle-angle_old) .gt. 1.e-5) then
c In principle, it is better to check also the layer side (iside).
              ipl = ipl + 1

              if (abs(angle) .le. 1.1*rotzmax+0.001) then ! angle - in degrees
                 it=0
              else if (angle.gt.0.) then
                 it=1
              else if (angle.lt.0.) then
                 it=2
              end if

c       write (*,*) 'angle=',angle,' rotzmax=',rotzmax,' it=',it,
c     @              ' ipl=',ipl

c layer types/sequences
c We will add 3 if later we find that this is double layer!
              iq(iqgesl_ltyp(isl)) = iq(iqgesl_ltyp(isl))*10 + it
           end if

           if (int(geomvers).eq.97) iside = 1   ! 97 run has correct data!


           sinrot = sin(rotzwt(nsl))
           cosrot = cos(rotzwt(nsl))
           rotz = rotzwt(nsl)*raddeg  ! correction of SL angles from alignment
           rotzmax = max(rotzmax, abs(rotz))
           angle = angle + rotz


c--------- find rotation matrix (create one if does not exist)
        call crrotm(irot,90.,-angle,90.,90.-angle,0.,0.)
        jrot = lq(jrotm-irot)
        if (jrot.eq.0) call meerro ('OTRGEO: illegal rot.matrix!')
        ir=irot
        if (angle.eq.0.) ir=0
        currot = ir


        cosang = cos(angle * 0.01745329)
        sinang = sin(angle * 0.01745329)
        tanang = sinang/cosang

c--------- Fill GESL table with new information
           if (.not. lsldec) then               ! Book new SL
c
c Alignment corrections
c
              if (int(geomvers).eq.97) then
                 xmax_sl = xmax_sl + dalign(1)
                 ymax_sl = ymax_sl + dalign(2)
                 zmax_sl = zmax_sl + dalign(3)

                 xmin_sl = xmin_sl + dalign(1)
                 ymin_sl = ymin_sl + dalign(2)
                 zmin_sl = zmin_sl + dalign(3)
c In principle, one should also take into account rotation!???
              end if

              q(qgesl_x1(isl)) = xmin_sl ! Already filled in KAGESL?
              q(qgesl_x2(isl)) = xmax_sl ! Already filled in KAGESL?
              q(qgesl_y1(isl)) = ymin_sl ! Already filled in KAGESL?
              q(qgesl_y2(isl)) = ymax_sl ! Already filled in KAGESL?
              q(qgesl_z1(isl)) = zmin_sl ! Already filled in KAGESL?
              q(qgesl_z2(isl)) = zmax_sl ! Already filled in KAGESL?

c           if (xmin_sl.lt.fgesl_x1(isl)) then
c              c78 = 'xmin changed'
c              call ch78bf(fgesl_x1(isl),-2)
c              call ch78bf(xmin_sl,-2)
c              call me78wa
c              q(qgesl_x1(isl)) = xmin_sl 
c           end if
c           if (xmax_sl.gt.fgesl_x2(isl)) then
c              c78 = 'xmax changed'
c              call ch78bf(fgesl_x2(isl),-2)
c              call ch78bf(xmax_sl,-2)
c              call me78wa
c              q(qgesl_x2(isl)) = xmax_sl 
c           end if
c           if (ymin_sl.lt.fgesl_y1(isl)) then
c              c78 = 'ymin changed'
c              call ch78bf(fgesl_y1(isl),-2)
c              call ch78bf(ymin_sl,-2)
c              call me78wa
c              q(qgesl_y1(isl)) = ymin_sl 
c           end if
c           if (ymax_sl.gt.fgesl_y2(isl)) then
c              c78 = 'ymax changed'
c              call ch78bf(fgesl_y2(isl),-2)
c              call ch78bf(ymax_sl,-2)
c              call me78wa
c              q(qgesl_y2(isl)) = ymax_sl 
c           end if
c           if (zmin_sl.lt.fgesl_z1(isl)) then
c              c78 = 'zmin changed'
c              call ch78bf(fgesl_z1(isl),-2)
c              call ch78bf(zmin_sl,-2)
c              call me78wa
c              q(qgesl_z1(isl)) = zmin_sl 
c           end if
c           if (zmax_sl.lt.fgesl_z2(isl)) then
c              c78 = 'zmax changed'
c              call ch78bf(fgesl_z2(isl),-2)
c              call ch78bf(zmax_sl,-2)
c              call me78wa
c              q(qgesl_z2(isl)) = zmax_sl 
c           end if

              iq(iqgesl_lay(isl)) = 0   ! number of layers

              if (igesl_mfl(isl).eq.0) then 
                 call cggesl(slname,'MOTH',0)
              else
                 call cggesl(slname,'MAGN',0)
              end if

              xp_sl = 0.5*(xmax_sl + xmin_sl)
              yp_sl = 0.5*(ymax_sl + ymin_sl)
              zp_sl = 0.5*(zmax_sl + zmin_sl)

              lsldec = .true.
           end if                               ! End of definition of new SL

        lhadec = .true.

c----------------------------------------------------------- command Module
        else if (otrcomc('module',buf1,6)) then

           angpar = 0.
           do i=1,len(buf1(1))
              if (buf1(1)(i:i).eq.'/') buf1(1)(i:i)='_'
           end do
           modtype = buf1(1)

           if (buf1(2)(2:2).eq.'c'
     @         .and.(buf1(2)(3:3).eq.'-' .or. buf1(2)(4:4).eq.'-')) then
              buf1(2) = buf1(3)
              buf1(3) = buf1(4)
              buf1(4) = buf1(5)
              buf1(5) = buf1(6)
           end if
           read (buf1(2),*,err=99) xlab
           read (buf1(3),*,err=99) ylab
           read (buf1(4),*,err=99) z
           isec = iread(buf1(5))

c------------------------------- Extract module information
           justloaded=.false.
           l = min(len(cotnam(1)),len(modtype))
           do imod=1,nmod
              if (cotnam(imod)(:l).eq.modtype(:l)) goto 30
           end do

           write (modfile,'(a)') modtype(:lenocc(modtype))//'.txt'

           nmod=nmod+1
           imod=nmod
           curmod=imod          ! used in otwd96.F

           call otrmod(imod,modfile,ierr)

           if (otdiam(imod).eq.0. .or. ierr.ne.0) then
              c78 = ' ? Module "'//modtype(:l)//'" not loaded'
              call me78wa

              nmod=nmod-1
              imod=nmod
              curmod=imod

              goto 100
           end if
           justloaded=.true.

           maxcell(imod) = 0
           do ilay=1,notlay(imod)
              do is=1,4                 ! is = sectors A,B,C,D
                 maxcell(imod) = max(maxcell(imod),notsew(ilay,is,imod))
              end do
           end do

30         continue
c----------------------------------------------- module found

c----------------------------------------------- Process coordinates
           if (int(geomvers).ne.97) then
              xlab = 0.1 * xlab
              ylab = 0.1 * ylab
              z    = 0.1 * z
           end if

           if (newgeom) then
              if (otdist(imod) .gt. 0.
     @      .and. abs(otdist(imod)-otdisd) .gt. 1.e-3) then
                 c78 = 'Illegal pin distance:'
                 call ch78bf(otdist(imod),-6)
                 call ch78ac(' cm, but default is')
                 call ch78bf(otdisd,-6)
                 call ch78ac(' cm.')
                 call meerro(c78)
              end if

c Calculate center of pins position from one pin position.
              d = 0.5 * otdist(imod)
              if (abs(d).lt.1.e-5) d = 0.5 * otdisd
              if (ylab .lt. 0.) d = -d
              ylab = ylab - cosang * d
              xlab = xlab - sinang * d
           end if

c------------------------------------------------ Alignment corrections
cccccccccc            if (int(geomvers).eq.97) then
              xlab = xlab + dalign(1)
              ylab = ylab + dalign(2)
              z    = z    + dalign(3)

c alignment rotation around Z axis
c              xnew =   xlab * cosrot + ylab * sinrot
c              ynew =  -xlab * sinrot + ylab * cosrot

c alignment rotation around a point with given X,Y-coordinates
              xnew = xref + (xlab-xref) * cosrot + (ylab-yref) * sinrot
              ynew = yref - (xlab-xref) * sinrot + (ylab-yref) * cosrot
              xlab = xnew
              ylab = ynew

cccccccccccccccccc           end if

           zstereo(ipl) = min(z,zstereo(ipl))
c-----------------------------------------------------------------------

           xrot =  xlab * cosang - ylab * sinang        ! lab --> rot
           yrot = +xlab * sinang + ylab * cosang

           if (abs(angle).gt.100.) then
              c78 = 'No HALF command was given (angle is undefined!)'
              call meerro (c78)
           end if


        sec1 =  100
        sec2 = -100

c Patch e.g. for pc-5d32-12 or tc-5d24-12
        if (otg10p(1,1,imod).ne.0.
     @.and. otg10p(1,2,imod).ne.0.
     @.and. otg10p(1,3,imod).ne.0.) then
           sec1=1
           sec2=4
        end if

        do ilay=1,notlay(imod)
           do is=1,4                 ! is = sectors A,B,C,D
              if (notsew(ilay,is,imod).ne.0
     @       .or. notnsw(ilay,is,imod).ne.0 ) then
                 sec1 = min(sec1,is)
                 sec2 = max(sec2,is)
              end if
           end do
        end do
        nsec = sec2-sec1+1

        if (iside.eq.1) then  ! ======================= right side or 97 run
           do ilay=1,notlay(imod)
              do iy=1,2                 ! iy = y>0, y<0
                 ttcefi(ilay,iy,imod) = otcefi(ilay,iy,imod) * iside
                 ttcela(ilay,iy,imod) = otcela(ilay,iy,imod) * iside
                 if (ttcefi(ilay,iy,imod).gt.otcela(ilay,iy,imod)) then
                    tmp = ttcela(ilay,iy,imod)
                    ttcela(ilay,iy,imod) = ttcefi(ilay,iy,imod)
                    ttcefi(ilay,iy,imod) = tmp
                 end if
              end do

              do iy=1,2                 ! iy = y>0, y<0
                 ttwifi(ilay,iy,imod) = min(ttcefi(ilay,1,imod),
     @                                      ttcefi(ilay,2,imod))
                 ttwila(ilay,iy,imod) = max(ttcela(ilay,1,imod),
     @                                      ttcela(ilay,2,imod))
              end do

              do is=1,nsec                 ! is = sectors A,B,C,D
                 if (notsew(ilay,is+sec1-1,imod).eq.0
     @         .and. fotsew(ilay,is+sec1-1,imod).eq.0) then
                    nttsew(ilay,is,imod) = 0
                    fttsew(ilay,is,imod) = 0
                 else
                    fttsew(ilay,is,imod) = 
     @                   maxcell(imod) + 1 - notsew(ilay,is+sec1-1,imod)

                    nttsew(ilay,is,imod) =
     @                   maxcell(imod) + 1 - fotsew(ilay,is+sec1-1,imod)
                 end if
              end do

              do ip=1,3                 ! ip = up,mid,down
                 ttg10p(ilay,ip,imod) = otg10p(ilay,ip,imod) * iside
              end do
              do ip=1,6                 ! ip = up:a,b,c, down:a,b,c
                 ttsupp(ilay,ip,imod) = otsupp(ilay,ip,imod) * iside
              end do
           end do


        else if (iside.eq.-1) then      ! =============== left side

           do ilay=1,notlay(imod)
              do iy=1,2                 ! iy = y>0, y<0
                 ttcela(ilay,iy,imod) = otcefi(ilay,3-iy,imod) * iside
                 ttcefi(ilay,iy,imod) = otcela(ilay,3-iy,imod) * iside
                 if (ttcefi(ilay,iy,imod).gt.otcela(ilay,iy,imod)) then
                    tmp = ttcela(ilay,iy,imod)
                    ttcela(ilay,iy,imod) = ttcefi(ilay,iy,imod)
                    ttcefi(ilay,iy,imod) = tmp
                 end if
              end do

              do iy=1,2
                 ttwifi(ilay,iy,imod) = min(ttcefi(ilay,1,imod),
     @                                      ttcefi(ilay,2,imod))
                 ttwila(ilay,iy,imod) = max(ttcela(ilay,1,imod),
     @                                      ttcela(ilay,2,imod))
              end do

              offset = 0
              if (otdist(imod) .le. 1.e-5
     @        .and. notcht(imod).eq.0 .and. notchb(imod).ne.0) then
                 offset = 1             ! for bottom module


                 nttsew(ilay,is,imod) = 0
                 fttsew(ilay,is,imod) = 0
              end if

              do is=1,nsec                 ! is = sectors A,B,C,D
                 nttsew(ilay,is+offset,imod)=notsew(ilay,sec2+1-is,imod)
                 fttsew(ilay,is+offset,imod)=fotsew(ilay,sec2+1-is,imod)
              end do

              do ip=1,3                 ! ip = up,mid,down
                 ttg10p(ilay,ip,imod) = otg10p(ilay,4-ip,imod) * iside
              end do
              do ip=1,6                 ! ip = up:a,b,c, down:a,b,c
                 ttsupp(ilay,ip,imod) = otsupp(ilay,ip,imod) * iside
              end do
           end do


           if (otdist(imod) .le. 1.e-5
     @    .and. notcht(imod).eq.0 .and. notchb(imod).ne.0) then
              nsec = nsec + 1           ! for bottom module
           end if

        else
           c78 = 'Illegal iside ='
           call ch78bi(iside)
           call meerro(c78)
        end if


c-----------------------------------------------
c Find out the volume division over y axis
c (maximum 4 volumes are possible).
c Division is made by dividing G10 strips.
c-----------------------------------------------

        if (otdist(imod) .eq. 0.) then
           if ( (notcht(imod).eq.0 .and. notchb(imod).eq.0)
     @     .or. (notcht(imod).ne.0 .and. notchb(imod).ne.0) ) then
              c78 = 'Illegal: no pin distance,'//
     @              ' but both top and bottom channels'
              call meerro(c78)
           end if
        end if

        ndiv=1          ! ndiv will denote number of initialized divisions

        do ilay=1,notlay(imod)
           if (otdist(imod) .gt. 0.) then
              ydivul(ilay,ndiv) = 0.5*otfoil(imod)              ! normal module
           else if (notcht(imod).ne.0 .and. notchb(imod).eq.0) then
              ydivul(ilay,ndiv) = 0.5*otdisd - foipin            ! only top pin
           else if (notcht(imod).eq.0 .and. notchb(imod).ne.0) then
              ydivul(ilay,ndiv) = -0.5*otdisd + foipin + otfoil(imod) ! bottom pin
           end if
        end do

        do idiv=1,3
           ldiv = .false.
           do ilay=1,notlay(imod)
              if (ttg10p(ilay,idiv,imod).ne.0.) ldiv = .true.
           end do

           if (ldiv) then
              do ilay=1,notlay(imod)
                 ydivdl(ilay,ndiv) = ttg10p(ilay,idiv,imod)
              end do
              ndiv = ndiv+1
              do ilay=1,notlay(imod)
                 ydivul(ilay,ndiv) = ttg10p(ilay,idiv,imod)
              end do
           end if
        end do

        do ilay=1,notlay(imod)
           if (otdist(imod) .gt. 0.) then
              ydivdl(ilay,ndiv) = -0.5*otfoil(imod)             ! normal module
           else if (notcht(imod).eq.0 .and. notchb(imod).ne.0) then
              ydivdl(ilay,ndiv) = - 0.5*otdisd + foipin         ! bottom pin
           else if (notcht(imod).ne.0 .and. notchb(imod).eq.0) then
              ydivdl(ilay,ndiv) = 0.5*otdisd - foipin - otfoil(imod) ! top pin
           end if
        end do

c------------ in case of e-beam cutout we have additional vertical divisions.
        ebeam = .false.
        do ip=1,2
           do il=1,notlay(imod)
              if (otbeam(il,ip,imod) .ne. 0) ebeam = .true.
           end do
        end do

        ndiv0 = ndiv
        if (ebeam) then
           ndiv = ndiv+2
           do ilay=1,notlay(imod)
              ydivul(ilay,ndiv  ) = otbeam(ilay,2,imod)
              ydivdl(ilay,ndiv  ) = ydivdl(ilay,ndiv-2)

              ydivul(ilay,ndiv-1) = otbeam(ilay,1,imod)
              ydivdl(ilay,ndiv-1) = otbeam(ilay,2,imod)

              ydivdl(ilay,ndiv-2) = otbeam(ilay,1,imod)
           end do
        end if

c---------- Extract trailing empty monolayers - they will be declared as walls.
        jl1=100
        jl2=-1
        do jl=1,notlay(imod)
           nowires=.true.
           do is=1,nsec
              if (nttsew(jl,is,imod).ne.0) nowires=.false.
           end do
           if (.not.nowires) jl1=min(jl1,jl)
           if (.not.nowires) jl2=max(jl2,jl)
        end do
        
        if ((jl2-jl1).gt.3) then
           c78 = 'Module "'
           call ch78ac(modtype(:lenocc(modtype)))
           call ch78ac('" has too many monolayers:')
           call ch78ai(jl2-jl1+1)
           call meerro(c78)
        end if
        
        il1=jl1
        il2=jl2



        if (lhadec) then
           if (il2-il1.gt.2 .and.mod(iq(iqgesl_ltyp(isl)),10).le.2)
     @     iq(iqgesl_ltyp(isl)) = iq(iqgesl_ltyp(isl)) + 3 ! layer types/sequences
           lhadec = .false.
        end if

c----- Form volume index ITYP
c--1     10 mm single layer    sensitive
c--2      5 mm single layer    sensitive
c--3      5 mm single layer nonsensitive
c--4     10 mm double layer    sensitive
c--5      5 mm double layer    sensitive
c--6      5 mm double layer nonsensitive
c
c--7     10 mm single layer    sensitive in magnetic field
c--8      5 mm single layer    sensitive in magnetic field
c--9      5 mm single layer nonsensitive in magnetic field
c
c--10    10 mm double layer    sensitive in magnetic field
c--11     5 mm double layer    sensitive in magnetic field
c--12     5 mm double layer nonsensitive in magnetic field

        if (otdiam(imod).eq.1.) then
           ityp0 = 1            ! sensitive wires (10 mm)
        else if (otdiam(imod).eq.0.5) then

c          if (sense_wires) then
              ityp0 = 2         ! sensitive wires (5 mm)
c          else
c             ityp0 = 3         ! nonsensitive wires (5 mm)
c          end if

        else
           c78 = 'Wrong OTR cell diameter:'
           call ch78bf(otdiam(imod),-2)
           call ch78ac(' cm')
           call meerro (c78)
        end if

        if (notlay(imod).le.2) then
           laytyp = 0      ! single layers
        else
           laytyp = 1      ! double layers
        end if

        ityp = 6*imflag + 3*laytyp + ityp0
        if (ityp.le.0 .or. ityp.gt.12) then
           c78 = 'Illegal ITYP = '
           call ch78bi(ityp)
           call atstop(c78)
        end if

c------------------------------- declare sensitive detector volume
        pit1=otdiam(imod)*sqrt(3.)      ! ideal-geometry pitch

        do il0=1,4
           do iy =1,2
              d = abs(ttcela(il0,iy,imod)-ttcefi(il0,iy,imod))
              ncells = int(d/pit1 + 1.5)
              if (ncells.gt.1 .and. d.gt.0.) pit = d/(ncells-1) ! real pitch
           end do
        end do


c---------------------- loop on subvolumes (GEDE volumes)
        xmin_mo = 1.e10 ! big positive number
        xmax_mo =-xmin_mo
        ymin_mo = xmin_mo
        ymax_mo = xmax_mo
        zmin_mo = xmin_mo
        zmax_mo = xmax_mo
        mcells=0

        do idiv=1,ndiv          ! division on y

c idiv  - index for ydivul, ydivdl
c idiv0 - index for module type arrays

          idiv0 = min(idiv,ndiv0)
          call mmbook(jgede,1)

          igede = ngede()
          if (idiv.eq.1) igede1=igede

          if (iq(iqgede_fwl(igede1)).eq.0)
     @        iq(iqgede_fwl(igede1)) = ngwal() + 1
          if (iq(iqgesl_fwl(isl)).eq.0)
     @        iq(iqgesl_fwl(isl)) = ngwal() + 1

          call mmbrel(jgsgd,isl,igede)

          if (igesl_fpl(isl).eq.0) iq(iqgesl_fpl(isl)) = igede ! 1. plane 
          iq(iqgesl_npl(isl)) = igesl_npl(isl) + 1             ! # of planes

          iq(iqgede_cmp_l())=lgesl_wch  ! detector component
          iq(iqgede_lay_l())=ipl        ! layer number within the superlayer
          q(qgede_cos_l())= 1.  ! cos of the wire angle with respect to y-axis
          q(qgede_sin_l())= 0.

c copy 3x3 rotation matrix, transposing it
        jrot = lq(jrotm-irot)
        if (jrot.eq.0) call meerro ('OTRGEO: illegal rot.matrix!')
        q(qgede_rot(1,igede)) = q(jrot+1)
        q(qgede_rot(2,igede)) = q(jrot+4)
        q(qgede_rot(3,igede)) = q(jrot+7)
        q(qgede_rot(4,igede)) = q(jrot+2)
        q(qgede_rot(5,igede)) = q(jrot+5)
        q(qgede_rot(6,igede)) = q(jrot+8)
        q(qgede_rot(7,igede)) = q(jrot+3)
        q(qgede_rot(8,igede)) = q(jrot+6)
        q(qgede_rot(9,igede)) = q(jrot+9)


c-- Use some global file for this stuff? (or possibly local files also?)!
        q(qgede_acc_l()) = SIGWT(1)     ! measurement accuracy ! which index???
        q(qgede_eff_l()) = EFFWT        ! hit efficiency
        q(qgede_dod_l()) = 0.5*otdiam(imod) - 0.01 ! max drift distance

        iq(iqgede_nsl_l())=il2-il1+1    ! number of sublayers in a segment

        xmin_de = 1.e10 ! big positive number
        xmax_de =-xmin_de
        ymin_de = xmin_de
        ymax_de = xmax_de
        zmin_de = xmin_de
        zmax_de = xmax_de

        do ilay=1,4
           iq(iqgede_nwi(ilay,igede))=0
        end do
        ilay = 0

        do il0=il1,il2    ! loop on monolayers
           il = il0-il1+1

c Here il0 = 2..5 - index for Module type arrays,
c      il  = 1..4 - index for GEDE banks.

c These are coordinates of GEDE volume in rotated system

           ymin = yrot + ydivdl(il0,idiv)
           ymax = yrot + ydivul(il0,idiv)

           ymid = 0.5*(ymin + ymax)
           if (ymid.gt.0.) then
              iy=1
           else
              iy=2
           end if

           x1 = ttcefi(il0,iy,imod)
           x2 = ttcela(il0,iy,imod)
c xmin, xmax set the bottom and the upper cell centers of cell row
           xmin = xrot + x1
           xmax = xrot + x2
           dx(il) = x2-x1

           if (xmin.ge.xmax) then
              c78 = 'For idiv='
              call ch78ai(idiv)
              call ch78ac(', il0=')
              call ch78ai(il0)
              call ch78ac(' xmin =')
              call ch78bf(xmin,-3)
              call ch78ac(', xmax =')
              call ch78bf(xmax,-3)
              call me78wa
           end if

c These are exact borders for this monolayer
           zmin = z + 0.5*(il0-1)*otdiam(imod)
           zmax = z + 0.5*(il0+1)*otdiam(imod)

c##
           if (ttwifi(il0,iy,imod).ne.0.
     @   .and. fttsew(il0,idiv0,imod).ne.0) then

c Narrow modules at e-beam pipe place (they are without wires here!)
c e.g. for TC1, module tc-5d24-31
              if (idiv.eq.ndiv0+1
     @      .and. nttsew(il0,idiv0,imod).le.24) then
                 goto 20
              end if

              ilay = ilay + 1
              nwires = 1 + nttsew(il0,idiv0,imod)-fttsew(il0,idiv0,imod)
              iq(iqgede_nwi(ilay,igede)) = nwires

              if (nwires.le.0) then
                 c78 = 'Illegal nwires ='
                 call ch78bi(nwires)
                 call ch78bc('for igede =')
                 call ch78bi(igede)
                 call ch78ac(', ilay =')
                 call ch78bi(ilay)
                 call meerro (c78)
              end if

              q(qgede_xfw(ilay,igede)) = xrot + ttwifi(il0,iy,imod)
     @                               + (fttsew(il0,idiv0,imod)-1)*pit
              q(qgede_zfw(ilay,igede)) = z + 0.5*il0*otdiam(imod)

              q(qgede_y1w(ilay,igede)) = ymin
              q(qgede_y2w(ilay,igede)) = ymax
           end if
20         continue
c##
           d = abs(ttcela(il0,iy,imod) - ttcefi(il0,iy,imod))
           d = x2 - x1
           ncells = int(d/pit1 + 1.5)

           mcells = max(mcells,ncells)          ! not needed now?
           if (ncells.gt.1 .and. d.gt.0.) pit = d/(ncells-1)

           q(qgede_pit(igede))=pit      ! pitch


c=======================================================================
c The following borders are calculated in rotated (with module) frame:
c
c xmin    - monolayer
c xmin_de - GEDE volume
c sxmn_de - sensitive GEDE volume
c xmin_mo - module (several GEDEs)
c xmin_mo - Geant volume (for common module xmin_mo=xmin_ge)
c
c The following sizes are in laboratory frame:
c
c xmin_all - SL - only modules volumes (used for finding positions of walls)
c xmin_sl  - SL Geant volume - should contain everything including walls
c                 (got from database)
c xmin_box - empty box inside walls
c                 (calculated and used inside wall routines)
c=======================================================================

c Now xmin, xmax set the bottom and the upper conners of cell row
           xmin = xmin - pit/3.
           xmax = xmax + pit/3.

           xmin_de = min(xmin,xmin_de)
           xmax_de = max(xmax,xmax_de)
           ymin_de = min(ymin,ymin_de)
           ymax_de = max(ymax,ymax_de)
           zmin_de = min(zmin,zmin_de)
           zmax_de = max(zmax,zmax_de)

           sxmn_de = xmin_de
           sxmx_de = xmax_de
           symn_de = ymin_de + 0.5*ydg10
           symx_de = ymax_de - 0.5*ydg10

           if (symn_de .ge. symx_de) then
              c78 = 'Wrong sensitive volume borders on Y:'
              call me78sp('T')
              c78 = 'symn_de ='
              call ch78bf(symn_de,-4)
              call ch78ac(', symx_de = ')
              call ch78af(symx_de,-4)
              call meerro(c78)
           end if

           xmin_mo = min(xmin,xmin_mo)
           xmax_mo = max(xmax,xmax_mo)
           ymin_mo = min(ymin,ymin_mo)
           ymax_mo = max(ymax,ymax_mo)
           zmin_mo = min(zmin,zmin_mo)
           zmax_mo = max(zmax,zmax_mo)

c Now xmin, xmax set the bottom and top corners
c  of GEANT VOLUME (parallelepiped)!
           xmin = xmin - 2./3.*pit
           xmax = xmax + 2./3.*pit

c rot --> lab
           xmin_lab =  xmin * cosang + min(ymin * sinang, ymax * sinang)
           xmax_lab =  xmax * cosang + max(ymin * sinang, ymax * sinang)
           ymin_lab = - max(xmin * sinang, xmax * sinang) + ymin*cosang
           ymax_lab = - min(xmin * sinang, xmax * sinang) + ymax*cosang

           xmin_all = min(xmin_lab,xmin_all)
           xmax_all = max(xmax_lab,xmax_all)
           ymin_all = min(ymin_lab,ymin_all)
           ymax_all = max(ymax_lab,ymax_all)
           zmin_all = min(zmin,zmin_all)
           zmax_all = max(zmax,zmax_all)

           if (il0.lt.il2
     @   .and. ttcefi(il0  ,iy,imod) .ne. 0.
     @   .and. ttcefi(il0+1,iy,imod) .ne. 0.) then

              dfio = dfi
              dlao = dla

              dfi =
     @  nint((ttcefi(il0+1,iy,imod) - ttcefi(il0,iy,imod))/(0.5*pit))
              dla =
     @  nint((ttcela(il0+1,iy,imod) - ttcela(il0,iy,imod))/(0.5*pit))


              if (il0 .gt. il1) then
                 if (dfi.ne.dfio .or. dla.ne.dlao) then
                    write (*,*) ' === !!! non-standard geometry !!!',
     @                      ' module = ',modtype(:lenocc(modtype)),
     @                      ' idiv=',idiv,', il0=',il0,', iy=',iy
                     call meerro('non-standard geometry')
                 end if
              end if

              if (dfi.eq.dla) then
                 modshape = 'PARA'
                 if (dfi.lt.0.) then
                    angpar = -60.
                 else
                    angpar = +60.
                 end if
              else if (dfi.eq.-dla) then
                 modshape = 'TRD1'
              else
                 modshape = 'Unknown'
                 c78 = 'Unknown module shape: dfi ='
                 call ch78bf(dfi,5)
                 call ch78ac(', dla =')
                 call ch78bf(dla,5)
                 call meerro(c78)
              end if

           end if

        end do                  ! loop on monolayers

c detector shape: 1-box, 2-TRD1, 10-parallelepiped
c TO INTRODUCE: 42 - another parallelepiped
c               34 - another TRD1
        if (modshape(:4).eq.'PARA') then
           iq(iqgede_shp_l()) = 42
        else if (modshape(:4).eq.'TRD1') then
           iq(iqgede_shp_l()) = 2               ! actually should be 34
        else
           c78 = 'Unknown module shape for igede ='
           call ch78bi(igede)
           call meerro(c78)
        end if



        iq(iqgede_nsl_l()) = ilay

        if (int(geomvers).eq.96) then
           c78='**In this Geant version run 96 does not exist anymore**'
           call me78ap
        else if (int(geomvers).eq.97) then
           continue     ! take isec from the database           ! run 97
        else
                                                                ! run 98
c Rotate coordinates
           xp_ = 0.5*(xmin_de+xmax_de)
           yp_ = 0.5*(ymin_de+ymax_de)
           ynew = -sinang * xp_ + cosang * yp_  ! rot --> lab (only for y)
           isec = otrsect(isl, xp_, ynew)

c           isec = otrsect(isl, 0.5*(xmin_de+xmax_de),
c     @                         0.5*(ymin_de+ymax_de))
        end if


        iq(iqgede_sec_l()) = isec       ! sector number

        q(qgede_xmin_l()) = xmin_de - 2./3.*pit
        q(qgede_xmax_l()) = xmax_de + 2./3.*pit

        q(qgede_ymin_l()) = ymin_de
        q(qgede_ymax_l()) = ymax_de
        q(qgede_zmin_l()) = zmin_de
        q(qgede_zmax_l()) = zmax_de

        q(qgede_sxmn_l()) = sxmn_de     ! x of the left edge of the sensitive area
        q(qgede_sxmx_l()) = sxmx_de     ! right
        q(qgede_symn_l()) = symn_de     ! y left
        q(qgede_symx_l()) = symx_de     ! y right

c------------- Put reference for material (copied from CRGEDE.F)
        CALL UHTOC(wtdt(ityp),4,nam,4)
        ivo = iucomp (nam, iq(jvolum+1), iq(jvolum-1))
        if (ivo.eq.0)  call meerro('Nonvalid volume'//nam)
        jvo = lq(jvolum-ivo)
        if (jvo.eq.0) then
           c78 = 'jvo = 0 for wtdt = '//nam//', ivo = '
           call ch78bi(ivo)
           call me78er
        endif

c Only first GEDE bank carries the GEANT name (used in digitization)
        if (igede.eq.igede1 .or. idiv.gt.idiv0) then
           iq(hqgede_gvol_l()) = wtdt(ityp)
        else
           call uctoh('    ', iq(hqgede_gvol_l()), 4,4)
        end if

        numed = q(jvo+4)                ! numed  The same as tracking medium?
        jtm = iq(jtmed-numed-8)         ! jtm = lq(jtmed-numed)
        if (jtm.eq.0) then
           c78 = 'Illegal jvo ='
           call ch78bi(jtm)
           call me78er
        endif

        nmat = q(jtm+6)                 ! material # in Geant
        call uhtoc(q(iq(jmate-nmat-8)+1),4,matnam,20)

        imat = matnum(matnam(1:lenocc(matnam))) ! material # in Arte
        call mmbrel(jgmgd,imat,igede)

c Multiple scattering angle for 1 GeV particles
        xms_tmp = fgmat_radl(imat)
        if (xms_tmp.eq.0.) CALL MEERRO('OTRGEO: Wrong XMS')
        xmsc = (zmax_de-zmin_de)/xms_tmp
        q(qgede_ms_l()) = xmsc*(0.0136*(1.+0.038*log(xmsc)))**2

       iq(iqgede_nwl_l())=0     ! number of walls connected to this detector
       iq(iqgede_fwl_l())=0     ! first wall connected to this detector

c Fill angle of parallelepiped (fgede_xmad) in degrees
       if (modshape .eq. 'PARA') then
          q(qgede_xmad_l())= tan(degrad*angpar)   ! angle of PARA: +-60 degrees
       else if (modshape .eq. 'TRD1') then
          q(qgede_xmad_l())= 0.
       end if

c Fill angle of GEDE bank borders: upper (fgede_ymad) and lower (fgede_zmad).
c Unit is degrees.
       q(qgede_ymad_l()) = 0.
       q(qgede_zmad_l()) = 0.
       if (fltmod(imod).eq.2) then
          if (idiv.gt.1 .and. idiv.le.ndiv0) 
     @       q(qgede_ymad_l()) = tan(degrad*angle)
          if (idiv.lt.ndiv0) 
     @       q(qgede_zmad_l()) = tan(degrad*angle)
       end if


        end do          ! division on y

c========================================= End of GEDE volume declaration


c========================================= Geant volume declaration
c---- Create Geant volume containing the whole module (several GEDE volumes)

        do idiv=ndiv0,ndiv      ! loop on vertical Geant volumes
                                ! above e-beam, near e-beam, under e-beam
           ymin_ge = ymin_mo
           ymax_ge = ymax_mo

           if (ndiv.ne.ndiv0) then
              if (idiv.eq.ndiv0) then
                 ymin_ge = +1000.
                 do ilay=1,notlay(imod)
                    if (otbeam(ilay,1,imod).ne.0.)
     @                 ymin_ge = min(yrot+otbeam(ilay,1,imod), ymin_ge)
                 end do
              else if (idiv.eq.ndiv0+1) then
                 ymin = -1000.
                 ymax = +1000.
                 do ilay=1,notlay(imod)
                    if (otbeam(ilay,1,imod).ne.0.)
     @                 ymax_ge = min(yrot+otbeam(ilay,1,imod), ymax_ge)
                    if (otbeam(ilay,2,imod).ne.0.)
     @                 ymin_ge = max(yrot+otbeam(ilay,2,imod), ymin_ge)
                 end do
              else if (idiv.eq.ndiv0+2) then
                 ymax_ge = -1000.
                 do ilay=1,notlay(imod)
                    if (otbeam(ilay,2,imod).ne.0.)
     @                 ymax_ge = max(yrot+otbeam(ilay,2,imod), ymax_ge)
                 end do
              else
                 call meerro('Something wrong with ndiv0')
              end if
           end if

           do iz=1,3    ! loop on horizontal (along Z) Geant volume:
                        ! wall, sensitive volume, wall

              if (iz.eq.1 .and. il1.eq.1) goto 200
              if (iz.eq.3 .and. il2.eq.notlay(imod)) goto 200

              mcells_ge = 0
              if (idiv.eq.ndiv0+1 .and. mcells .le. 24) goto 200

              x1_ = 1.e10
              x2_ = -1.e10
              if (iz.eq.1) then
                 ilay1=1
                 ilay2=il1-1
              else if (iz.eq.2) then
                 ilay1=il1
                 ilay2=il2
              else if (iz.eq.3) then
                 ilay1=il2+1
                 ilay2=notlay(imod)
              end if

         if (ilay1.gt.ilay2)
     @    write (*,*) '=========illegal ilay1,ilay2=',ilay1,ilay2

              do il0=ilay1,ilay2
                 if (ymax_ge .ge. 0.) then
                    iy1 = 1
                 else
                    iy1 = 2
                 end if
                 if (ymin_ge .le. 0.) then
                    iy2 = 2
                 else
                    iy2 = 1
                 end if
         if (iy1.gt.iy2) write (*,*) '=========illegal iy1,iy2=',iy1,iy2

                 x1 = +10000.
                 x2 = -10000.
                 do iy=iy1,iy2
                    if (ttcefi(il0,iy,imod).ne.0.) then
                       x1 = min(ttcefi(il0,iy,imod),x1)
                       if (idiv.eq.ndiv0+1)
     @                    x1 = max(ttcefi(il0,iy,imod)+15.,x1)
                    end if
                    if (ttcela(il0,iy,imod).ne.0.) then
                       x2 = max(ttcela(il0,iy,imod),x2)
                    end if

                    x1_ = min(x1,x1_)
                    x2_ = max(x2,x2_)
                 end do


                 dx(il0) = x2-x1
                 d = x2 - x1
                 ncells = int(d/pit1 + 1.5)
                 mcells_ge = max(mcells_ge,ncells)
              end do
          if(x1_.ge.x2_) write (*,*) '=========illegal x1_,x2_=',x1_,x2_
          if (mcells_ge.le.0) write (*,*) '=========mcells_ge',mcells_ge
              xmin_ge = xrot + x1_
              xmax_ge = xrot + x2_

              dx1 = dx(ilay1)
              dx2 = dx(ilay2)

              nvol = igede1 + (idiv-ndiv0)
              if (iz.ne.2) nvol = ngwal()+1

              zmin_ge = z + 0.5*(ilay1-1)*otdiam(imod)
              zmax_ge = z + 0.5*(ilay2+1)*otdiam(imod)
              if (iz.eq.1) then
                 zmax_ge = zmax_ge - 0.5*otdiam(imod)
                 if (modshape(:4).eq.'TRD1') then
                    dx2 = dx2 + pit
                 else
                    if (angpar.lt.0.) then
                       xmin_ge = xmin_ge + 0.5*pit
                    else
                       xmax_ge = xmax_ge - 0.5*pit
                    end if
                 end if


              else if (iz.eq.3) then
                 zmin_ge = zmin_ge + 0.5*otdiam(imod)
                 if (modshape(:4).eq.'TRD1') then
                    dx1 = dx1 - pit
                 else
                    if (angpar.gt.0.) then
                       xmin_ge = xmin_ge + 0.5*pit
                    else
                       xmax_ge = xmax_ge - 0.5*pit
                    end if
                 end if
              end if

              xp_rot = 0.5*(xmax_ge + xmin_ge)
              yp_rot = 0.5*(ymax_ge + ymin_ge)

              zp_mo  = 0.5*(zmax_ge + zmin_ge)
              zp = zp_mo - zp_sl

              if (iz.eq.2) then
                 ityp_g = 6*imflag + 3*laytyp + ityp0  ! sensitive: 10 & 5 mm
              else
                 ityp_g = 6*imflag + 3*laytyp + 3      ! nonsensitive: 5 mm
              end if

              if (modshape(:4).eq.'PARA') then !shape = PARA = parallelepiped
                 npar = 6
                 par(1) = 0.5*(dx1 + pit)
                 par(2) = 0.5*(ymax_ge - ymin_ge)
                 par(3) = 0.5*(zmax_ge - zmin_ge)
                 par(4) = 0.
                 par(5) = angpar
                 par(6) = 0.
                 do i=1,3
                    if (par(i).le.0.) write (*,*)
     @                ' === PARA negative par(',i,') =',par(i)
                 end do
                 ityp_ge = ityp_g

              else if (modshape(:4).eq.'TRD1') then ! shape = TRD1 = trapezoid 1
                 npar = 4
                 par(1) = 0.5*dx1
                 par(2) = 0.5*dx2
                 if ((par(2)-par(1)) .gt. 0.1*pit) then
                    par(2) = par(2) + pit
                 else
                    par(1) = par(1) + pit
                 end if
                 par(3) = 0.5*(ymax_ge - ymin_ge)
                 par(4) = 0.5*(zmax_ge - zmin_ge)
                 do i=1,4
                    if (par(i).le.0.) write (*,*)
     @                ' === TRD1 negative par(',i,') =',par(i)
                 end do

                 ityp_ge = ityp_g + 12
              end if

c rot --> lab
              xp_mo =  cosang * xp_rot + sinang * yp_rot
              yp_mo = -sinang * xp_rot + cosang * yp_rot
              xp = xp_mo - xp_sl
              yp = yp_mo - yp_sl


              call gsposp(wtdt(ityp_ge),nvol,slname,xp,yp,zp,
     @              ir,'ONLY',par,npar)

              if (iz.ne.2) then
                 call crgwal(wtdt(ityp_ge),ngwal()+1)
              end if


200           continue
           end do
        end do

c------------------------------------- Install the dividing G10-strips
        if (ipl.le.0 .or. ipl.gt.maxstereo) then
           c78 = 'Illegal number of stereo layer ='
           call ch78bi(ipl)
           call meerro(c78)
        end if
        if (zstereo(ipl).eq.0.) then
           c78 = 'Illegal zstereo('
           call ch78ai(ipl)
           call ch78ac(') =')
           call ch78bf(zstereo(ipl),-3)
           call meerro(c78)
        end if

        if (int(geomvers).ne.97) then
           dyg10 = 1.0
        else
           dyg10 = 0.6
        end if

        do il = 1,notlay(imod)          ! monolayers
           do idiv = 1,3                ! vertical divisions
              par(2) = 0.5 * dyg10
              par(3) = 0.5 * 0.01

              if (ttg10p(il,idiv,imod) .ne. 0.) then

                 if ((yrot + ttg10p(il,idiv,imod)) .gt. 0.) then
                    iy = 1
                 else
                    iy = 2
                 end if

c##
                 if (ttcefi(il,iy,imod).eq.0.) then
                    c78 = 'For '//modtype(:lenocc(modtype))
                    call ch78ac(': ttcefi = 0. for il=')
                    call ch78ai(il)
                    call ch78ac(', iy=')
                    call ch78ai(iy)
                    call me78wa
                 end if

                 iy1 = iy
                 iy2 = iy
                 if (idiv.eq.2) then
                    iy1 = 1
                    iy2 = 2
                 end if

                 xmin = xrot + ttcefi(il,iy,imod) - pit/3.
                 xmax = xrot + ttcela(il,iy,imod) + pit/3.
                 do iy=iy1,iy2
                    xmin = min(xmin, xrot + ttcefi(il,iy,imod) - pit/3.)
                    xmax = max(xmax, xrot + ttcela(il,iy,imod) + pit/3.)
                 end do

                 par(1) = 0.5 * (xmax-xmin)

                 xp = 0.5 * (xmax + xmin)
                 yp = yrot + ttg10p(il,idiv,imod)

                 zmin = z + 0.5*il*otdiam(imod)
                 if (zstereo(ipl).ne.0.) 
     @              zmin = zstereo(ipl) - 0.01/0.25 *(zmin-zstereo(ipl))

                 xp_ =  cosang * xp + sinang * yp - xp_sl
                 yp_ = -sinang * xp + cosang * yp - yp_sl
                 zp = zmin      - zp_sl

                 if (fltmod(imod).eq.1) then
                    ir_g10 = 0          ! div. G10 are along lab. x-axis
                 else if (fltmod(imod).eq.2) then
                    ir_g10 = ir         ! div. G10 are along rot. x-axis
                 else
                    call meerro('Illegal fltmod')
                 end if

                 call gsposp(matg10(imflag+1),ngwal()+1,slname(:4),
     @                 xp_,yp_,zp,ir_g10,'ONLY',par,3)
                 call crgwal(matg10(imflag+1),ngwal()+1)

c Put wall for last GEDE bank
                 iq(iqgede_nwl(igede1)) =
     @                              ngwal() - iq(iqgede_fwl(igede1)) + 1
                 iq(iqgesl_nwl(isl)) = ngwal() - iq(iqgesl_fwl(isl)) + 1


              end if
           end do                       ! vertical divisions

c------------------------------------- Install e-beam G10-strips
           do idiv = 1,2                ! vertical divisions
              par(2) = 0.5 * dyg10
              par(3) = 0.5 * 0.01

              if (otbeam(il,idiv,imod) .ne. 0.) then

                 if ((yrot + otbeam(il,idiv,imod)) .gt. 0.) then
                    iy = 1
                 else
                    iy = 2
                 end if

                 if (ttcefi(il,iy,imod).eq.0.) then
                    c78 = 'For '//modtype(:lenocc(modtype))
                    call ch78ac(': ttcefi = 0. for il=')
                    call ch78ai(il)
                    call ch78ac(', iy=')
                    call ch78ai(iy)
                    call me78wa
                 end if

                 xmin = xrot + ttcefi(il,iy,imod) - pit/3.
                 xmax = xrot + ttcela(il,iy,imod) + pit/3.

                 par(1) = 0.5 * (xmax-xmin)

                 xp = 0.5 * (xmax + xmin)
                 yp = yrot + otbeam(il,idiv,imod)

                 zmin = z + 0.5*il*otdiam(imod)
                 if (zstereo(ipl).ne.0.) 
     @              zmin = zstereo(ipl) - 0.01/0.25 *(zmin-zstereo(ipl))

                 xp_ =  cosang * xp + sinang * yp - xp_sl
                 yp_ = -sinang * xp + cosang * yp - yp_sl
                 zp = zmin      - zp_sl

                 ir_g10 = ir

                 call gsposp(matg10(imflag+1),ngwal()+1,slname(:4),
     @                 xp_,yp_,zp,ir_g10,'ONLY',par,3)
                 call crgwal(matg10(imflag+1),ngwal()+1)
c--------

c Put wall for last GEDE bank
                 iq(iqgede_nwl(igede1)) =
     @                              ngwal() - iq(iqgede_fwl(igede1)) + 1
                 iq(iqgesl_nwl(isl)) = ngwal() - iq(iqgesl_fwl(isl)) + 1


              end if
           end do                       ! vertical divisions

c------------------------------------- Install the supporting G10-strips
           do idiv = 1,6                ! vertical divisions
              par(2) = 0.5 * dyg10
              par(3) = 0.5 * 0.01

              if (ttsupp(il,idiv,imod) .ne. 0.) then

                 if ((yrot + ttsupp(il,idiv,imod)) .gt. 0.) then
                    iy = 1
                 else
                    iy = 2
                 end if

c##
                 if (ttcefi(il,iy,imod).eq.0.) then
                    c78 = 'For '//modtype(:lenocc(modtype))
                    call ch78ac(': ttcefi = 0. for il=')
                    call ch78ai(il)
                    call ch78ac(', iy=')
                    call ch78ai(iy)
                    call me78wa
                 end if

                 xmin = xrot + ttcefi(il,iy,imod) - pit/3.
                 xmax = xrot + ttcela(il,iy,imod) + pit/3.

                 par(1) = 0.5 * (xmax-xmin)

                 xp = 0.5 * (xmax + xmin)
                 yp = yrot + ttsupp(il,idiv,imod)

                 zmin = z + 0.5*il*otdiam(imod)
                 if (zstereo(ipl).ne.0.) 
     @              zmin = zstereo(ipl) - 0.01/0.25 *(zmin-zstereo(ipl))

                 xp_ =  cosang * xp + sinang * yp - xp_sl
                 yp_ = -sinang * xp + cosang * yp - yp_sl
                 zp = zmin      - zp_sl

                 call gsposp(matg10(imflag+1),ngwal()+1,slname(:4),
     @                 xp_,yp_,zp,ir,'ONLY',par,3)
                 call crgwal(matg10(imflag+1),ngwal()+1)

c Put wall for last GEDE bank
                 iq(iqgede_nwl(igede1)) =
     @                              ngwal() - iq(iqgede_fwl(igede1)) + 1
                 iq(iqgesl_nwl(isl)) = ngwal() - iq(iqgesl_fwl(isl)) + 1


              end if

           end do                       ! vertical divisions
        end do                          ! monolayers

c========================================= End of Geant volume declaration

c------------------------------------------------ Command was not found above.
        else
           c78 = 'Unknown command: '//buf(:nlast)
           call meerro(c78)
        end if
        goto 100

99      call meerro
     @     ('Error reading parameters in command: '//buf(:nlast))


c================================================ end of reading loop

c------------------- Here at end of file: Perform final checks, etc.

991     continue
        call ioclos(lread)
992     continue

c------------------------ Create walls (and assign them to the last GEDE bank)
        if (cosmic) then
           c78='Walls for cosmic tests defined'
           call me78ap
        else if (int(geomvers).eq.97) then
           if (slname(:2) .eq. 'PC') then
              call otwl96(isl)          ! Zeuthen chamber walls
           else if (slname(:2) .eq. 'MC') then
              call otwd96(isl)          ! Dubna chamber walls
           else
              call meerro('?? No walls are defined for this superlayer')
           end if
        else
           call otwall(isl)          ! run 98
        end if

        iq(iqgede_nwl(igede1)) = ngwal() - iq(iqgede_fwl(igede1)) + 1
        iq(iqgesl_nwl(isl)) = ngwal() - iq(iqgesl_fwl(isl)) + 1

c Comparison of coordinates
        if (xmin_sl.gt.xmin_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' xmin_sl = ')
           call ch78af(xmin_sl,-4)
           call ch78ac(' xmin_all = ')
           call ch78af(xmin_all,-4)
           call me78wa
        end if
        if (xmax_sl.lt.xmax_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' xmax_sl = ')
           call ch78af(xmax_sl,-4)
           call ch78ac(' xmax_all = ')
           call ch78af(xmax_all,-4)
           call me78wa
        end if

        if (ymin_sl.gt.ymin_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' ymin_sl = ')
           call ch78af(ymin_sl,-4)
           call ch78ac(' ymin_all = ')
           call ch78af(ymin_all,-4)
           call me78wa
        end if
        if (ymax_sl.lt.ymax_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' ymax_sl = ')
           call ch78af(ymax_sl,-4)
           call ch78ac(' ymax_all = ')
           call ch78af(ymax_all,-4)
           call me78wa
        end if

        if (zmin_sl.gt.zmin_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' zmin_sl = ')
           call ch78af(zmin_sl,-4)
           call ch78ac(' xmin_all = ')
           call ch78af(zmin_all,-4)
           call me78wa
        end if
        if (zmax_sl.lt.zmax_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' zmax_sl = ')
           call ch78af(zmax_sl,-4)
           call ch78ac(' zmax_all = ')
           call ch78af(zmax_all,-4)
           call me78wa
        end if

        iq(iqgesl_lay(isl)) = ipl ! number of layers

c reverse digits in a decimal number
        ltyp_new = 0
        ltyp_old = igesl_ltyp(isl)
        do i=1,ipl
           l = mod(ltyp_old,10)
           ltyp_old = ltyp_old / 10
           ltyp_new = 10*ltyp_new + l
        end do
        iq(iqgesl_ltyp(isl)) = ltyp_new


        return

997     c78 = 'Error opening file "'//cnamfil(:lenocc(cnamfil))//'"'
        call meerro(c78)
        goto 992

c Here to stop after error
998     call meerro ('Error in OTRSL')
        end
