        integer function otrsect(isl,x,y)
******************************************************************
*
* Returns sector number for OTR
* based on the supplied coordinates
* of the center of the module or GEDE volume.
*
* Parameters:
*   isl  - number of this superlayer
*   x,y  - coordinates of module center
*
* Author: Alexander Lanyov, May 1997
*
******************************************************************
*
* These data have been taken from TDR (DESY PRC 95-01), p.236, Table 61
* "Superlayer section parameters in the HERA-B Outer Tracking System".
*
* These data are used for automatic assignment of sector numbers
* while actual module information will be fetched from OTR database.
*
******************************************************************
        parameter (maxsl=9+4+2)
        parameter (npar=6)
        parameter (ixr=1, ixi=2, ixm=3, ixo=4, iyr=5, iyo=6)

#include "arte/Arte.fi"
#include "auto/GEO.fi"

	integer iread
	character*4 slname

        integer nsl
        real par(npar,maxsl),x,y

#include "arte/Arte.ff"
#include "auto/GEO.ff"

        data par/
     @
     @! xr   xi    xm   xo    yr    yo
     @
     @  18., 35.,  55.,  57., 18.,  37.,        ! MC1
     @  18., 35.,  55.,  70., 18.,  45.,        ! MC2
     @  18., 40.,  60.,  85., 18.,  54.,        ! MC3
     @  18., 40.,  60., 100., 18.,  64.,        ! MC4
     @  18., 45.,  70., 115., 18.,  73.,        ! MC5
     @  18., 45.,  70., 130., 18.,  83.,        ! MC6
     @  18., 50.,  80., 145., 18.,  93.,        ! MC7
     @  18., 50.,  80., 160., 18., 102.,        ! MC8
     @  18., 53.,  90., 174., 18., 112.,        ! MC9
     @
     @  18., 56., 100., 190., 18., 122.,        ! PC1
     @  18., 60., 108., 197., 18., 126.,        ! PC2
     @  18., 82., 116., 204., 18., 131.,        ! PC3
     @  18., 85., 124., 213., 18., 137.,        ! PC4
     @
     @  22., 85., 160., 263., 20., 191.,        ! TC1
     @  22., 92., 166., 291., 20., 212./        ! TC2

        if (isl.le.0 .or. isl.gt.ngesl()) then
	   c78 = 'Illegal SL number: isl ='
	   call ch78bi(isl)
	   call meerro(c78)
           otrsect = -1
           return
        end if

	call uhtoc(iq(hqgesl_name(isl)),4,slname,4)
	nsl = iread(slname(3:4))
	if (slname(:2).eq.'MC') then
	   continue
	else if (slname(:2).eq.'PC') then
	   nsl = nsl+9
	else if (slname(:2).eq.'TC') then
	   nsl = nsl+13
	else
	   call meerro('Illegal SL name = ' // slname)
           otrsect = -1
	   return
	end if

        if (nsl.le.0 .or. nsl.gt.maxsl) then
	   c78 = 'Illegal OTR SL number: nsl ='
	   call ch78bi(nsl)
	   call meerro(c78)
           otrsect = -1
           return
        end if

        if (x .ge. par(ixm,nsl)) then
           otrsect = 11
        else if (x .le. -par(ixm,nsl)) then
           otrsect = 12
        else if (x .ge. par(ixi,nsl)) then
           if (y .ge. 0.) then
              otrsect = 7
           else
              otrsect = 8
           end if
        else if (x .le. -par(ixi,nsl)) then
           if (y .ge. 0.) then
              otrsect = 9
           else
              otrsect = 10
           end if
        else if (y .ge. par(iyr,nsl)) then
           otrsect = 1
        else if (y .le. -par(iyr,nsl)) then
           otrsect = 2
        else if (x .ge. 0.) then
           if (y .ge. 0.) then
              otrsect = 3
           else
              otrsect = 4
           end if
        else if (x .le. 0.) then
           if (y .ge. 0.) then
              otrsect = 5
           else
              otrsect = 6
           end if
        else
           otrsect = -1
        end if

        return
        end
