      subroutine  otwd96(isl)
******************************************************************
*
* Defines walls for Dubna OTR superlayer MC01 for run 96/97.
* Geant volumes are defined and the corresponding Arte tables.
*
* Called from: otrsl.F
*
* The definition of walls was copied
* from wtdg96.F on 5-Mar-1997 (written by S.Nowak),
* and then adapted for calls from otrsl.F routine
* (some errors were also corrected).
*
* External references:
*
* Arguments  in/out       Description
*
* isl        in           superlayer number (index for GESL table)
*
* Alexander Lanyov     May 1997
*
************************************************************************
      implicit none

#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGBPIP.fi"
#include "gean/ZGOTR.fi"
#include "gean/ZGDETV.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"

#include "gean/otrgeo.fi"

      integer ifirst,isl,nsl,igede,i,irotm(2)
      real par(8),xp,yp,zp,wm,zmin,zmax,xdim,ydim
      character*4 csl
      save ifirst
ccc      integer nwi(2) /32,14/,ng10(2)/6,2/
c     Dimension of cable frame
c     Dimension of gas frame
      real dxgb,dygb,dzgb
c shift of gas frame relative to superlayer
      real xsh,ysh,zsh
c b) alu plate
      real dxalu,dyalu,dzalu
c c) alu u-profile 80*45*6 beam off side
      real dxalup, dyalup,dzalup
c d) rohacell plate
      real dxroh,dyroh,dzroh
c d) rohacell plate
      real dxnom,dynom,dznom
c d) space between rohacell wall and modules
      real dxplex, dyplex, dzplex
c e) g10-strips (size and y-position)
ccc      real dyg10 /.6/, dzg10 /.01/


ccc      real yg10(8) /18.5,17.5,.5,-0.5,17.5,-18.5,.5,-0.5/

c inefficient region around g10 strips
ccc      real ydg10 /1.0/

        real x,y,z
        real xdiff,ydiff,zdiff
        real xmin_box,xmax_box,ymin_box,ymax_box,zmin_box,zmax_box

c dimension of modules
cccc      data dymod/95.2/, dzmod/1.25/, dztyp/0.5/

c dimension of modules
	data xdim/29.5/, ydim/65./

      data ifirst/0/

      data dxgb /44./, dygb /100./, dzgb /9./
c shift of gas frame relative to superlayer
      data xsh /7.5/ ,ysh /0.0/
c b) alu plate
      data dxalu /70./, dyalu /134./, dzalu /.1/
c c) alu u-profile 80*45*6 beam off side
      data dxalup/9./, dyalup/ .85/, dzalup/.15/
c d) rohacell plate
      data dxroh/70./,dyroh/100./,dzroh /.4/
c d) rohacell plate
      data dxnom/41.2/,dynom/99./,dznom /2./
c d) space between rohacell wall and modules
      data dxplex/70./, dyplex/15./, dzplex /0.525/
c e) g10-strips (size and y-position)
ccc      data dyg10 /.6/, dzg10 /.01/

#include "arte/Arte.ff"
#include "auto/GEO.ff"

c------------------ Positions of empty box to put the chambers

c wrong!
c       xmin_box = xsh -0.5*dxnom + 1.          ! WTRM
c       xmax_box = xsh +0.5*dxnom - 1.

        xmin_box = xsh -0.5*dxgb + 4.5 +1.25    ! WTBM 4.5 --> 3. ???
        xmax_box = xsh +0.5*dxgb - 4.5 -1.25

        ymax_box = 0.5*dygb - dyalup            ! WTAM
        ymax_box = min(ymax_box, ysh + 0.5*dygb - 2*0.25)! WTAM
        ymin_box = -ymax_box

        zmin_box = zsh -0.5*dzgb + dznom        ! WTNM
        zmax_box = zsh +0.5*dzgb - dznom        ! WTNM

c------------------ Center the chambers in the empty box

        x = 0.5*(xmin_all+xmax_all) - 0.5*(xmin_box+xmax_box) - xp_sl
        y = 0.5*(ymin_all+ymax_all) - 0.5*(ymin_box+ymax_box) - yp_sl
        z = 0.5*(zmin_all+zmax_all) - 0.5*(zmin_box+zmax_box) - zp_sl

        if (iverbose.ge.2) then
           write (c78,48) 'BOX ',
     @        xmin_box,xmax_box,ymin_box,ymax_box,zmin_box,zmax_box
           call me78sp('T')
           write (c78,48) 'BOXf',
     @ xmin_box+x,xmax_box+x,ymin_box+y,ymax_box+y,zmin_box+z,zmax_box+z
           call me78sp('T')
           write (c78,48) 'ALL ',
     @        xmin_all,xmax_all,ymin_all,ymax_all,zmin_all,zmax_all
           call me78sp('T')
           write (c78,48) 'SL  ',
     @        xmin_sl,xmax_sl,ymin_sl,ymax_sl,zmin_sl,zmax_sl
           call me78sp('T')

48         format(' OTWD96: ',a,'= ',3('[',f8.2,',',f8.2,'], '))

           write (c78,'('' OTWD96: offsets are'',3f8.2)') x,y,z
           call me78sp('T')
        end if


c------------------ Compare do the chambers fit?
        xdiff = (xmax_box - xmin_box) - (xmax_all - xmin_all)
        ydiff = (ymax_box - ymin_box) - (ymax_all - ymin_all)
        zdiff = (zmax_box - zmin_box) - (zmax_all - zmin_all)

        if (xdiff.lt.0.) then
           c78='OTWD96: Modules do not fit into on X axis: difference ='
           call ch78bf(xdiff,-2)
           call me78sp('T')
        end if
        if (ydiff.lt.0.) then
           c78='OTWD96: Modules do not fit into on Y axis: difference ='
           call ch78bf(ydiff,-2)
           call me78sp('T')
        end if
        if (zdiff.lt.0.) then
           c78='OTWD96: Modules do not fit into on Z axis: difference ='
           call ch78bf(zdiff,-2)
           call me78sp('T')
        end if

c------------------

      if (ifirst.eq.0) then
         do i=1,2
            wm = float(i)*90.
            call crrotm( irotm(i),90.,wm, 180.,0.,90.,amod(wm+90.,360.))
         end do
         ifirst = 1
      end if

c-- install the superlayer volume
      currsl = isl
      call uhtoc(hgesl_name(isl),4,csl,4)
ccccc      call cggesl(csl,'MAGN')
ccccc      call gsatt(csl,'COLO',-5)

      nsl = igesl_nsl(isl)


        if (ichwa.eq.1) goto 100  ! in this case install only chamber walls


c install the front alu plate of the gas box

c u-profile 90*10*1.5  near chamber
c vertical
      par(1) =  0.5*dyalup
      par(2) =  0.5*dygb
      par(3) =  0.5*dzalup
      xp =  -17.5 + dzalup + par(1)
      yp =   0.
      zp =   -0.5*dzgb + par(3) !
      call crpos('WTAM',csl,0,0,xp+x,yp+y, zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+x,yp+y,-zp+z,0,'ONLY',par)

      par(1) = 0.5*dzalup
      par(3) = 0.5*dxalup
      xp = -17.5 + par(1)
      zp =   0.
      call crpos('WTAM',csl,0,0,xp+x,yp+y, zp+z,0,'ONLY',par)

c horizontal endpieces
      par(1) =  1.
      par(2) =  0.5*dyalup
      par(3) =  0.5*dzalup
      xp = xsh - 0.5*dxgb - par(1)
      yp = 0.5*dygb - par(2)
      zp =   -0.5*dzgb + par(3) !
      call crpos('WTAM',csl,0,0,xp+x, yp+y, zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+x, yp+y,-zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+x,-yp+y, zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+x,-yp+y,-zp+z,0,'ONLY',par)

      par(1) = 3.
      par(2) = 0.5*dzalup
      par(3) = 0.5*dxalup
      xp =   xsh - 0.5*dxgb
      yp =   0.5*dygb + par(2)
      zp =   0.
      call crpos('WTAM',csl,0,0,xp+x, yp+y,zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+x,-yp+y,zp+z,0,'ONLY',par)

c u-profile 90*10*1.5  far away
c vertical
      par(1) =  0.5*dyalup
      par(2) =  24.5
      par(3) =  0.5*dzalup

c      xp =  0.5*(xiwt(nsl)-xowt(nsl)) - dzalup -par(1)
c      yp =   yowt(nsl) - par(2) - 1.
      xp = -xdim - dzalup -par(1)
      yp =  ydim - par(2) - 1.

      zp =   -0.5*dzgb + par(3) !
      call crpos('WTAM',csl,0,0,xp+x, yp+y, zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+x, yp+y,-zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+x,-yp+y, zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+x,-yp+y,-zp+z,0,'ONLY',par)

      par(1) = 0.5*dzalup
      par(3) = 0.5*dxalup

c      xp = 0.5*(xiwt(nsl)-xowt(nsl)) -par(1)
      xp = -xdim - par(1)

      zp =   0.
      call crpos('WTAM',csl,0,0,xp+x, yp+y,zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+x,-yp+y,zp+z,0,'ONLY',par)

c horizontal

c      par(1) = 0.5 * (xowt(nsl)-xiwt(nsl))
      par(1) = xdim

      par(2) = 0.5*dyalup
      par(3) = 0.5*dzalup
      xp = 0.

c      yp = yowt(nsl) - dzalup - par(3)
      yp = ydim - dzalup - par(3)

      zp =   -0.5*dzgb + par(3) !
      call crpos('WTAM',csl,0,0,xp+x, yp+y, zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+x, yp+y,-zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+x, yp+y, zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+x,-yp+y,-zp+z,0,'ONLY',par)

      par(2) =0.5*dzalup
      par(3) =0.5*dxalup
      yp =   ydim - par(2)
      zp =   0.
      call crpos('WTAM',csl,0,0,xp+x, yp+y,zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+x,-yp+y,zp+z,0,'ONLY',par)

c distance holders
      par(1) = 0.
      par(2) =0.5
      par(3) = 7.
      xp =0.5*(dxgb - 1.5)
      yp =0.5*dygb + par(3)
      zp =0.5*(dzgb - dznom) !


c     if (iverbose.ge.3) write (*,*) 'WTBM xp=',xp,' xsh=',xsh,' x=',x,
c    #               '  yp=',yp,' ysh=',ysh,' y=',y,
c    #               '  zp=',zp,' z=',z

      call crpos('WTBM',csl,0,0, xp+xsh+x, yp+ysh+y, zp+z,irotm(2),
     @                                                      'ONLY',par)
      call crpos('WTBM',csl,0,0, xp+xsh+x, yp+ysh+y,-zp+z,irotm(2),
     @                                                      'ONLY',par)
      call crpos('WTBM',csl,0,0, xp+xsh+x,-yp+ysh+y, zp+z,irotm(2),
     @                                                      'ONLY',par)
      call crpos('WTBM',csl,0,0, xp+xsh+x,-yp+ysh+y,-zp+z,irotm(2),
     @                                                      'ONLY',par)
      call crpos('WTBM',csl,0,0,-xp+xsh+x, yp+ysh+y, zp+z,irotm(2),
     @                                                      'ONLY',par)
      call crpos('WTBM',csl,0,0,-xp+xsh+x, yp+ysh+y,-zp+z,irotm(2),
     @                                                      'ONLY',par)
      call crpos('WTBM',csl,0,0,-xp+xsh+x,-yp+ysh+y, zp+z,irotm(2),
     @                                                      'ONLY',par)
      call crpos('WTBM',csl,0,0,-xp+xsh+x,-yp+ysh+y,-zp+z,irotm(2),
     @                                                      'ONLY',par)

      par(3) = 6.
      xp = -0.5*dxgb - 3. - par(3)
      yp = 20.
      call crpos('WTBM',csl,0,0, xp+xsh+x, yp+ysh+y, zp+z,irotm(1),
     @                                                      'ONLY',par)
      call crpos('WTBM',csl,0,0, xp+xsh+x, yp+ysh+y,-zp+z,irotm(1),
     @                                                      'ONLY',par)
      call crpos('WTBM',csl,0,0, xp+xsh+x,-yp+ysh+y, zp+z,irotm(1),
     @                                                      'ONLY',par)
      call crpos('WTBM',csl,0,0, xp+xsh+x,-yp+ysh+y,-zp+z,irotm(1),
     @                                                      'ONLY',par)

c inside the gas box

c-- lower and upper alu material
      par(1) = 0.5*dxgb
      par(2) =  .25
      par(3) = 0.5*dzgb !
      xp =   0.
      yp = 0.5*dygb - par(2)
      zp = 0.
      call crpos('WTAM',csl,0,0,xp+xsh+x, yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTAM',csl,0,0,xp+xsh+x,-yp+ysh+y,zp+z,0,'ONLY',par)


c g10 and rohacell   side walls
      par(1) =0.5*1.5
      par(2) =0.5*dygb - 6.0
      par(3) =0.5*dzgb
      xp = -0.5*dxgb + par(1)
      yp = 0.
      zp = 0.
      call crpos('WTRM',csl,0,0, xp+xsh+x,yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTRM',csl,0,0,-xp+xsh+x,yp+ysh+y,zp+z,0,'ONLY',par)

      par(2) =0.5 * 5.5
      yp =0.5*dygb - 6. + par(2)
      call crpos('WTGM',csl,0,0, xp+xsh+x, yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTGM',csl,0,0,-xp+xsh+x, yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTGM',csl,0,0, xp+xsh+x,-yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTGM',csl,0,0,-xp+xsh+x,-yp+ysh+y,zp+z,0,'ONLY',par)

      par(1) =0.5*1.
      par(2) =0.5*dynom
      par(3) =0.5*dzgb - dznom
      xp =0.5*dxnom - par(1)
      yp = 0.
      zp = 0.
      call crpos('WTRM',csl,0,0, xp+xsh+x,yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTRM',csl,0,0,-xp+xsh+x,yp+ysh+y,zp+z,0,'ONLY',par)

c distance holders
      par(1) = .7
      par(2) = 1.25
      par(3) =0.5*dzgb
      xp =0.5*dxgb - 4.5
      yp = 20.5
      zp = 0.
      call crpos('WTDM',csl,0,0, xp+xsh+x, yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTDM',csl,0,0, xp+xsh+x,-yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTDM',csl,0,0,-xp+xsh+x, yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTDM',csl,0,0,-xp+xsh+x,-yp+ysh+y,zp+z,0,'ONLY',par)

      par(1) = 0.
      par(2) = .4
      par(3) =0.5*dzgb
      call crpos('WTBM',csl,0,0, xp+xsh+x, yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTBM',csl,0,0, xp+xsh+x,-yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTBM',csl,0,0,-xp+xsh+x, yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTBM',csl,0,0,-xp+xsh+x,-yp+ysh+y,zp+z,0,'ONLY',par)

      yp = 42.5
      call crpos('WTBM',csl,0,0, xp+xsh+x, yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTBM',csl,0,0, xp+xsh+x,-yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTBM',csl,0,0,-xp+xsh+x, yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTBM',csl,0,0,-xp+xsh+x,-yp+ysh+y,zp+z,0,'ONLY',par)
      par(1) = .7
      par(2) = 1.25
      par(3) =0.5*dzgb
      call crpos('WTDM',csl,0,0, xp+xsh+x, yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTDM',csl,0,0, xp+xsh+x,-yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTDM',csl,0,0,-xp+xsh+x, yp+ysh+y,zp+z,0,'ONLY',par)
      call crpos('WTDM',csl,0,0,-xp+xsh+x,-yp+ysh+y,zp+z,0,'ONLY',par)

      zp = -0.5*dzgb
c install the front nomex wall
      par(1) =0.5*dxnom
      par(2) =0.5*dynom
      par(3) =0.5*dznom
      xp =   0.
      yp =   0.
      call crpos('WTNM',csl,0,0,xp+xsh+x,yp+ysh+y,zp+par(3)+z,0,
     @                                                      'ONLY',par)
      zp = zp + 2.*par(3)

100     continue

c Is it correct to use _lay ????
      do i=1,igesl_lay(isl)             ! loop over layers
c install the detector
c-- install preamp's !1'
         par(1) =  28.0/2.
         par(2) =   7.0/2.
         par(3) =   0.04/2.
         xp =  0.
         yp = 0.5*dygb +par(2)

c now find all GEDE planes from given layer and get their z coordinates
         zmin = 1.e10
         zmax = -zmin
         do igede = igesl_fpl(isl), igesl_fpl(isl) + igesl_npl(isl) - 1
            if (igede_lay(igede) .eq. i) then
               zmin = min(zmin,fgede_zmin(igede))
               zmax = max(zmax,fgede_zmax(igede))
            end if
         end do

         if (zmin .ge.zmax) then
            c78 = 'OTWD96: zmin='
            call ch78af(zmin,-2)
            call ch78ac(' >= ')
            call ch78af(zmax,-2)
            call meerro (c78)
         end if

ccc         zp = zp +0.5*dzmod
         zp = 0.5*(zmin + zmax) - zp_sl
         call crpos('WTCM',csl,0,0,xp+xsh+x, yp+ysh+y,zp,0,'ONLY',par)
         call crpos('WTCM',csl,0,0,xp+xsh+x,-yp+ysh+y,zp,0,'ONLY',par)

      end do                    ! layers

        if (ichwa.eq.1) return  ! in this case install only chamber walls

c nomex back side
      par(1) = 0.5*dxnom
      par(2) = 0.5*dynom
      par(3) = 0.5*dznom
      xp =   0.
      yp =   0.
      zp =  0.5* dzgb - par(3)   ! 0.5 was inserted!!!
      call crpos('WTNM',csl,0,0,xp+xsh+x,yp+ysh+y,zp+z,0,'ONLY',par)

      return
      end
