/***********************************************************************
*
* SISGEO
* written by Siegmund Nowak
*
* This routines defines silicon detector geometry  for the HERA-B detector
*
*       called from G..MAIN
*
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/Arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "arte/ArteUtil.h"
#include "gean/zgeom.h"
#include "gean/ZGESL.h"
#include "auto/GEO.hh"

  extern "C" {
    void sisgeo_(int &);
    void cggesl_(char*, char*, int*);
    void gsatt_(char*, char*, int*);
    void gsdet_(char*,char*,int*,char*,int*,int*,int*,int*,int*,int*);}

    static int first_sisgeo=0;
    int    read_dat(FILE *, int&); 
    void   cr_arte(const int);
    FILE  *utfilp(char*,char*);

void sisgeo_(int &isl) {

    char colo[5], mot[5], set[5], filnam[16];
    static FILE *SIL_inp;
    zgrot ZGROT;
    zgvol ZGVOL;
    zgpos ZGPOS;

    strcpy(colo,"COLO");
    strcpy(set ,"SIST");

    zgvol::Set_ind_pt(LGESL_SIL);

    if(first_sisgeo == 0) {
//     select file name 
       if(int(arteflag_.geomvers)==97) strcpy(filnam,"sis97.dat");           
       else  strcpy(filnam,"sis00.dat");
       SIL_inp = utfilp(filnam,"\n");
    
//     reserve storage for zgrot, zgvol and zgpos 
       zgdet::ini(5);   
       zgrot::ini(5);   
       zgvol::ini(80);
       zgpos::ini(60);
       first_sisgeo = 1;         }

    int icopy = 0;
    int imodule = 0;   
    
//  now the card handling begins

    for (;;)   {
//  read in one volume block
      int datagroup=0; int nr =0;
     datagroup = read_dat(SIL_inp,nr); 

//   no action 
     if (datagroup == 0) continue;                          

     if (datagroup == 1) {
       arteMsg << "media cards not foreseen in sisgeo";
       arteMsg.warn();
       continue;}

//   rotations
     if (datagroup == 2) {
        int nrot=999;
        ZGROT.Create_Grot(nrot);
        continue; }

     char volname[5];
     int nr_vol=999;
     strncpy(volname,ZGVOL.Get_volname(nr_vol),4);

//   components or modules
     if (datagroup == 3 || datagroup==4 ) {
//      check whether the volume already exists
        int Gvolnum = ZGVOL.Check_Gvol(volname);
        if (Gvolnum == 0) {
            ZGVOL.Create_Gvol(nr_vol);
            if(ZGVOL.Get_sens_flag(nr_vol) > 2 ) {
               int iset,idet,subdet_type=LGESL_SIL,i20=20,i32=32,i50=50,i1=1;
               gsdet_(set,volname,&i1,volname,&i32,&subdet_type,
                      &i50,&i20,&iset,&idet);}}}

     else {
//      define the superlayer volume and posit it in gean 
        ArtePointer<GESL> gesl(isl);
        if(!strncmp(gesl->name,volname,4))  {
           if(gesl->lay == 0) break;  
           zgmoth_.CURRSL = isl; 
           strncpy(mot,"MOTH",4);
           int i0=0, color=-7;
           cggesl_(gesl->name,mot,&i0);          
           gsatt_(gesl->name,colo,&color); }}
      
//    modules 
      if(datagroup == 4 )    {
         ZGPOS.Gposit_all(nr_vol,icopy); 
         icopy = 0;}

//    superlayers
      if(datagroup == 5 || datagroup == -99)    {
         ZGPOS.Gposit_all(nr_vol,imodule);
         cr_arte(nr_vol);   }

     if ( datagroup == -99 )  {                              //end of file
       arteMsg << " CLOSE FILE " << filnam;
       arteMsg.warn();
       zgdet::fin();
       zgrot::fin();
       zgvol::fin();
       zgpos::fin();           
       break;}                      

     if (datagroup == 5)  break;     
   }}
  
/* sisgeo_ */

