#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "arte/Arte.hh"
#include "arte/ArteMessage.hh"
#include "auto/GEO.hh"
#include "gean/zgeom.h"
#include "gean/gclink.h"

  int zgvol::maxvol[33] ;
  int zgvol::nrvol[33] ;
  int zgvol::ind_pt;
  hgvol *zgvol::ptvol[33];

extern "C" {
    int mednum_(int&,const char*,int);
    void gsvolu_(char*, char*, int*, float [], int*, int*);
    void gsatt_(char*,char*,int*); }


void zgvol::ini(int n){ 
        ptvol[ind_pt] = new hgvol[n];
        if(ptvol[ind_pt]==NULL) {
               arteMsg << "no hgvol[n] allocated"; arteMsg.error();}
        maxvol[ind_pt] = n;
        nrvol[ind_pt] =-1;  }

void zgvol::fin() {   
        delete [] ptvol[ind_pt];
        nrvol[ind_pt] = -1; }   

void zgvol::increase() {
        if(nrvol[ind_pt] < maxvol[ind_pt]-1) nrvol[ind_pt]++;
        else {
        hgvol *vol, *temp, *tmp; 
        int n =2*maxvol[ind_pt];
        temp = new hgvol[n];    
        if(temp == NULL) {
             arteMsg << "no hgvol[2*n] allocated";   arteMsg.error();} 
        for(int i=0; i <= nrvol[ind_pt]; i++) {
          vol = ptvol[ind_pt] + i ;    tmp = temp + i ;   *tmp = *vol; }
        delete [] ptvol[ind_pt];        
        ptvol[ind_pt] = temp;
        maxvol[ind_pt] = 2* maxvol[ind_pt]; 
        nrvol[ind_pt]++;}}

void zgvol::Set_ind_pt(int n) {
        if(n >= 0 && n < 34){
           ind_pt = n;
           zgdet::ind_pt = n;           
           zgpos::ind_pt = n;           
           zgrot::ind_pt = n;}
        else{
          arteMsg << "zgvol: ind_pt outside limits";
          arteMsg.warn();      }} 

hgvol *zgvol::get_ptvol(int n) {
        return ptvol[n];} 
  
void zgvol::Set_default() {
        hgvol *vol = ptvol[ind_pt] + nrvol[ind_pt] ;
        memset(vol,0,sizeof(hgvol));
        vol->seen = vol->color = 1; }

void zgvol::Copy_previous() {
        if (nrvol[ind_pt] >0 ) {
           hgvol *vol = ptvol[ind_pt] + nrvol[ind_pt] ;
           hgvol *old = vol -1;       
           *vol = *old;
           vol->ncomp =0;
           if(vol->sens_flag >2 ) { 
              zgdet::increase();
              zgdet ZGDET;
              ZGDET.Copy_previous();
              ZGDET.Set_vol_det_Rel(); }}
        else { 
           arteMsg <<"zgvol::Copy previous: previous does not exist";
           arteMsg.error();}}

void zgvol::Read_vol(char *card,int  &nr) {
        increase(); 
        nr = nrvol[ind_pt];
        hgvol *vol = ptvol[ind_pt] + nr ;
        char taga[41], tagb[16];
        int nitem=sscanf(card,"%s %s ",taga,tagb);
        if (nitem!=2){arteMsg <<"check data on volume card"; arteMsg.error();}
        char *ptr = strstr(tagb,"*");
        if(ptr == NULL) Set_default();    
        else Copy_previous();
        fillstr(taga,40);
        fillstr(tagb,4);
        strncpy(vol->name,taga,40);
        strncpy(vol->object,tagb,4);  }

void  zgvol::Read_shape(char *card) {
        char taga[16];
        hgvol *vol = ptvol[ind_pt] + nrvol[ind_pt] ;
        int nitem=sscanf(card,"%s ",taga);
        if(nitem!=1) {arteMsg <<"check data on shape card"; arteMsg.error();}
        fillstr(taga,4);            
        strncpy(vol->shape,taga,4);
        if(!strncmp(taga,"BOX ",4) || !strncmp(taga,"TUBE",4) ||
           !strncmp(taga,"ELTU",4)      ) vol->npar =3; 
        else if(!strncmp(taga,"TRD1",4) ) vol->npar =4; 
        else if(!strncmp(taga,"TRD2",4) || !strncmp(taga,"TUBS",4) ||
                !strncmp(taga,"CONE",4) ) vol->npar =5; 
        else if(!strncmp(taga,"SPHE",4)||!strncmp(taga,"PARA",4)) vol->npar =6;
        else if(!strncmp(taga,"CONS",4) ) vol->npar =7;
        else if(!strncmp(taga,"TRAP",4) ) vol->npar =11;
        else { 
            arteMsg <<"shape" << vol->shape << "not foreseen" ;
            arteMsg.error();}}

void  zgvol::Read_dimension(char *card) {
        hgvol *vol = ptvol[ind_pt] + nrvol[ind_pt] ;
        if(vol->npar == 0) {
            arteMsg <<"in volume" << vol->object << "define first the shape" ;
            arteMsg.error();} 
        else {
           for (int i=0; i<vol->npar; i++) {  
              int nitem=sscanf(card,"%f",&vol->dimension[i]); 
              if (nitem!=1) {
                 arteMsg <<"check data on dimension cards";
                 arteMsg.error();} 
              card+=itemlength(card,nitem);} } }

void  zgvol::Read_type(char *card) {  
        hgvol *vol = ptvol[ind_pt] + nrvol[ind_pt] ;
        int nitem=sscanf(card,"%d",&vol->type);
        if (nitem!=1) {
            arteMsg <<"check data on type cards";
            arteMsg.error();}      }

void  zgvol::Read_sens_fl(char *card) {
        hgvol *vol = ptvol[ind_pt] + nrvol[ind_pt] ;
        int sflag = vol->sens_flag; 
        int nitem=sscanf(card,"%d",&vol->sens_flag);
        if (nitem!=1) {
            arteMsg <<"check data on sens_flag cards";
            arteMsg.error();}
        if(vol->sens_flag > 2 && sflag == 0) {
          zgdet::increase(); 
          zgdet ZGDET;
          ZGDET.Set_default();
          ZGDET.Set_vol_det_Rel();}}  

void  zgvol::Read_material(char *card) {
        char taga[16];
        hgvol *vol = ptvol[ind_pt] + nrvol[ind_pt] ;
        int nitem=sscanf(card,"%s",taga);
        if (nitem!=1) {
            arteMsg <<"check data on material cards";
            arteMsg.error();}
        fillstr(taga,12);       
        strncpy(vol->material,taga,12);   }

void  zgvol::Read_rel_point(char *card) {
        char taga[16];
        hgvol *vol = ptvol[ind_pt] + nrvol[ind_pt] ;
        sscanf(card,"%s",taga);
        for (int m=0;m<3;m++) taga[m] = toupper(taga[m]);
        if(!strncmp(taga,"DET",3)) {
          for(unsigned int isl=1;isl<=ArtePointer<GESL>::nrow();isl++) {
               ArtePointer<GESL> gesl(isl);
               if(!strncmp(gesl->name,vol->object,4)) {
                  vol->rel_px = -0.5*(gesl->x1 + gesl->x2);
                  vol->rel_py = -0.5*(gesl->y1 + gesl->y2);
                  vol->rel_pz = -0.5*(gesl->z1 + gesl->z2);
                  return;}}}
        else {  
           int nitem=sscanf(card,"%f %f %f",
                                &vol->rel_px,&vol->rel_py,&vol->rel_pz); 
        if (nitem!=3) {
            arteMsg <<"check data on rel_point cards";
            arteMsg.error();}}}

void  zgvol::Read_color(char *card) {
        hgvol *vol = ptvol[ind_pt] + nrvol[ind_pt] ;
        int nitem=sscanf(card,"%d",&vol->color);
        if (nitem!=1) {
            arteMsg <<"check data on color cards";
            arteMsg.error();}      
        if(vol->color!=0) vol->seen=1;              } 

char* zgvol::Get_volname(int &nr) {
        if (nr> nrvol[ind_pt]) nr=nrvol[ind_pt]; 
        hgvol *vol = ptvol[ind_pt] + nr;
        return vol->object;}

int   zgvol::Get_volnr(char* volname,int voltype) {
        for(int n=0;n<=zgvol::nrvol[zgvol::ind_pt];n++) {
           hgvol *dau = zgvol::ptvol[zgvol::ind_pt] + n;
           if(!strncmp(dau->object,volname,4)&&dau->type==voltype) 
                                                         return n;}            
        return -1;} 

char* zgvol::Get_shape(int &nr) {
        if (nr> nrvol[ind_pt]) nr=nrvol[ind_pt]; 
        hgvol *vol = ptvol[ind_pt] + nr;
        return vol->shape;}

int  zgvol::Get_dimension(int &nr,float dim[]) {
        if (nr> nrvol[ind_pt]) nr=nrvol[ind_pt]; 
        hgvol *vol = ptvol[ind_pt] + nr;
        for (int i=0; i<vol->npar; i++) dim[i] = vol->dimension[i];
        return vol->npar;}

int  zgvol::Get_par(int &nr,float dim[]) {
       if (nr> nrvol[ind_pt]) nr=nrvol[ind_pt]; 
       hgvol *vol = ptvol[ind_pt] + nr;
       if(!strncmp(vol->shape,"BOX ",4) || !strncmp(vol->shape,"TRD1",4) ||
          !strncmp(vol->shape,"TRD2",4))
          for (int i=0; i<vol->npar; i++) dim[i] = 0.5*vol->dimension[i];
       else if(!strncmp(vol->shape,"TUBE",4) || !strncmp(vol->shape,"TUBS",4) ||
               !strncmp(vol->shape,"ELTU",4)){ 
          for (int i=0; i<vol->npar; i++) dim[i] = vol->dimension[i];
          dim[2] = 0.5 * vol->dimension[2];}
       else if(!strncmp(vol->shape,"CONE",4) || !strncmp(vol->shape,"CONS",4)){
          for (int i=0; i<vol->npar; i++) dim[i] = vol->dimension[i];
          dim[0] = 0.5*dim[0]; }     
       else if(!strncmp(vol->shape,"SPHE",4))  
          for (int i=0; i<vol->npar; i++) dim[i] = vol->dimension[i];
       else if(!strncmp(vol->shape,"PARA",4)) {  
          for (int i=0; i< 3; i++) dim[i] = 0.5*vol->dimension[i];
          for (int i=3; i<vol->npar; i++) dim[i] = vol->dimension[i];}
       return vol->npar;}

int  zgvol::Get_type(int &nr) {  
        if (nr> nrvol[ind_pt]) nr=nrvol[ind_pt]; 
        hgvol *vol = ptvol[ind_pt] + nr;
        return vol->type;}

int   zgvol::Get_sens_flag(int &nr) {
        if (nr > nrvol[ind_pt]) nr = nrvol[ind_pt];
        hgvol *vol = ptvol[ind_pt] + nr;
        return vol->sens_flag;}

char* zgvol::Get_material(int &nr, int &len) {
        if (nr> nrvol[ind_pt]) nr=nrvol[ind_pt]; 
        hgvol *vol = ptvol[ind_pt] + nr;
        len = strlen(vol->material);
        if(len > 12) len=12;
        return vol->material;}

void  zgvol::Get_rel_point(int &nr,float *xp, float *yp, float *zp) {
        if (nr> nrvol[ind_pt]) nr=nrvol[ind_pt]; 
        hgvol *vol = ptvol[ind_pt] + nr;
        *xp = vol->rel_px;
        *yp = vol->rel_py;
        *zp = vol->rel_pz;}

int   zgvol::Get_color(int &nr) {
        if (nr> nrvol[ind_pt]) nr=nrvol[ind_pt]; 
        hgvol *vol = ptvol[ind_pt] + nr;
        return vol->color;}

int   zgvol::Get_ncomp(int &nr) {
        if (nr > nrvol[ind_pt]) nr = nrvol[ind_pt];
        hgvol *vol = ptvol[ind_pt] + nr;
        return vol->ncomp;}

void  zgvol::Create_Gvol(int n) {
        int imed, ivol, len;
        int zero=0;
        hgvol *vol = ptvol[ind_pt] + n;
        float par[vol->npar];
        char colo[5];
        strcpy(colo,"COLO");
        len = strlen(vol->material);
        if(len >12) len=12;  
        imed = mednum_(vol->sens_flag, vol->material, len);
	gsvolu_(vol->object,vol->shape, &imed, par, &zero, &ivol);
        gsatt_(vol->object, colo, &vol->color);  }


int   zgvol::Check_Gvol(char *volname) {
//      check whether the volume already exists
        union  { int ii; char name[4];};
        int i=0, vol_number=0;
        int nvol=*(gcbank_.iq + gclink_.jvolum -2);
        while( i<nvol) {
          ii = *(gcbank_.iq+gclink_.jvolum+i);
          if(!strncmp(name,volname,4)) {
             vol_number=i+1; 
             return vol_number; }
          i++; }
        return vol_number;  }
  






