#include <sys/types.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <gpack.h>
#include <gpackproto.h>

int main()
{
  EventHdr h;
  char tname[20], desc[512];
  int ncols, nrows;
  int rc;
  int ntab, nevents, lun;
  int i,j;
  int count;
  struct Row
    {
      float zf;
      float xf;
      float yf;
      float pf;
      int   cvf[15];
      int   hsi;
      int   hin;
      int   hou;
      int   hri;
      float chi;
  } row[200];

  printf("Start job outshm ...\n");

  /*
   * Get first free stream lun (Logical Unit Number? or File Descriptor?)
   */

  lun = gp_setstream( ShmStream );
  if( lun < 0 )
    {
      printf("Cannot get lun. rc = %d\n", lun);
      return 1;
    }
  
  /* Set event type : Dst or Mini */
  gp_setevtype(lun, Mini);

  /*
   * Define the size of internal buffer
   * This has influence on performance
   */
  gp_setbuf(lun, 512 * 1024);    /* buffer size in bytes */

  /*
   *  Optionally you can set target type if you want: 
   *     IEEE_BigEndian        (SGI, PowerPC, HP, ...)
   *     IEEE_LittleEndian     (Linux PCs, OSF Alpha's)
   *     G_Float_LittleEndian  (OpenVMS Alpha's)
   *
   */
   if( (rc = gp_settarget(lun, IEEE_LittleEndian)) < 0 )
     {
       printf("Cannot set target type. rc = %d : %s\n",rc, gp_errmsg(lun));
       return 1;
     }

  /*
   *  Set the key to the ring  (ATTENTION! Assign your own key!)
   */

/*  if( (rc = gp_setkey(lun, 17849)) < 0 ) */
  if( (rc = gp_setname(lun, "test.dst", 0)) < 0 )
    {
      printf("Cannot set the key. rc = %d : %s\n", rc, gp_errmsg(lun));
      return 1;
    }

  /* Open ring */
  count = 0;
  while( (rc = gp_open(lun)) < 0 )
    {
      if( count++ > 0 )
        {
          printf("Cannot open ring. rc = %d : %s\n", rc, gp_errmsg(lun));
          return 1;
        }
      if( (rc = gp_create_ring(lun)) < 0 )
        {
          printf("Cannot create ring. rc = %d : %s\n", rc, gp_errmsg(lun));
          return 1;
        }
    }

  nevents = 1000;
  for( h.event = 1; h.event <= nevents; h.event++ )
    {
      sleep(0);
      h.datime    = time(NULL);
      h.classmask = random();
      h.run = 2727;
      h.experiment = 33;
      
      rc = gp_putevt(lun, &h);
      if( rc < 0 )
	{
	  printf("Cannot put event header. rc = %d : %s\n",rc,gp_errmsg(lun));
	  return 1;
	}

      for( ntab = 0; ntab < 4; ntab ++ )
	{
	  sprintf(tname,"Tab%03d",ntab);
	  sprintf(desc,
"ZF:1F,XF:1F,YF:1F,PF:F,CVF:15B1,HSI:B1,HIN:B1,HOU:I1");
	  ncols = 24;
	  nrows = 200;
	  for(i = 0; i < nrows; i++)
	    {
	      row[i].zf  = 1.0 * (float)(random()%(100+i));
	      row[i].xf  = 2.0 * (float)(random()%(100+i));
	      row[i].yf  = 3.0 * (float)(random()%(100+i));
	      row[i].pf  = 4.0 * (float)(random()%(10+i));
	      for(j = 0; j < 15; j++)
		row[i].cvf[j] = random()%20;
	      row[i].hsi = random()%1024;
	      row[i].hin = random()%1024;
	      row[i].hou = random()%1024;
	      row[i].hri = random()%1024;
	      row[i].chi = 1.0 * (float)(random()%100);
	      if( i < 2 )
		printf("zf=%f, xf=%f, yf=%f, hsi=%d, hri=%d\n",
		       row[i].zf,row[i].xf,row[i].yf,row[i].hsi,row[i].hri);
	    }
	  rc = gp_puttab(lun, tname, ncols, nrows, desc, row );
	  if( rc < 0 )
	    {
	      printf("Cannot put table. rc=%d : %s\n",rc,gp_errmsg(lun));
	      return 1;
	    }
	  printf("--------- Table %s written --------\n",tname);    
	}

      /*
       * flush event
       */

      rc = gp_flush(lun);
      if( rc < 0 )
	{
	  printf("Cannot flush event. rc = %d : %s\n", rc, gp_errmsg(lun));
	  return 1;
	}

      printf("\n============= EVENT %d Written ===============\n", h.event);
    }

  /*
   * Close stream
   */ 
  rc = gp_close(lun);
  if( rc < 0 ) 
    {
      printf("Cannot close stream. rc=%d : %s\n", rc, gp_errmsg(lun));
      return 1;
    }

  printf("Job 'outshm' is finished successful!\n");
  return 0;
}
