#ifndef GPACK_H
#define GPACK_H

#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef enum
{
  GP_SUCCESS          =   0,
  GP_FAILURE          =  -1,
  GP_END_OF_DATA      =  -2,
  GP_NOT_OPEN         =  -3,
  GP_BUFFER_OVERFLOW  =  -4,
  GP_BUFFER_UNDERFLOW =  -5,
  GP_WRONG_DATA_SIZE  =  -6,
  GP_NOT_EXIST        =  -7,
  GP_DISK_OVERFLOW    =  -8,
  GP_RESOURCE_BUSY    =  -9,
  GP_TOO_MANY_LOCKS   = -10,
  GP_INTERRUPTED      = -11,
  GP_DEADLOCK         = -12,
  GP_TIMEOUT          = -13,
  GP_INTERNAL_ERROR   = -14,
  GP_OUT_OF_MEMORY    = -15,
  GP_INVALID_DATA     = -16,
  GP_INVALID_FORMAT   = -17,
  GP_UNEXPECTED_EOF   = -18,
  GP_NOT_IMPLEMENTED  = -19,
  GP_UNKNOWN_TARGET   = -20,
  GP_EMPTY_STRUCTURE  = -21,
  GP_EMPTY_DATA_FORM  = -22
} StreamErrorCode;

typedef enum StreamType 
{ 
  Undefined = 0, 
  FileStream, 
  ShmStream, 
  PublicStream,
  PrivateStream
} StreamType;

typedef enum TargetType
{
  TargetTypeUnknown = 0,
  IEEE_BigEndian,
  IEEE_LittleEndian,
  G_Float_LittleEndian
} TargetType;

typedef enum Origin 
{ 
  OriginUnknown = 0,
  LittleEndian,            
  BigEndian
} Origin;

typedef enum OriginFP
{  
  FPOriginUnknown = 0,
  IEEE,                    /* The most computers default */
  G_Float                  /* OpenVMS alpha              */
} OriginFP; 

typedef enum Compression
{
  NoCompression	= 0,
  ZeroSuppress	= 1,
  NoTailBlanks	= 1
} Compression;

typedef enum FileModes    
{ 
  ReadOnly = 1, 
  WriteOnly 
} FileModes;

typedef enum EventType  
{ 
  Dst, 
  Mini,
  Indx,
  Daq 
} EventType;

typedef enum ObjectType 
{ 
  Unknown = 0,
  EventHeader, 
  TableDefinition,
  TableHeader,
  EventTrailer
} ObjectType;

typedef struct EventHdr
{
  int       run;         /* run number                */
  int       event;       /* event number              */
  int       experiment;  /* experiment number         */
  int       datime;      /* UNIX time		      */
  int       classmask;   /* event classification mask */
  EventType type;        /* event type: dst, mini, ... */
} EventHdr;

typedef struct EventTail 
{
  int run;                /* run number   */
  int event;              /* event number */
} EventTail;

typedef struct EventProp
{
  int year;
  int exp;
  int run;
  int event;
  int datime;
  int classmask;
} EventProp;
  
#ifdef __cplusplus
}
#endif

#endif /* GPACK_H */
