#ifndef SHM_STREAM_HH
#define SHM_STREAM_HH

#include <signal.h>
#include <sys/types.h>
#include <stdio.h>
#include <errno.h>
#include "gpack/gpack.h"

#ifdef __linux
typedef int key_t;
#endif

enum Semaphore {Gate = 0, Data, Space};

struct ringHdr 
{
  uint count;
  uint rx;
  uint wx;
};

class ringbase
{
private:
  int        semID;
  int        nsems;
  int        shmID;
  char*      attach;
  sigset_t   signalMask;
  int        locksem(Semaphore s);
  int        setsem(Semaphore s, int value);
  int        freesem(Semaphore s);
  char       errmsg[256];

protected:
  uint       ringSize;
  key_t      CommonKey;

public:
             ringbase();
             ~ringbase();

  char*      getErrorMsg()     { return errmsg; }
  StreamType getStreamType()   { return ShmStream; }
  int        getsem(Semaphore s, int& value);
  void       setkey(key_t key) { CommonKey = key; }
  void       setbuf(uint size) { ringSize = size; }
  int        setname(char *name, int m); // set key by filename (use ftok)
  int        open();
  int        close();
  int        is_open();
  int        create();
  int        destroy();
  uint       numsems();
  uint       ringsz();
  int        snapsem(ushort *semvals); 
  void       snaphdr(ringHdr *h, int lock);
  int        skip(void);
  int        read(char *array, int& length);
  int        write(char *array, int length);
};

class shmstream: public ringbase
{
protected:
  char       *buffer;
  int        bufsize;
  int        used;
  int        cursor;

public:
             shmstream();
             ~shmstream();
  char*      getname()           { return NULL; }
  int        getmode()           { return 0; }
  int        open();
  int        close();
  int        receive();
  int        buflush();
  int        get(char *s, int length);
  int        put(char *s, int length);
  size_t     getpos();
  int        setpos(size_t pos);
};

#include "gpack/shmstream.icc"

#endif  /* SHM_STREAM_HH */
