*-- Author :    Jenny Ivarsson   05/12/96

      SUBROUTINE KACHAN

************************************************************************
*
* Description: Define the decay table for the selected particle.
*              "mother" is a neutral meson or part of its decay chain.
*              This special treatment is important, since the decay in Pythia
*	       does not distinguish between mesons and antimesons.
*              In the mixing of neutral mesons the original flavour is 
*	       forgotten, but will be retained using the input in this routine.
*
*
* Called from: *.kumac
*
* External references:
*
* Argument in/out description
*
************************************************************************

      IMPLICIT NONE

#include "arte/ZDECAY.fi"
#include "arte/Arte.fi"
#include "auto/TOPP.fi"

      character*4 mother
      integer length, nchannels, generation, mixing, tpname

      save generation
      data generation/0/

#include "arte/Arte.ff"
#include "auto/TOPP.ff"
c-------------------------------------------------------------

      CALL KUGETC(mother,length)
      CALL KUGETI(nchannels)
      CALL KUGETI(mixing)

c Decay Table for particle "mother"

      generation = generation + 1

c Save the LUND KF code for particle "mother"

      gKF(generation) = ITOPP_LUND(tpname(mother))

c Save # Decay Channels (reading from top) for particle "mother",
c which are switched on for mesons and switched off for antimesons
        
      gMDCY(generation) = nchannels

c Save information on if "mother" is the mixing meson or not.
c Only one of the specified "mothers" is allowed to be a mixing
c meson.

      gMeson(generation)= mixing

      if(mixing.ne.0) then
	if(meson.gt.0) then
          CALL MEWARC (12364123,
     +   'CHANNEL: More than one particle with mixing flag' //
     +   ' is not allowed.')
	else
	   meson = generation
	endif
      endif
      if(meson.eq.0) meson = -generation

C------------------------------------------------------------------
C     Ex: For the Golden decay specification      B.  -->  J/Psi K0S
C                                               J/Psi -->  l+l-
C                                                K0S  -->  pi+ pi-
C      
C      gMDCY(1) = 1  ! Total number of selected decay channels for mother
C      gMDCY(2) = 2  !   "     "          "          "           1st daughter
C      gMDCY(3) = 1  !   
C
C      gMeson(1) = 1  ! B. is neutral meson which mixes
C      gMeson(2) = 0
C      gMeson(3) = 0
C-------------------------------------------------------------------

      
c      PRINT*,'SDECAY: # decay channels for ',mother,' is ',nchannels

      END

************************************************************************
* End of KACHAN
************************************************************************
