*CMZ :  1.00/00 11/01/95  12.54.34  by  Dominik Ressing
*-- Author :    Dominik Ressing   27/07/94
************************************************************************
*
* L1POHI1
* written by Dominik Ressing, 27/07/94 17.18.03
*
* This routines action:
* return the hit number for the central cell hitted by a track transversing
* at point rPoint.
* No limitations of the hit numbers due to the finite detector volume are
* performed.
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* iDetSpe     in          detector specifier as defined in $l1detdf
* rPoint      in          point description as defined in $l1detdf
* iHit        out         hit number for point ( -99999 on error )
*
************************************************************************
c
      SUBROUTINE L1POHI1(iDetSpe,rPoint,iHit)
c
      IMPLICIT NONE
c
#include "gean/gcbank.fi"
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1detdf.fi"
c
      INTEGER iDetSpe(L1DETSPElen),
     &        iDetInfo(L1DETINFlen),iHit
      REAL    rPoint(L1POINTlen)
c
      INTEGER nHit,                  ! number of hits in a row (y=const)
     &        jDetGeo,               ! geometry of specified detector
     &        jCellGeo,              ! geometry of a detector cell
     &        jDetInfo               ! information about detector cells
      REAL    dz,                    ! z-distance of hit-point from detector plane
     &        xP,yP,                 ! track intersept with detector
     &        dx,dy,                 ! x,y in detector plane frame
     &        xx,                    ! x-distance from origin
     &        rHit                   ! x-distance from origin in units of cells
c
      iHit = -99999 ! no hit in detector (so far)
c
      CALL l1detif(iDetSpe,jDetGeo,jDetInfo,jCellGeo) ! get detector geometry
      if ( jDetGeo.le.0 ) GO TO 777                   ! no success
      IF (IQ(jDetInfo+L1DETINFnCol).gt.1 ) GO TO 777  ! pad-like detector -> invalid
c
      dz = Q(jDetGeo+L1GEOINFz) - rPoint(L1POINTz0)   ! distance of detector from point
      xP = rPoint(L1POINTx0) + dz*rPoint(L1POINTdx)   ! track intersept with detector
      yP = rPoint(L1POINTy0) + dz*rPoint(L1POINTdy)   ! 
c
      dx = xP - Q(jDetGeo+L1GEOINFx0)
      dy = yP - Q(jDetGeo+L1GEOINFy0)
      xx = dx - dy*Q(jDetGeo+L1GEOINFsin_al)/Q(jDetGeo+L1GEOINFcos_al)
      xx = xx - Q(jDetInfo+L1DETINFxlap)/2.          ! central value is at lap/2.
c
      rHit = xx / Q(jDetInfo+L1DETINFxspace)         ! in units of driftcells
      iHit = INT( rHit ) + IQ(jDetInfo+L1DETINFfirst)! add first-hit offset
      IF ( rHit.lt.0. ) iHit = iHit - 1
c
      RETURN
c
 777  CONTINUE
c     --- error exit
      WRITE(L1LUNprn,900) iDetSpe(L1DETSPEdet),
     &                    iDetSpe(L1DETSPElayer),
     &                    iDetSpe(L1DETSPEsection),
     &                    iHit
 900  FORMAT(1x,
     & 'L1POHI1 ==> invalid detector iseq-det,layer,section,hits:'/
     & 10x,4i8)
c
      RETURN
c
      END
