*CMZ :          23/04/96  23.15.29  by  Fedor Ratnikov
*CMZ :  1.00/00 19/12/94  16.54.01  by  Dominik Ressing
*-- Author :    Dominik Ressing   17/12/94
************************************************************************
*
* L1SLOPE
* written by Dominik Ressing, 17/12/94 17.41.14
*
* This routines action:
* returns the allowed track slopes at the given  detector point.
*
* Arguments  kind         description
*          (in,update,out)
*----------------------------------------------------------------------*
* x,y,z       in          Point in x,z plane of detector
* P           in          (minimal) momentum of track for slope boundaries
* SlopeMax    out         maximal track slope
* SlopeMin    out         minimal track slope
*
************************************************************************
c
      SUBROUTINE L1SLOPE(x,y,z,P,SlopeMax,SlopeMin)
c
      IMPLICIT NONE
c
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1detdf.fi"
c
      REAL x,y,z,BendMax,bbo1,bbo2,bbma,bbmi,SlopeMax,SlopeMin,
     &     P,rLe,a,b
c
c     --- length of magnetig field along track:
      a   = ((x*T1DETmagz)/(z*T1DETmagz))**2
      b   = (0.5*T1DETmagd/T1DETmagz)**2
      rLe = T1DETmagd*sqrt(1.-a/(1.-a*b))   ! length of the magnetig field
      rLe = rLe/cos(y/z)
c
c     --- bending for this momentum track
      BendMax = 0.003*T1DETmagB*rLe/P
c
      SlopeMax = (x + BendMax*T1DETmagz)/z     ! for q = +1
      SlopeMin = (x - BendMax*T1DETmagz)/z     ! for q = -1
c
      bbo1 = (0.5*T1DETmagd-x)/(T1DETmagz-z)    ! geometric boundaries
      bbo2 = (-0.5*T1DETmagd-x)/(T1DETmagz-z) !
      bbma = MAX(bbo1,bbo2)                    ! max. slope due to detector geometry
      bbmi = MIN(bbo1,bbo2)                    ! min. 
c
      SlopeMax = MIN(SlopeMax,bbma)
      SlopeMin = MAX(SlopeMin,bbmi)
c
      RETURN
c
      END
