//**********************************************************************/
// Created: 23.08.97,   Olya Igonkina   for   MARPLE package
//
// implementation of RcTraPoint class
//
//**********************************************************************/
#include <iostream.h>
#include <iomanip.h>

#include "arte/Arte.hh"
//#include <Pointer.hh>
#include "auto/MTRA.h"

#include "marple/marple.hh"

RcTraPoint::RcTraPoint( const ArtePointer <RTRA>  rtraLink)
  : RcPoint(), fTraLink(rtraLink)  
{
  if( rtraLink == 0 || rtraLink.index() > rtraLink.nrow() )
    { 
      arteMsg << " MARPLE: RTRA entry number " << rtraLink 
	      << "does not exist" ;
      arteMsg.allp();
      arteMsg << "- empty object is created" ;
      arteMsg.allp();
      return;
    }
  setX(   rtraLink->xf  );
  setY(   rtraLink->yf  );
  setZ(   rtraLink->zf  );
  settx(  rtraLink->txf );
  setty(  rtraLink->tyf );
  setpe(  rtraLink->pf );
  setcov( rtraLink->cvf );

  checkAndCorrectCov();
}

// returns relation with MTRA 
vector<ArtePointer<MTRA> > RcTraPoint::mtraList() const
{
  vector<ArtePointer<MTRA> > mList;
  for( ArteRelationIterator<RTRX_MTRA> 
	 rsMtra( (ArtePointer<RTRX>(link() ))->mtra);
       rsMtra.valid();
       rsMtra.next() )
    {
      ArtePointer<MTRA> mEntry( rsMtra);
      mList.push_back(mEntry);
    }
  return mList;
}

// returns relation with HITB 
vector<ArtePointer<HITB> > RcTraPoint::hitbList() const
{
  vector<ArtePointer<HITB> > hList;
  for( ArteRelationIterator<RTRX_HITB> 
	 rsHitb( (ArtePointer<RTRX>(link() ))->hitb);
       rsHitb.valid();
       rsHitb.next() )
    {
      ArtePointer<HITB> hEntry( rsHitb);
      hList.push_back(hEntry);
    }
  return hList;
}

// update RTRA bank by RcTraPoint information
void RcTraPoint::updateARTE( int & iErr) const
{
  iErr = 0;
  ArtePointer<RTRA> rtra = link();
  if( rtra== 0 || rtra.index() > rtra.nrow() )
    {
      arteMsg << " MARPLE: updateARTE: RTRA entry " 
	      << rtra << " does not exist";
      arteMsg.allp();
      iErr = 1;
      return;
    }
  static int idMess = 76576;
  arteMsg << "MARPLE:  updateARTE: RcTraPoint saved as a first point of RTRA";
  arteMsg.allp(idMess,1);
  
  rtra->xf  = x();
  rtra->yf  = y() ;
  rtra->zf  = z() ;
  rtra->txf = tx() ;
  rtra->tyf = ty();
  rtra->pf  = pe();
  for( int i=0; i<5; i++){
    for(int j=i; j<5; j++){
      rtra->cvf[c_matrix_pack[i][j]] = cov()[i][j]; 
      // start numbering from [0][0] as in struct RTRA
    }
  }
}

ostream & operator << (ostream & os, const RcTraPoint &rp)
{
  //os << static_cast<RcPoint&>(rp) ;
  const RcPoint * ptPoint = &rp;
  os << *ptPoint;
  os << "            RTRA link: " << rp.link() << endl ;
  return os ;
}
