#ifndef _RCTRAPOINT_HH_
#define _RCTRAPOINT_HH_
//**********************************************************************/
// Created: 13.08.97,   Olya Igonkina
//
// MARPLE class of Points for Reconstruction which have link to RTRA bank
//
//**********************************************************************/

#include <iostream.h>
#include "marple/RcPoint.hh"

#include "arte/Arte.hh"
#include "arte/ArteMessage.hh"
#include "auto/RTRA.hh"

/** class with link to RTRA bank.
    Class is inherited from RcPoint
*/
class RcTraPoint : public RcPoint
{
public:
  ///                 default constructor
  inline RcTraPoint(); 
  ///                 constructor with link to RTRA bank
  inline RcTraPoint( const RcPoint &rp, 
		     const ArtePointer <RTRA> rtraLink = 0 );
  /// constructor from RTRA bank entry (first point of RTRA is passed to RcTraPoint object ) if any diagonal element of covariant matrix equal 0 it is set to infinity
  RcTraPoint( const ArtePointer <RTRA> rtraLink);

  ///                 copy constructor
  inline RcTraPoint( const RcTraPoint &rp ); 

  ///                 destructor
  inline ~RcTraPoint(); 

  ///                 returns link to RTRA table
  inline ArtePointer <RTRA>  link() const;

  ///                 set link to RTRA table
  inline void  setlink( const ArtePointer <RTRA> rtraLink = 0 );
  
  ///                 returns relation with MTRA 
  vector<ArtePointer<MTRA> > mtraList() const; 

  ///                 returns relation with HITB 
  vector<ArtePointer<HITB> > hitbList() const; 

  /// update RTRA bank by RcTraPoint information
  void updateARTE( int & iErr ) const ;

  ///                 assignment operator
  inline RcTraPoint & operator = (const RcTraPoint &rp); 
  ///                 assignment operator
  inline RcTraPoint & operator = (const RcPoint &rp); 

private:
  ///                ArtePointer to RTRA bank
  ArtePointer <RTRA> fTraLink;
};

//                   ostream operator to print RcTraPoint information
ostream & operator << (ostream &, const RcTraPoint &);

#include "marple/RcTraPoint.icc"

#endif

