//--------------------------------------------------------------------
// Created: 02.09.97 by   Olya Igonkina

// MARPLE function for matching SUSI and Ranger

//--------------------------------------------------------------------

#include "marple/marple.hh"
#include "marple/RcMatchBuilder.hh"

#include "arte/ArteMessage.hh"
#include "arte/RSEG_cmp.hh"

//definition of constants for SUSI-RANGER matching ( see \Ref{RcMatchConst})
// flag, z_plane, z_min, z_max, use_pair_cut, n_pair_left, n_pair_right, 
// use_delta_cut, delta_x, delta_y, delta_tx, delta_ty, delta_pe, chi2, 
RcMatchConst rcSvdOtConst = { 
11110,     160.,   30.,  800.,      1,          1,            1,
    1,      3.,      5.,      0.01,      .01,   1.e+10,   60.,  
};


//--------------------------------------------------------------------
//   matching of SVD segments and segments reconstructed behind magnet(OT)
//   with estimation of momentum information 
//   by requirement of segment point to target
//--------------------------------------------------------------------
  
extern "C"  
void FORTRAN_ROUTINE(matchsvdot)() 
{

  RcMatchBuilder< RcSegPoint, RcSegPoint> SvdOtMatch;

  //--------------------------------------------------------------------
  // put constants to the MatchBuilder
  //--------------------------------------------------------------------
  SvdOtMatch.setConst(rcSvdOtConst);

  //--------------------------------------------------------------------
  // create vector of SVD segments
  //--------------------------------------------------------------------
  
  int iErr;
  for( ArteTableIterator <RSEG> irseg;
        irseg.valid(); irseg.next() )
    {
      if( irseg->fit >= 0 && irseg->cmp & Rsegc::vxd && 
	  SvdOtMatch.ptToMatchConst()->geomAccept(HepDouble(irseg->ze)) )
	{ 
	  RcSegPoint * ptToPoint = new RcSegPoint( irseg , 1 );
	  ptToPoint->prolongate( SvdOtMatch.ptToMatchConst()->z_plane,
				 iErr);
	  if( iErr )
	    {
	      delete ptToPoint;
	    }else{
	      SvdOtMatch.addPointLeft( *ptToPoint);
	    }
	  ptToPoint = 0;
	}
    }
  //--------------------------------------------------------------------
  // create vector of Outer Tracker segments 
  //--------------------------------------------------------------------
  
  for( ArteTableIterator <RSEG> irseg;
       irseg.valid() ; irseg.next() )
    {
      if( irseg->fit >= 0 && irseg->cmp & Rsegc::patt && 
	  SvdOtMatch.ptToMatchConst()->geomAccept(HepDouble(irseg->zf)) )
	{
	  RcSegPoint * ptToPoint = new RcSegPoint( irseg, 0 );

	  ptToPoint->estimatePEfromTAR( 5, iErr);
	  if( iErr ){ delete ptToPoint; ptToPoint = 0; continue; }
	  ptToPoint->prolongate( SvdOtMatch.ptToMatchConst()->z_plane, iErr);
	  if( iErr ){ delete ptToPoint; ptToPoint = 0; continue; }

	  SvdOtMatch.addPointRight( *ptToPoint);
	  ptToPoint = 0;	      
	}
    }  
  
  //--------------------------------------------------------------------
  // create a list of matching points with chi2 
  //--------------------------------------------------------------------

  SvdOtMatch.doMatch();
  
  //--------------------------------------------------------------------
  // print result of matching
  //--------------------------------------------------------------------
  static int firstSVDOT = 1;
  if( firstSVDOT )
    {
      cout << *SvdOtMatch.ptToMatchConst();
      firstSVDOT = 0;
    }
  SvdOtMatch.stat(0);
  
  //SvdOtMatch.fillDeltas(2000);
  SvdOtMatch.fillMatch( 1000);

  //--------------------------------------------------------------------
  // save found relation to the table
  //--------------------------------------------------------------------

  SvdOtMatch.saveRelation(iErr);
  if( iErr )
    { arteMsg << "MARPLE: some relations were not saved "; arteMsg.allp();}

  //    refit information
  //SvdOtMatch.refitLeftFound(       iErr);
  //SvdOtMatch.refitRightFound(      iErr);
  //    and update RSEG tables
  //SvdOtMatch.updateARTELeftFound(  iErr);
  //SvdOtMatch.updateARTERightFound( iErr);

  //--------------------------------------------------------------------
  // clean everything
  //--------------------------------------------------------------------
  SvdOtMatch.deleteAll();  
}

