//**********************************************************************/
// Created: 23.10.97,   Olya Igonkina   for   MARPLE package
//
// implementation of saveRelation() functions of RcMatchBuilder class
// for special casses 
// at the moment implemented function for :
//               RSEG_RSEG (RSEG_RBEG) relation
//               RTRA_RSEG     relation
//               RTRA_RCCL     relation
//               RSEG_RCCL     relation
//
//**********************************************************************/

#include "arte/Arte.hh"
#include "arte/Pointer.hh"
#include "arte/ArteMessage.hh"
#include "marple/marple.hh"
#include "marple/RcMatchBuilder.hh"
#include "auto/RSEG.hh"
#include "auto/RTRA.hh"
#include "auto/RCCL.hh"
#include "auto/RSRS.hh"
#include "auto/RSRC.hh"

extern "C" 
{
  void  FORTRAN_ROUTINE(mmbrel)(int &, 
				int *, 
				int *);
}

//-----------------------------------------------------------------
// RcPoint-RcPoint relation  - nothing is done, type of ARTE bank is unknown
//-----------------------------------------------------------------

void RcMatchBuilder< RcPoint, RcPoint>::saveRelation( 
                                                 const RcPoint * ptPoint1,
						 const RcPoint * ptPoint2,
						 int & iErr) const
{
  iErr = 1;
  return ;
}
//-----------------------------------------------------------------
// RSEG-RSEG relation 
//-----------------------------------------------------------------

void RcMatchBuilder< RcSegPoint, RcSegPoint>::saveRelation( 
                                                 const RcSegPoint * ptPoint1,
						 const RcSegPoint * ptPoint2,
						 int & iErr) const
{
  //unsigned int irow1 = ptPoint1->link().index();
  //unsigned int irow2 = ptPoint2->link().index();
  int irow1 = ptPoint1->link().index();
  int irow2 = ptPoint2->link().index();

  if( irow1 >0 && irow1 <= ArtePointer<RSEG>::nrow() &&
      irow2 >0 && irow2 <= ArtePointer<RSEG>::nrow() )
    {
      iErr = 0;
      if( ptPoint1->z() < ptPoint2->z() )
	{
          FORTRAN_ROUTINE(mmbrel)( ArtePointer<RSEG_RBEG>::getLink().getRef(), 
				   &irow1,
				   &irow2);
        }else{
          FORTRAN_ROUTINE(mmbrel)( ArtePointer<RSEG_RBEG>::getLink().getRef(), 
				   &irow2,
				   &irow1);
        }
    }else{
      arteMsg << "MARPLE: unable to save relation RSEG_RBEG" ;
      arteMsg.allp();
      iErr = 1;
    }
  return ;
}

//-----------------------------------------------------------------
// RSEG-RCCL relation 
//-----------------------------------------------------------------

void RcMatchBuilder< RcSegPoint, RcClusPoint>::saveRelation( 
                                                 const RcSegPoint * ptPoint1,
						 const RcClusPoint * ptPoint2,
						 int & iErr) const
{
  int irow1 = ptPoint1->link().index();
  int irow2 = ptPoint2->link().index();

  if( irow1 >0 && irow1 <= ArtePointer<RSEG>::nrow() &&
      irow2 >0 && irow2 <= ArtePointer<RCCL>::nrow() )
    {
      FORTRAN_ROUTINE(mmbrel)( ArtePointer<RSEG_RCCL>::getLink().getRef(), 
			      &irow1,
			      &irow2);
      iErr = 0;
    }else{
      iErr = 1;
      arteMsg << "MARPLE: unable to save relation RSEG_RCCL" ;
      arteMsg.allp();
    }
  return ;
}

void RcMatchBuilder< RcClusPoint, RcSegPoint>::saveRelation( 
                                                 const RcClusPoint * ptPoint1,
						 const RcSegPoint * ptPoint2,
						 int & iErr) const
{
  int irow1 = ptPoint1->link().index();
  int irow2 = ptPoint2->link().index();

  if( irow1 >0 && irow1 <= ArtePointer<RCCL>::nrow() &&
      irow2 >0 && irow2 <= ArtePointer<RSEG>::nrow() )
    {
      FORTRAN_ROUTINE(mmbrel)( ArtePointer<RCLX_RSEG>::getLink().getRef(), 
			      &irow1,
			      &irow2);
      iErr = 0;
    }else{
      iErr = 1;
      arteMsg << "MARPLE: unable to save relation RCLX_RSEG" ;
      arteMsg.allp();
    }
  return ;
}

//-----------------------------------------------------------------
// RTRA-RSEG relation 
//-----------------------------------------------------------------

void RcMatchBuilder< RcTraPoint, RcSegPoint>::saveRelation( 
                                                 const RcTraPoint * ptPoint1,
						 const RcSegPoint * ptPoint2,
						 int & iErr) const
{
  int irow1 = ptPoint1->link().index();
  int irow2 = ptPoint2->link().index();

  if( irow1 >0 && irow1 <= ArtePointer<RTRA>::nrow() &&
      irow2 >0 && irow2 <= ArtePointer<RSEG>::nrow() )
    {
      FORTRAN_ROUTINE(mmbrel)(
			      ArtePointer<RTRX_RSEG>::getLink().getRef(), 
			      &irow1,
			      &irow2);
      iErr = 0;
    }else{
      iErr = 1;
      arteMsg << "MARPLE: unable to save relation RTRX_RSEG" ; 
      arteMsg.allp();
    }
  return ;
}

void RcMatchBuilder< RcSegPoint, RcTraPoint>::saveRelation( 
                                                 const RcSegPoint * ptPoint1,
						 const RcTraPoint * ptPoint2,
						 int & iErr) const
{
  // replace ptPoint1 and ptPoint2
  int irow1 = ptPoint2->link().index();  // row in RTRA table
  int irow2 = ptPoint1->link().index();  // row in RSEG table

  if( irow1 >0 && irow1 <= ArtePointer<RTRA>::nrow() &&
      irow2 >0 && irow2 <= ArtePointer<RSEG>::nrow() )
    {
      FORTRAN_ROUTINE(mmbrel)(
			      ArtePointer<RTRX_RSEG>::getLink().getRef(), 
			      &irow1,
			      &irow2);
      iErr = 0;
    }else{
      iErr = 1;
      arteMsg << "MARPLE: unable to save relation RTRX_RSEG" ; 
      arteMsg.allp();
    }
  return ;
}

//-----------------------------------------------------------------
// RTRA-RCCL relation 
//-----------------------------------------------------------------

void RcMatchBuilder< RcTraPoint, RcClusPoint>::saveRelation( 
                                                 const RcTraPoint * ptPoint1,
						 const RcClusPoint * ptPoint2,
						 int & iErr) const
{
  int irow1 = ptPoint1->link().index();
  int irow2 = ptPoint2->link().index();

  if( irow1 >0 && irow1 <= ArtePointer<RTRA>::nrow() &&
      irow2 >0 && irow2 <= ArtePointer<RCCL>::nrow() )
    {
      FORTRAN_ROUTINE(mmbrel)(
			      ArtePointer<RTRX_RCCL>::getLink().getRef(), 
			      &irow1,
			      &irow2);
      iErr = 0;
    }else{
      arteMsg << "MARPLE: unable to save relation RTRX_RCCL" ; 
      arteMsg.allp();
      iErr = 1;
    }
  return ;
}

void RcMatchBuilder< RcClusPoint, RcTraPoint>::saveRelation( 
                                                 const RcClusPoint * ptPoint1,
						 const RcTraPoint * ptPoint2,
						 int & iErr) const
{
  //replace ptPoint1 and ptPoint2
  int irow1 = ptPoint2->link().index(); // row in RTRA table
  int irow2 = ptPoint1->link().index(); // row in RCCl table

  if( irow1 >0 && irow1 <= ArtePointer<RTRA>::nrow() &&
      irow2 >0 && irow2 <= ArtePointer<RCCL>::nrow() )
    {
      FORTRAN_ROUTINE(mmbrel)(
			      ArtePointer<RTRX_RCCL>::getLink().getRef(), 
			      &irow1,
			      &irow2);
      iErr = 0;
    }else{
      arteMsg << "MARPLE: unable to save relation RTRX_RCCL" ; 
      arteMsg.allp();
      iErr = 1;
    }
  return ;
}

