*-- Author :    Rainer Mankel   21/09/95

      subroutine dmdir()

*     **** create plane navigation table using sector logic
      implicit none
      integer maxnv, maxpl, min_cmp, max_cmp, max_gsector, sector,
     +        lun_nav, layer, jauxpl, b
      parameter (maxnv=3000,      ! max number of navigable planes
     +           maxpl=5000,      ! largest foreseen gede plane #
     +           min_cmp=1,       ! range of navigable
     +           max_cmp=3,       !      detector components
     +           max_gsector=16)  ! max number of navigable gsectors

      parameter (lun_nav=53)      ! navigation table output unit
      parameter (b=65536)         ! truncation base for igede_sec

      integer iplanv(maxnv),      ! plane number (gede) for inv number
     +        method_nv(maxnv),   ! =2 for 0deg-planes, =3 for stereo
     +        invpl(maxpl),       ! inverse translation
     +                            ! (plane number to nv number)
     +        indznv(maxnv),      ! index for z-sort
     +        ipla0nv(maxnv)      ! auxiliary array used for sort
      real    zplanv(maxnv)       ! z values for sort

      integer inv, method,
     +        isl, iipl, ipl, nplanv,
     +        cmp, ii, dm96sec, twin, ntwin, dg
      character*50 cfile
      logical lopened

      integer packed_sector(min_cmp:max_cmp,max_gsector)

            ! The term "sector" as stored in igede_sec refers to a
            ! number unique only within a particular device type. In
            ! this routine we call this a "gsector" (g=gede). Here
            ! we introduce instead a "packed" sector numbering which
            ! is unique everywhere. This is what we call "sector" here.
            ! Translation from "gsector" to "sector" through array
            ! "packed_sector".

#include "rang/zdomain.fi"
#include "rang/zdomdir.fi"
#include "rang/zfpar.fi"
#include "rang/zfout.fi"
#include "arte/Arte.fi"
#include "auto/GEO.fi"
      integer layer_count_nv(maxnv,max_method_dm)
      integer sector_nv(maxnv)

      integer nlayer_matrix(max_method_dm,max_sector_dm,
     +                                              min_sl_dm:max_sl_dm)

*     **** packed numbering of sectors:

        ! device cmp   sec       sector
        !  WCH    2    1-12   ->  1-12
        !  MSG    3    1-8    -> 13-20
        !  SI/M   1    9-16   -> 21-28
        !  SI/V   1    1-4    -> 29-32
        !                5    -> 32
        !                6    -> 31
        !                7    -> 30
        !                8    -> 29

      data (packed_sector(2,ii),ii=1,max_gsector) /
     +   1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12,    ! honeycombs
     +   4*0 /

      data (packed_sector(3,ii),ii=1,max_gsector) /
     +   13,14,15,16,17,18,19,20,                  ! MSGCs
     +   8*0 /

      data (packed_sector(1,ii),ii=1,max_gsector) /
     +   29,30,31,32,32,31,30,29,
     +   21,22,23,24,25,26,27,28/                  ! Si

      save packed_sector

#include "arte/Arte.ff"
#include "auto/GEO.ff"
      
*     **** reset invpl
      do ipl=1,maxpl
       invpl(ipl) = 0
      enddo

*     **** prepare sort of planes
      nplanv = 0

*     **** select planes & sort in z
      do 10 isl=1,iq(jgesl+2)
*      **** check if wirechamber, MSGC or magnet-Si
       cmp = igesl_cmp(isl)
       if (.not.(cmp.eq.1.or.cmp.eq.2.or.cmp.eq.3)) go to 10
       if (cmp.lt.min_cmp.or.cmp.gt.max_cmp) go to 10
*      **** loop over planes in sl
       do iipl=1,igesl_npl(isl)
        if (iipl.eq.1) then
         jauxpl = pgslx_gede(isl)
        else
         jauxpl = xgslx_gede(jauxpl)
        endif
        if (jauxpl.eq.0) stop ' didraw: jauxpl zero '
        ipl = mgslx_gede(jauxpl)
        if (dlevel_zfpar.eq.1) then
           sector = dm96sec(ipl)
        else
           sector = packed_sector(cmp,mod(igede_sec(ipl),b))
        endif

*       **** take allowed sectors only
        if (sector.ge.1.and.sector.le.max_sector_dm) then
         if (ipl.gt.maxpl) then
          print *,' () dmdir: ipl overflow, ipl=', ipl
          stop 'dmdir'
         endif
         if (nplanv.ge.maxnv) stop ' () dmdir: overflow nplanv '
         nplanv = nplanv + 1
         ipla0nv(nplanv) = ipl
         zplanv(nplanv) = 0.5 * (fgede_zmin(ipl) + fgede_zmax(ipl))
        endif
       enddo
 10   continue

*     **** sort according to decreasing z
      call sortzv(zplanv,indznv,nplanv,+1,+1,0)
      if (llog_zfout) print *,' dmdir: #planes sorted:', nplanv,
     +     ' zmin/zmax=',
     +        zplanv(nplanv), zplanv(1)

*     **** fill plane info according to sort index
      do inv=1,nplanv
       ipl = ipla0nv(indznv(inv))
       cmp = igede_cmp(ipl)
       iplanv(inv) = ipl
       zplanv(inv) = 0.5 * (fgede_zmin(ipl) + fgede_zmax(ipl))
                                 ! reuse zplanv for sorted planes
***       sector_nv(inv) = packed_sector(igede_cmp(ipl),igede_sec(ipl))
       if (dlevel_zfpar.eq.1) then
          sector_nv(inv) = dm96sec(ipl)
       else
          sector_nv(inv) = packed_sector(igede_cmp(ipl),
     +         mod(igede_sec(ipl),b))
       endif

       if (abs(fgede_sin(ipl)).lt.1.e-4
     +    .and.abs(fgede_rot(2,ipl)).lt.1.e-4)   then
        method_nv(inv) = 2       ! set stereo type
       else
        method_nv(inv) = 3
       endif

       invpl(iplanv(inv)) = inv  ! set backward translation array
      enddo

*     **** setup layer-count array
      do inv=1,maxnv
       do method=1,max_method_dm
        layer_count_nv(inv,method) = 0
       enddo
      enddo

*     **** reset plane matrix array
      do isl=min_sl_dm,max_sl_dm
       do sector=1,max_sector_dm
        do method=1,max_method_dm
         nlayer_matrix(method,sector,isl) = 0
         do layer=1,max_layer_dm
          plane_matrix_dm(layer,method,sector,isl) = 0
         enddo
        enddo
       enddo
      enddo

*     **** setup plane_matrix_dm
      do 40 inv=nplanv,1,-1         ! loop according to increasing z
       ipl = iplanv(inv)
       cmp = igede_cmp(ipl)
       isl = ogede_gesl(ipl)
       if (isl.lt.min_sl_dm.or.isl.gt.max_sl_dm) go to 40
**       sector = packed_sector(igede_cmp(ipl),igede_sec(ipl))
       if (dlevel_zfpar.eq.1) then
          sector = dm96sec(ipl)
       else
          sector = packed_sector(igede_cmp(ipl),mod(igede_sec(ipl),b))
       endif

       if (sector.lt.1.or.sector.gt.max_sector_dm) go to 40
       method = 1
       if (nlayer_matrix(method,sector,isl).lt.max_layer_dm) then
        nlayer_matrix(method,sector,isl) =
     +                            nlayer_matrix(method,sector,isl) + 1
        plane_matrix_dm(nlayer_matrix(method,sector,isl),
     +                                         method,sector,isl) = ipl
        layer_count_nv(inv,method) = nlayer_matrix(method,sector,isl)
       else
        print *,' () dmdir: layer overflow,method,sector,isl,cmp= ',
     +         method,sector,isl,igede_cmp(ipl)
       endif

       method = method_nv(inv)
       if (nlayer_matrix(method,sector,isl).lt.max_layer_dm) then
        nlayer_matrix(method,sector,isl) =
     +                            nlayer_matrix(method,sector,isl) + 1
        plane_matrix_dm(nlayer_matrix(method,sector,isl),
     +                                         method,sector,isl) = ipl
        layer_count_nv(inv,method) = nlayer_matrix(method,sector,isl)
       else
        print *,' () dmdir: layer overflow '
       endif
 40   continue              ! inv loop

*     **** setup twin directory
      do dg=1,maxgede_dg
         twin_dg(dg) = 0
      enddo

      ntwin = 0

      do isl=min_sl_dm,max_sl_dm
         do sector=1,max_sector_dm
            do layer=1,max_layer_dm
               method = 1
               dg = plane_matrix_dm(layer,method,sector,isl)
               if (dg.gt.maxgede_dg)
     +              print *,' () dmdir: dg=',dg
               do 50 twin=dg-1,dg+1,2
                  if (twin.ge.1.and.twin.le.ngede()) then
                     if ((ogede_gesl(twin).eq.ogede_gesl(dg))
     +                    .and. (igede_lay(twin).eq.igede_lay(dg))
     +                    .and. (mod(igede_sec(twin),b)
     +                           .eq.mod(igede_sec(dg),b))) then
                        ntwin = ntwin + 1
                        twin_dg(dg) = twin
                        go to 51
                     endif
                  endif
 50            continue
 51            continue
            enddo
         enddo
      enddo

      if (llog_zfout) print *,' dmdir: #twins detected:', ntwin



      end



