#include "rich/PhotonDetection.h"

extern int RICHdebug;

// ==========================================================================
//
//  LenseSystem
//


LensSystem::LensSystem(ArtePointer<CLEN> l) {
  id=l->id;
  WaveEff=Table(l->iwav,l->nwav);
  AngEff =Table(l->iang,l->nang);
  }  


LensSystem::LensSystem(ScriptFile& f) {
  char n[256],tag[256];
  id=0;
  while (f.getline(n)) {
    char *s=n;
    sscanf(s,"%s",tag);s+=f.itemlength(s,1);
    int err=0;
    if      (!strcmp(tag,"LensSystem")) continue;  
    else if (!strcmp(tag,"WavelengthEfficiency"))  WaveEff=Table(f);
    else if (!strcmp(tag,"AngularEfficiency")) AngEff=Table(f);
    else if (!strcmp(tag,"type"))       
      err=(sscanf(s,"%d",&id)  !=1);
    else if (!strcmp(tag,"end"))  break;
    else {
      fprintf(stderr,
         "LensSystem(ScriptFile):  Unknown tag: %s %d\n",tag,err);
      exit(0);
      }
    }
    
  CLEN l;
  l.id=id;
  l.iwav=WaveEff.firstpoint();
  l.nwav=WaveEff.numpoints();
  l.iang=AngEff.firstpoint();
  l.nang=AngEff.numpoints();
  ArtePointer<CLEN>::append(l);
  
  }  

//
//  print data
//
void LensSystem::print(FILE* fp) {
  fprintf(fp,"#\n LensSystem\n");
  fprintf(fp,"  type      %d\n",type());
  WaveEff.print(fp,"WavelengthEfficiency");
  AngEff.print(fp,"AngularEfficiency");
  fprintf(fp,"  end\n");
  }


