#ifndef __LOSSES_H__
#define __LOSSES_H__

typedef enum {
  NO_LOSS,
  SMI_LOSS_OUTOFSPHERE,
  SMI_LOSS_REFLECTIVITY,
  SMI_LOSS_MIRRORGAPS,
  SMI_LOSS_OUTOFMIRROR,
  SMI_LOSS_PBEAMCUTOUT,
  SMI_LOSS_EBEAMCUTOUT,
  PMI_LOSS_MIRRORGAPS,
  PMI_LOSS_REFLECTIVITY,
  PMI_LOSS_OUTOFMIRROR,
  PMI_LOSS_EBEAMCUTOUT,
  PDW_LOSS_OUTOFBOUNDS,
  DIM_LOSS_REFLEXIONS,
  DIM_LOSS_RAYLEIGH,
  DIM_LOSS_ABSORPTION,
  VES_LOSS_PBEAMSHROUD,
  VES_LOSS_EBEAMSHROUD,
  VES_LOSS_PMI_BACKSIDE,
  SUP_LOSS_OUTOFAREA,
  SUP_LOSS_CELLEMPTY,
  LEN_LOSS_WAVELENGTH,
  LEN_LOSS_ANGULAR,
  PMT_LOSS_EFFICIENCY,
  CEL_LOSS_OUTOFPMTAREA
  } RLosses;
  
  
inline double lossfrac(int loss,int *norm) {
  double ret=double(loss)/double(*norm);
  *norm-=loss;
  return ret;
  }

inline void printloss(int *number,int norm=1,FILE* fp=stdout) {
  int normold=norm;
  int norm0=norm;
  fprintf(fp,"----------------------------------------------------------------------\n");
  fprintf(fp,"Spherical Mirror .....\n");
  fprintf(fp,"  photon not inside mirror .....: %6d %e\n",norm,lossfrac(number[SMI_LOSS_OUTOFSPHERE],&norm));
  fprintf(fp,"  reflexion losses .............: %6d %e\n",norm,lossfrac(number[SMI_LOSS_REFLECTIVITY],&norm));
  fprintf(fp,"  mirror gaps ..................: %6d %e\n",norm,lossfrac(number[SMI_LOSS_MIRRORGAPS],&norm));
  fprintf(fp,"  outside of mirror bounds .....: %6d %e\n",norm,lossfrac(number[SMI_LOSS_OUTOFMIRROR],&norm));
  fprintf(fp,"  p-beam cut out ...............: %6d %e\n",norm,lossfrac(number[SMI_LOSS_PBEAMCUTOUT],&norm));
  fprintf(fp,"  e-beam cut out ...............: %6d %e\n",norm,lossfrac(number[SMI_LOSS_EBEAMCUTOUT],&norm));
  fprintf(fp,"  total ........................: %6d %e\n\n",norm,1.0-double(norm)/double(normold)); 
  normold=norm;
  fprintf(fp,"----------------------------------------------------------------------\n");
  fprintf(fp,"Plane Mirror .....\n");
  fprintf(fp,"  reflexion losses .............: %6d %e\n",norm,lossfrac(number[PMI_LOSS_REFLECTIVITY],&norm));
  fprintf(fp,"  mirror gaps ..................: %6d %e\n",norm,lossfrac(number[PMI_LOSS_MIRRORGAPS],&norm));
  fprintf(fp,"  outside of mirror bounds .....: %6d %e\n",norm,lossfrac(number[PMI_LOSS_OUTOFMIRROR],&norm));
  fprintf(fp,"  e-beam cut out ...............: %6d %e\n",norm,lossfrac(number[PMI_LOSS_EBEAMCUTOUT],&norm));
  fprintf(fp,"  total ........................: %6d %e\n\n",norm,1.0-double(norm)/double(normold)); 
  normold=norm;
  fprintf(fp,"----------------------------------------------------------------------\n");
  fprintf(fp,"Photon Detector Entry Window .....\n");
  fprintf(fp,"  outside of boundaries ........: %6d %e\n",norm,lossfrac(number[PDW_LOSS_OUTOFBOUNDS],&norm));
  fprintf(fp,"  total ........................: %6d %e\n\n",norm,1.0-double(norm)/double(normold)); 
  normold=norm;
  fprintf(fp,"----------------------------------------------------------------------\n");
  fprintf(fp,"Dielectrics .....\n");
  fprintf(fp,"  surface reflexions ...........: %6d %e\n",norm,lossfrac(number[DIM_LOSS_REFLEXIONS],&norm));
  fprintf(fp,"  Rayleigh scattering ..........: %6d %e\n",norm,lossfrac(number[DIM_LOSS_RAYLEIGH],&norm));
  fprintf(fp,"  absorption ...................: %6d %e\n",norm,lossfrac(number[DIM_LOSS_ABSORPTION],&norm));
  fprintf(fp,"  total ........................: %6d %e\n\n",norm,1.0-double(norm)/double(normold)); 
  normold=norm;
  fprintf(fp,"----------------------------------------------------------------------\n");
  fprintf(fp,"Vessel .....\n");
  fprintf(fp,"  p-beam shroud ................: %6d %e\n",norm,lossfrac(number[VES_LOSS_PBEAMSHROUD],&norm));
  fprintf(fp,"  e-beam shroud ................: %6d %e\n",norm,lossfrac(number[VES_LOSS_EBEAMSHROUD],&norm));
  fprintf(fp,"  plane mirror backside ........: %6d %e\n",norm,lossfrac(number[VES_LOSS_PMI_BACKSIDE],&norm));
  fprintf(fp,"  total ........................: %6d %e\n\n",norm,1.0-double(norm)/double(normold)); 
  normold=norm;
  fprintf(fp,"----------------------------------------------------------------------\n");
  fprintf(fp,"  all losses vessel ............: %6d %e\n\n",norm,1.0-double(norm)/double(norm0));
  fprintf(fp,"----------------------------------------------------------------------\n");
  fprintf(fp,"Detector .....\n");
  fprintf(fp,"  out of supermodule coverage ..: %6d %e\n",norm,lossfrac(number[SUP_LOSS_OUTOFAREA],&norm));
  fprintf(fp,"  cell empty ...................: %6d %e\n",norm,lossfrac(number[SUP_LOSS_CELLEMPTY],&norm));
  fprintf(fp,"  lens efficiency wavelength ...: %6d %e\n",norm,lossfrac(number[LEN_LOSS_WAVELENGTH],&norm));
  fprintf(fp,"  lens angular efficiency ......: %6d %e\n",norm,lossfrac(number[LEN_LOSS_ANGULAR],&norm));
  fprintf(fp,"  pm efficiency ................: %6d %e\n",norm,lossfrac(number[PMT_LOSS_EFFICIENCY],&norm));
  fprintf(fp,"  out pmt area .................: %6d %e\n",norm,lossfrac(number[CEL_LOSS_OUTOFPMTAREA],&norm));
  fprintf(fp,"  total ........................: %6d %e\n\n",norm,1.0-double(norm)/double(normold)); 
  normold=norm;
  fprintf(fp,"----------------------------------------------------------------------\n");
  fprintf(fp,"  all losses ...................: %6d %e\n\n",normold,1.0-double(norm)/double(norm0));

  }
#endif
