#ifndef __PHOTONDETECTORCELL_H__
#define __PHOTONDETECTORCELL_H__


// -------------------------------------------------------------------------
/// description of a cell of a supermodule 
/**
  The set up in one cell of a supermodule is describe using the 
  class PhotonDetectorCell. The describtion of the lense system,
  which is at the moment assumed to be the same for all cells,
  is done in the "dodetect" member function. Up to now the implementation
  of the lense system is only a 2:1 mapping of the x,y-coordinates.
  Work has to be done here.
 */
 
class PhotonDetectorCell {
  PM *ppm;
  int *id;
  inline int ID(int channel) const { return id[channel]; }
  
public:
/// detection of a photon, which hits this cell of the supermodule. See also page:  
  int dodetect(const Photon& p,int *ChannelID);
/**
  The function "dodetect" describes the imaging of the lens system 
  as well as losses, which occur during the transmission. 
  After doing this it is checked, if the pmt detects the photon.
  The return value indicated, if the photon is detected (return value=1)
  or not (return value=0). In the first case the channelid is return
  with the last argument.
 */  
/// best guess position on the front of the lense-system / supermodule
  Vec3 pos(int channel,double costhein=0,double phiin=0);

/// return a reference to the pmt plugged into this cell
  inline PM& pm()            const { return *ppm; }
/// short cut for pm().SuperModule() (see class PM for an explanation)
  inline int SuperModule()   const { return ppm->SuperModule(); }
/// short cut for pm().Row() (see class PM for an explanation)
  inline int Row()           const { return ppm->Row(); }
/// short cut for pm().Col() (see class PM for an explanation)
  inline int Col()           const { return ppm->Col(); }

// constructor have to be supplied with the information about the pmt in the cell
  PhotonDetectorCell(PM* pm);
// destructor to clean up private member functions.
  ~PhotonDetectorCell();
  };


#endif
