#ifndef __PLANEMIRROR_H__
#define __PLANEMIRROR_H__

#include "rich/SinglePlaneMirror.h"

// ------------------------------------------------------------------------
/// description of the planar mirror
/**
  At the moment the mirror consist out of 6 flat SinglePlaneMirror's
  for the upper half, and six for the lower half.
  */
class PlaneMirror {

protected:
int       nnumcol;
int       nummirror;
int       speed;
SinglePlaneMirror **mirrors;
SinglePlaneMirror fullupper,fulllower;

double    radiuselectronhole;
Table refl;

inline PlaneMirror(){;}

int index(int row,int col) {
  return row*nnumcol+col;
  }
  
public:
/// returns the radius of the hole around the electron beam shroud
  inline double  RadiusElectronHole() const { return radiuselectronhole; }
/// sets the radius of the hole around the electron beam shroud
  inline double& RadiusElectronHole()       { return radiuselectronhole; }
/// returns the number of rows
  inline int numrow() const {  return nummirror/nnumcol; }
/// returns the number of columns
  inline int numcol() const {  return nnumcol; }
/// returns the SinglePlaneMirror at position (row,column)
  inline SinglePlaneMirror* SingleMirror(int row,int col) {
    int k=index(row,col);
    if (k<0||k>=nummirror) return 0;
    return mirrors[k];
    }
/// return the reflectivity curve (at the moment common for a single mirrors)
  inline const Table&  reflectivity() const   { return refl; }
  inline       Table&  reflectivity()         { return refl; }
  inline double        reflectivity(double l) { return refl.f(l); }
//@Man: reflection
//@{
  int reflectonly(Photon* pp);
/// reflexion of a photon. Return 1, if there are no losses.
/**
  Reflexion of a photon. If no losses happen the photon
  position is changed to the intersection point on the mirror
  and the flight direction is turned to the reflected one.
  In this case the function returns 1. If losses occure the
  function returns 0 and one can analyse the reason using
  the functions ReflexionLoss and GeometricalLoss.
  */
  int reflect(Photon* pp);
/// switch to full simulation in function reflect (default)
  inline void FullSimulation() { speed=0; }
/// switch to fast simulation in function reflect : no subdivision of the mirrors
  inline void FastSimulation() { speed=1; }
/// calculation intersection point with full mirror
  int fullmirror_intersection(const StraightLine& pp,double *path);
/// reflection on full mirror without reflexion losses
  int fullmirror_reflectonly(Photon* pp);
/// reflection on full mirror with reflexion losses
  int fullmirror_reflect(Photon* pp);
/// reflexion especially for tracing the track direction to the detector      
  inline int upper_reflect(StraightLine* t) {return fullupper.reflect(t); }
/// reflexion especially for tracing the track direction to the detector
  inline int lower_reflect(StraightLine* t) { return fulllower.reflect(t); }
/// tracing taking only the geometrical losses into account
  int  geometry_trace(Photon* pp);
//@}

/// print the contents of the class to an ASCII file.
  void print(FILE* fp);
/// constructor reading from an ASCII-FILE produced using 'print'.  
  PlaneMirror(ScriptFile& f);
/// constructor using a record in the ARTE--Table CPMI
  PlaneMirror(ArtePointer<CPMI>);
/// destructor
  ~PlaneMirror();
  };

#endif
