
      SUBROUTINE NPHOTON(XR,PR,MASS,NPHOT)
************************************************************************
*
* Created: October 1997, by Ignacio Arinyo.
*
* Description: Routine to compute the expected number of photons for a 
*             given mass hypothesis.
*
* Called from: ricrec.F
*
* Arguments:
*  IN   XR(3)   Reconstructed entry point to the RICH.
*  IN   PR(3)   Reconstructed momentum ant entry point.
*  IN   MASS    Mass hypothesis.
*  OUT  NPHOT   Expected number of photons.
*
*  External calls: to function geomeff in f77reco.C
*
************************************************************************




      implicit none


#include "arte/Arte.fi"
#include "auto/EV.fi"
#include "auto/CON.fi"
#include "auto/GEO.fi"

#include "rire/ricpar.fi"

      REAL*4 XR(3),PR(3),P0(3),MASS,ngamma
      REAL*4 P2,P,E2,E,CTHETA,RC(3),A,B,PATH,DRH(3)
      REAL*4 NMED,EGAMMA,PHI,NREFR,emin,emax,emean
      integer numiter

      parameter(emin=1.8,emax=6.7,emean=3.4,numiter=20)

      real*4 beta,ephot,r0(3),rndm
      integer i,n,itra,iv,nphot
      real geomeffi
      real geomeff
      external geomeff

#include "arte/Arte.ff"
#include "auto/EV.ff"
#include "auto/CON.ff"
#include "auto/GEO.ff"


C   APROXIMATE REFRACTIVE INDEX

      NREFR(EPHOT) = 1.0013178 + 1.72422E-7 * EPHOT +
     +                           5.49512E-6 * EPHOT**2


C RC   CENTER OF MIRROW (X=0)
      RC(1) = 0.0
      RC(2) = 185.
      RC(3) = ZRICHMAX - SQRT(RADIUS**2-RC(2)**2-RC(1)**2)

      P2   = PR(1)**2+PR(2)**2+PR(3)**2
      P    = SQRT(P2)
      E2   = P2 + MASS**2
      BETA = MIN(1.,SQRT(P2/E2))


      DO I=1,3
         P0(I) = PR(I)/P
      ENDDO

C PATH LENGTH

      DO I=1,3
         DRH(I) = XR(I) - RC(I)
      ENDDO
      B    = DRH(1)*P0(1)+DRH(2)*P0(2)+DRH(3)*P0(3)
      PATH = - B + SQRT(B**2+RADIUS**2-DRH(1)**2-DRH(2)**2-DRH(3)**2)
      A    = MAX(PATH,0.0) 
      
      DRH(2) = XR(2) + RC(2)
      B    = DRH(1)*P0(1)+DRH(2)*P0(2)+DRH(3)*P0(3)
      PATH = - B + SQRT(B**2+RADIUS**2-DRH(1)**2-DRH(2)**2-DRH(3)**2)
      B    = MAX(PATH,0.0)
      PATH = MAX(A,B)

	
      nmed=nrefr(emean)

      CTHETA = 1./(NMED*BETA)


      IF (CTHETA.LT.1.0) then

         A      = SIN(ACOS(1./NMED))
         NGAMMA = PATH / LRAD * PHOTON * (SIN(ACOS(CTHETA))/A)**2

         geomeffi=geomeff(r0,pr,CTHETA, numiter)

      ELSE
         geomeffi=0
      ENDIF
      nphot=ngamma*geomeffi 


      END








