//-----------------------------------------------------------------------//
//
//  RingSearchARTE: Program for stand alone RICH reconstruction
//
//  Author: Reinhard Eckmann
//
//  Created: May-98
//
//  All Files: RingSearchARTE.C, RingSearchARTE.h, ring.C, ring.h,
//             phila.C,  phila.h    
//
//-----------------------------------------------------------------------//

#include "rise/RingSearchARTE.h"
#include "rich/PhotonDetection.h"

#include "auto/GEPT.hh"
#include "auto/DIGB.hh"
#include "auto/MIMP.hh"
#include "auto/MIMX.hh"
#include "auto/MIDI.hh"
#include "auto/HIDI.hh"
#include "auto/MIHI.hh"
#include "auto/MTRA.hh"

#include "arte/ArteAll.hh"
#include "arte/RSEG_cmp.hh"

RingSearchARTE::RingSearchARTE(char *path) : phila(path) {
;
  }
  
void RingSearchARTE::makeHitMap() {
  RICHDetector& pd=*RICH_Detector;
  h.reset();

  unsigned gede;
  for (gede=1;gede<=ArtePointer<GEDE>::nrow();gede++) 
    if (ArtePointer<GEDE>(gede)->cmp==5) break;
  ArtePointer<GEPT> gept(gede);
  int n0=gept->dig1;
  int n1=gept->digl+1;

  int i;
  for (i=n0;i<n1;i++) {
    ArtePointer<DIGB> digi(i);
    int id=digi->id;
    int s=smFromChannelID(id);
    int r=rowFromChannelID(id);
    int c=colFromChannelID(id);
    int p=pmchanFromChannelID(id);
    if (RICH_Detector->ChannelExist(s,r,c,p)) {
      double phi,lambda;
      phila.Daqid2PhiLambda(s,r,c,p,pd.pm().numchannel(),&phi,&lambda);  
       
// approx. correction due to magnetic field bending 
      //phi+=.0155*phi;
      
      ArteRelationIterator<DIGB_MIMP> imimp(digi->mimp);
      ArtePointer<MIMP> mimp(imimp);
      ArtePointer<MTRA>imtra(mimp->mtra);     
      h.append(d.dualx(phi*1000),d.dualy(lambda*1000),imtra.index());
      }
    }  
  h.randomize(); 
  d.nextevent();
  }
  

void RingSearchARTE::getrings() {
  makeHitMap();
  for (float radius=52;radius>40;radius-=1) {// mrad /////
    d.define(radius,5);
    d.ringsearch(h);
    }
  FillRSEG();  
  }

void RingSearchARTE::getrings(double radius) {
  makeHitMap();
  d.define(radius,5);
  d.ringsearch(h);
  FillRSEG();  
}


extern "C" {
  extern F77NAME(mmbrel)(int*,int*,int*);
  inline void mmbrel(int* rel, int row1, int row2) {
    F77NAME(mmbrel)(rel,&row1,&row2);
    }
}



void RingSearchARTE::FillRSEG() {

  RSEG r;
  ArtePointer<RSEG>rseg;
  cout<<"RingSearchARTE::FillRSEG() numrings="<<numrings()<<endl;
  for (int i=0;i<numrings();i++) {
    Ring ring=getRing(i);

    r.xf=ring.phi();
    r.yf=ring.lambda();
    r.time=ring.thetaC();

    float beta=(1/1.00138/cos(ring.thetaC()));
    if (beta>1) beta=1;
    r.dtim=beta;

    r.txf=tx(i);
    r.tyf=ty(i);

    r.chi=ring.chi2*1e-2;

    r.cvf[ 0]=0;
    r.cvf[ 1]=0;
    r.cvf[ 2]=0;
    r.cvf[ 5]=ring.covariance(0,0);
    r.cvf[ 8]=ring.covariance(0,1);
    r.cvf[12]=ring.covariance(0,2);
    r.cvf[ 9]=ring.covariance(1,1);
    r.cvf[13]=ring.covariance(1,2);
    r.cvf[14]=ring.covariance(2,2);
    r.cvf[ 3]=0;
    r.cvf[ 6]=0;
    r.cvf[10]=0;
    r.cvf[ 4]=0;
    r.cvf[ 7]=0;
    r.cvf[11]=0;

    r.cmp = Rsegc::rich;

    r.fit = imtra(i);

    rseg=ArtePointer<RSEG>::append(r);

    if (imtra(i)!=0)
      mmbrel(&ArtePointer<RSEG_MTRA>::getLink().getRef(),
	     rseg.index(),imtra(i));
  }
}

RingSearchARTE *RICH_Search=0;



void rcrings_() {
  if (!RICH_Search) {
    char dir[256];
    char* RICHconfigdir=getenv("RICH_CONFIGDIR");
    if (RICHconfigdir) {
      strcpy(dir,RICHconfigdir);
      if (dir[strlen(dir)-1]!='/') strcat(dir,"/");
      }
    else {
      printf("Set environement variable RICH_CONFIGDIR before starting the
      program\n");
      exit(1);
      }
    RICH_Search=new RingSearchARTE(dir);
    }
  RICH_Search->getrings();
}
