*CMZ :  1.00/01 27/06/96  16.05.32  by   Ivan Belyaev
*CMZ :  1.04/11 18/12/95  14.55.48  by  joerg f. rieling
*CMZ :  1.04/10 22/11/95  12.36.10  by  joerg f. rieling
*CMZ :  1.04/08 07/11/95  10.03.40  by  joerg f. rieling
*CMZ :  1.04/06 29/06/95  12.40.57  by  joerg f. rieling
*CMZ :  1.04/03 22/05/95  15.51.11  by  joerg f. rieling
*-- Author :

c     ------------------------------------------------------------------

      subroutine sgethit3 (x_ex, pl, lastpl, nhtt, htt, mhtt)

c     -- returns the at most N_MAXHTT = 3 hits that lie in the
c     -- tolerance tol.
c     -- Tolerance is calculated from the distance to the last
c     -- plane that measures the respective coordinate.
c     -- THIS MEANS that sgethit3 is no longer independent of
c     -- the succesion of measurement planes in the detector but provides
c     -- alternating x/y/ measurememts.
c     -- If there should be more hits than N_MAXHTT, then the closest
c     -- N_MAXHTT are returned.
c
c     -- input:  x_ex : 2-dim vector in x/y plane; defines
c     --                intersection of track with plane
c     --         pl   : number of hit plane (Si nomenclature)
c     --  normal_step : logical which is T if normal Kalman step,
c     --                F if track initialization
c     -- output: nhtt : number of hits in tolerance
c     --         htt  : array(N_MAXHTT) with hit pointers
c     --         mhtt :   "  (    "   ) with hit coo.
c
c     >>> bad (but fast) style: sgethit assumes that N_MXHTT is 3. <<<
c     >>>        sgethit assumes that z is less than z_last        <<<
c     >>>  sgethit assumes calling routine inits at igede_lay > 1  <<<
c     ------------------------------------------------------------------

      implicit none

c     --- constants

#include "arte/Arte.fi"
#include "auto/GEO.fi"
#include "auto/SI.fi"
#include "auto/HITB.fi"

#include "susi/zsiconst.fi"

#include "susi/zsigeo.fi"

c     --- formal parameters
      real    x_ex(2), mhtt(N_MAXHTT)
      integer pl, lastpl, nhtt, htt(N_MAXHTT)
c     --- misc
      integer i, ppl, worst , FH, LH 
      real    lcoo, co, si
      real    meas, diff, tol
      real    dz

      INTEGER  LENOCC 
      EXTERNAL LENOCC 

c     --- common blocks

#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/SI.ff"
#include "auto/HITB.ff"
#include "susi/zsigeo.ff"

c     --- init
      nhtt = 0
      ppl  = plid (pl)
      co   = plcsr(pl)
      si   = plsnr(pl)
      lcoo = x_ex(1)*co - x_ex(2)*si ! local coordinate
      dz   = plz(lastpl) - plz(pl)

c     --- calculate tolerance using dz
      IF( JSPAR .LE. 0)CALL SUSIDEFV 

      tol = MAX( FSPAR_MNTO(NSPAR()),(dz * FSPAR_ANTO(NSPAR())))
      tol = MIN( TOL, FSPAR_MXTO(NSPAR()) )

c     --- find the N_MAXHTT best hits in tolerance tol
      FH = IGEPT_HIT1( ppl ) 
      IF( FH .LE. 0 )RETURN 
      LH = IGEPT_HITL( ppl )
      IF( LH .LE. 0 )RETURN  
      do i = FH , LH ! I.B.
         meas = fhitb_x(i)
         diff = abs(meas - lcoo)
         if (diff .le. tol) then
            if (nhtt.eq.N_MAXHTT) then  ! if already full...
               if ((abs(mhtt(1)-lcoo)) .gt.
     &             (abs(mhtt(2)-lcoo))) then ! get worst element so far
                  worst = 1
               else
                  worst = 2
               end if
               if ((abs(mhtt(worst)-lcoo)) .lt.
     &             (abs(mhtt(3)    -lcoo))) worst = 3

               if (abs(mhtt(worst)-lcoo).gt.diff) then ! ... perhaps replace.
                  mhtt(worst) = meas
                  htt (worst) = i
               end if
            else
               nhtt       = nhtt + 1
               mhtt(nhtt) = meas
               htt (nhtt) = i
            end if
         end if
      end do ! i

      return
      end
